/**
 * 系统上下文环境基类
 */
qx.Class.define("xlsys.base.XlsysContext", 
{
	type : "abstract",
	extend : qx.core.Object,
	
	construct: function()
	{
		if(arguments.length==1) this.parent = arguments[0];
		this.defaultMap = new xlsys.java.util.HashMap();
	},
	
	members :
	{
		parent : null,
		defaultMap : null,
		sourceContext : null,
		
		clear : function()
		{
			this.defaultMap.clear();
			this.doClear();
		},

		getSourceContext : function()
		{
			return this.sourceContext;
		},

		setSourceContext : function(sourceContext)
		{
			this.sourceContext = sourceContext;
		},

		doClear : function() {},
		
		containsKey : function(key)
		{
			var contains = false;
			contains = this.doContainsKey(key);
			if(!contains)
			{
				contains = this.defaultMap.containsKey(key);
				if(!contains&&this.parent!=null) contains = this.parent.containsKey(key);
			}
			return contains;
		},
		
		doContainsKey : function(key) {},

		containsValue : function(value)
		{
			var contains = false;
			contains = this.doContainsValue(value);
			if(!contains)
			{
				contains = this.defaultMap.containsValue(value);
				if(!contains&&this.parent!=null) contains = this.parent.containsValue(value);
			}
			return contains;
		},
		
		doContainsValue : function(value) {},

		entrySet : function()
		{
			var set = this.defaultMap.entrySet();
			var subSet = this.doEntrySet();
			if(subSet!=null) set.addAll(subSet);
			return set;
		},
		
		doEntrySet : function() {},
		
		allEntrySet : function()
		{
			var set = this.entrySet();
			if(this.parent!=null) set.addAll(this.parent.allEntrySet());
			return set;
		},

		get : function(key)
		{
			var value = this.doGet(key);
			if(value==null) value = this.defaultMap.get(key);
			if(value==null&&this.parent!=null) value = this.parent.get(key);
			return value;
		},
		
		doGet : function(key) {},

		isEmpty : function()
		{
			var empty = this.doIsEmpty();
			if(empty) empty = this.defaultMap.isEmpty();
			return empty;
		},
		
		doIsEmpty : function() {},

		isAllEmpty : function()
		{
			var empty = this.isEmpty();
			if(empty&&this.parent!=null) empty = this.parent.isAllEmpty();
			return empty;
		},
		
		keySet : function()
		{
			var set = new xlsys.java.util.HashSet();
			set.addAll(this.defaultMap.keySet());
			var subSet = this.doKeySet();
			if(subSet!=null) set.addAll(subSet);
			return set;
		},
		
		doKeySet : function() {},
		
		allKeySet : function()
		{
			var set = new xlsys.java.util.HashSet();
			set.addAll(this.keySet());
			if(this.parent!=null) set.addAll(this.parent.allKeySet());
			return set;
		},

		put : function(key, value)
		{
			if(!this.doPut(key, value)) this.defaultMap.put(key, value);
			return true;
		},
		
		doPut : function(key, value) {},

		putAll : function(map)
		{
			if(!this.doPutAll(map)) this.defaultMap.putAll(map);
			return true;
		},
		
		doPutAll : function(map) {},

		remove : function(key)
		{
			var obj = null;
			if(this.doRemove(key)==null) obj = this.defaultMap.remove(key);
			return obj;
		},
		
		doRemove : function(key) {},

		size : function()
		{
			var s = this.defaultMap.size();
			s += this.doSize();
			return s;
		},
		
		doSize : function() {},
		
		allSize : function()
		{
			var s = this.size();
			if(this.parent!=null) s += this.parent.allSize();
			return s;
		},
		
		values : function()
		{
			var values = this.defaultMap.values();
			var subValues = this.doValues();
			if(subValues!=null) values.addAll(subValues);
			return values;
		},
		
		doValues : function() {},
		
		allValues : function()
		{
			var values = this.values();
			if(this.parent!=null) values.addAll(this.parent.allValues());
			return values;
		},
		
		getDefaultMap : function()
		{
			return this.defaultMap;
		},
		
		getParent : function()
		{
			return this.parent;
		}
	}
});