/**
 * 缓存管理类，要使用此类管理里的缓存类必须实现XlsysBuffer接口
 */
qx.Class.define("xlsys.base.buffer.BufferManager",
{
	type : "singleton",
	extend : qx.core.Object,
	
	construct: function()
	{
		this.bufferMap = new xlsys.java.util.HashMap();
	},
	
	members : 
	{
		bufferMap : null,
		
		registerBuffer : function(bufferName, buffer)
		{
			if(bufferName!=null&&buffer!=null) this.bufferMap.put(bufferName, buffer);
		},
		
		cancelRegisterBuffer : function(bufferName)
		{
			if(bufferName!=null) this.bufferMap.remove(bufferName);
		},
		
		isRegistered : function(bufferName)
		{
			return this.bufferMap.containsKey(bufferName);
		},
		
		reloadBuffer : function(bufferName, paramMap, forceLoad, callback)
		{
			if(forceLoad==null) forceLoad = false;
			if(bufferName==null)
			{
				if(callback) callback();
				return;
			}
			var buffer = this.bufferMap.get(bufferName);
			if(buffer==null)
			{
				if(callback) callback();
				return;
			}
			if(paramMap!=null) buffer.reloadData(bufferName, paramMap, forceLoad, callback);
			else buffer.reloadAllData(bufferName, forceLoad, callback);
		},
		
		reloadAllBuffer : function(forceLoad, callback)
		{
			if(forceLoad==null) forceLoad = false;
			var keyArray = this.bufferMap.keys();
			if(callback)
			{
				var counter = [0];
				if(keyArray.length>0)
				{
					for(var i=0;i<keyArray.length;++i)
					{
						this.reloadBuffer(keyArray[i], null, forceLoad, qx.Bootstrap.bind(this.__reloadAllBuffer1, this, callback, keyArray, counter));
					}
				}
				else callback();
			}
			else
			{
				for(var i=0;i<keyArray.length;++i)
				{
					this.reloadBuffer(keyArray[i], null, forceLoad);
				}
			}
		},
		
		__reloadAllBuffer1 : function(callback, keyArray, counter)
		{
			++counter[0];
			if(counter[0]==keyArray.length) callback();
		},
		
		getAllRegisteredBufferNames : function()
		{
			return this.bufferMap.keys();
		},
		
		updateBufferVersion : function(bufferName, callback)
		{
			var buffer = this.bufferMap.get(bufferName);
			if(buffer==null)
			{
				if(callback) callback(-1);
				return -1;
			}
			return buffer.updateCurrentVersion(bufferName, callback);
		},
		
		getBufferVersion : function(bufferName)
		{
			var buffer = this.bufferMap.get(bufferName);
			if(buffer==null) return -1;
			return buffer.getCurrentVersion(bufferName);
		},
		
		getLocalBufferVersion : function(bufferName)
		{
			var buffer = this.bufferMap.get(bufferName);
			if(buffer==null) return -1;
			return buffer.getLocalStorageVersion(bufferName);
		}
	}
});