/**
 * 模型缓冲抽象类
 */
qx.Class.define("xlsys.base.buffer.ModelBuffer", 
{
	type : "abstract",
	extend : xlsys.base.buffer.AbstractBuffer,

	statics : 
	{
		BUFFER_KEY_MODEL_KEY : "_BUFFER_KEY_MODEL_KEY",
		BUFFER_KEY_DATABASE : "_BUFFER_KEY_DATABASE",
		BUFFER_KEY_SESSION : "_BUFFER_KEY_SESSION",
		BUFFER_KEY_ENV_ID : "_BUFFER_KEY_ENV_ID"
	},
	
	construct : function()
	{
		this.base(arguments);
		this.bindModelCreatorMap = new xlsys.java.util.HashMap();
		this.modelCountMap = new xlsys.java.util.HashMap();
		this.modelTransMap = new xlsys.java.util.HashMap();
	},
	
	members : 
	{
		bindModelCreatorMap : null,
		modelCountMap : null,
		// <envId, <bufferName, <model, <language, transModel>>>>
		modelTransMap : null,

		initAllModels : function(bufferName, dataBase, modelCreator, callback)
		{
			if(modelCreator==null)
			{
				modelCreator = this.bindModelCreatorMap.get(bufferName);
				if(modelCreator==null) throw new Error("Please bind model creator first!");
			}
			var envId = dataBase.getEnvId();
			var modelMap = this.getBufferMap(envId, bufferName);
			if(callback) this.getModelCount(bufferName, dataBase, qx.Bootstrap.bind(this.__initAllModels1, this, bufferName, dataBase, modelCreator, callback, modelMap));
			else
			{
				var allCount = this.getModelCount(bufferName, dataBase);
				if(allCount==-1||modelMap.size()<allCount)
				{
					modelMap.clear();
					modelMap.putAll(modelCreator.createAllModels(dataBase));
				}
			}
		},
		
		__initAllModels1 : function(bufferName, dataBase, modelCreator, callback, modelMap, allCount)
		{
			if(allCount==-1||modelMap.size()<allCount)
			{
				modelMap.clear();
				modelCreator.createAllModels(dataBase, qx.Bootstrap.bind(this.__initAllModels2, this, modelMap, callback));
			}
			else callback();
		},
		
		__initAllModels2 : function(modelMap, callback, allModels)
		{
			modelMap.putAll(allModels);
			callback();
		},
		
		getModelCount : function(bufferName, sed, callback)
		{
			var envId = null;
			var dataBase = null;
			if(qx.lang.Type.isNumber(sed)) envId = sed;
			else if(sed instanceof xlsys.base.session.Session) envId = sed.getAttribute(xlsys.base.XLSYS.SESSION_ENV_ID);
			else if(qx.Class.implementsInterface(sed, xlsys.base.database.IDataBase))
			{
				dataBase = sed;
				envId = dataBase.getEnvId();
			}
			var countMap = this.modelCountMap.get(envId);
			if(countMap==null)
			{
				countMap = new xlsys.java.util.HashMap();
				this.modelCountMap.put(envId, countMap);
			}
			var allCount = countMap.get(bufferName);
			if(allCount==null)
			{
				var modelCreator = this.bindModelCreatorMap.get(bufferName);
				if(modelCreator==null) throw new Error("Please bind model creator first!");
				var needClose = false;
				if(dataBase==null)
				{
					dataBase = xlsys.base.database.ClientDataBase.getInstance();
					needClose = true;
				}
				if(callback) modelCreator.getAllCount(dataBase, qx.Bootstrap.bind(this.__getModelCount1, this, bufferName, dataBase, needClose, countMap, callback));
				else
				{
					allCount = modelCreator.getAllCount(dataBase);
					if(needClose) dataBase.close();
					countMap.put(bufferName, allCount);
					return allCount;
				}
			}
			else
			{
				if(callback) callback(allCount);
				else return allCount;
			}
		},
		
		__getModelCount1 : function(bufferName, dataBase, needClose, countMap, callback, allCount)
		{
			if(needClose) dataBase.close();
			countMap.put(bufferName, allCount);
			callback(allCount);
		},
				
		bindModelCreator : function(bufferName, modelCreator)
		{
			this.bindModelCreatorMap.put(bufferName, modelCreator);
		},
		
		/**
		 * 支持以下方法:
		 * getAllModels(bufferName, session, callback);
		 * getAllModels(bufferName, envId, callback);
		 * getAllModels(bufferName, dataBase, callback);
		 */
		getAllModels : function(bufferName, sed, callback)
		{
			var paramMap = new xlsys.java.util.HashMap();
			if(qx.lang.Type.isNumber(sed)) paramMap.put(this.self(arguments).BUFFER_KEY_ENV_ID, sed);
			else if(sed instanceof xlsys.base.session.Session) paramMap.put(this.self(arguments).BUFFER_KEY_SESSION, sed);
			else if(qx.Class.implementsInterface(sed, xlsys.base.database.IDataBase)) paramMap.put(this.self(arguments).BUFFER_KEY_DATABASE, sed);
			if(callback) this.getBufferData(bufferName, paramMap, callback);
			else return this.getBufferData(bufferName, paramMap);
		},
		
		_getAllTranslatedModels : function(srcModelList, envId, bufferName, dataBase, language, callback)
		{
			if(srcModelList==null)
			{
				if(callback) callback(null);
				return null;
			}
			// 先翻译当前Model
			var transModelList = new xlsys.java.util.ArrayList();
			if(callback)
			{
				if(srcModelList.size()>0)
				{
					var counter = [0];
					for(var i=0;i<srcModelList.size();++i)
					{
						this._getTranslatedModel(srcModelList.get(i), envId, bufferName, dataBase, language, qx.Bootstrap.bind(this.__getAllTranslatedModels1, this, srcModelList, callback, transModelList, counter));
					}
				}
				else callback(transModelList);
			}
			else
			{
				for(var i=0;i<srcModelList.size();++i)
				{
					var transModel = this._getTranslatedModel(srcModelList.get(i), envId, bufferName, dataBase, language);
					transModelList.add(transModel);
				}
				return transModelList;
			}
		},
		
		__getAllTranslatedModels1 : function(srcModelList, callback, transModelList, counter, transModel)
		{
			transModelList.add(transModel);
			++counter[0];
			if(counter[0]==srcModelList.size()) callback(transModelList);
		},
		
		_translateSubList : function(model, envId, bufferName, dataBase, language, callback)
		{
			var fieldArr = model.getSerializableMembers();
			var todoMap = new xlsys.java.util.HashMap();
			for(var i=0;i<fieldArr.length;++i)
			{
				var field = fieldArr[i];
				if(model[field]!=null&&qx.Class.implementsInterface(model[field], xlsys.java.util.List))
				{
					if(!model[field].isEmpty()&&qx.Class.implementsInterface(model[field].get(0), xlsys.base.model.IModel))
					{
						var subModelList = model[field];
						var cloneList = new xlsys.java.util.ArrayList();
						var size = subModelList.size();
						if(callback)
						{
							if(size>0) todoMap.put(field, size);
							for(var j=0;j<subModelList.size();++j)
							{
								this._getTranslatedModel(subModelList.get(j), envId, bufferName, dataBase, language, qx.Bootstrap.bind(this.__translateSubList1, this, model, callback, field, cloneList, todoMap));
							}
						}
						else
						{
							for(var j=0;j<subModelList.size();++j)
							{
								var transModel = this._getTranslatedModel(subModelList.get(j), envId, bufferName, dataBase, language);
								cloneList.add(transModel);
							}
							model[field] = cloneList;
						}
					}
				}
			}
			if(callback&&todoMap.isEmpty()) callback();
		},
		
		__translateSubList1 : function(model, callback, field, cloneList, todoMap, transModel)
		{
			cloneList.add(transModel);
			if(cloneList.size()==todoMap.get(field))
			{
				model[field] = cloneList;
				todoMap.remove(field);
			}
			if(todoMap.isEmpty()) callback();
		},
		
		__getAllModelsWithSession : function(bufferName, session, modelCreator, callback)
		{
			var envId = session.getAttribute(xlsys.base.XLSYS.SESSION_ENV_ID);
			return this.__getAllModelsWithEnvId(bufferName, envId, modelCreator, callback);
		},
				
		__getAllModelsWithEnvId : function(bufferName, envId, modelCreator, callback)
		{
			var modelMap = this.getBufferMap(envId, bufferName);
			if(callback) this.getModelCount(bufferName, envId, qx.Bootstrap.bind(this.__getAllModelsWithEnvId1, this, bufferName, envId, modelCreator, callback, modelMap));
			else
			{
				var allCount = this.getModelCount(bufferName, envId);
				if(allCount==-1||modelMap.size()<allCount)
				{
					var dataBase = xlsys.base.database.ClientDataBase.getInstance();
					this.initAllModels(bufferName, dataBase, modelCreator);
					dataBase.close();
				}
				var modelList = new xlsys.java.util.ArrayList();
				var valueArray = modelMap.values();
				for(var i=0;i<valueArray.length;++i)
				{
					modelList.add(valueArray[i]);
				}
				return modelList;
			}
		},
		
		__getAllModelsWithEnvId1 : function(bufferName, envId, modelCreator, callback, modelMap, allCount)
		{
			if(allCount==-1||modelMap.size()<allCount)
			{
				var dataBase = xlsys.base.database.ClientDataBase.getInstance();
				this.initAllModels(bufferName, dataBase, modelCreator, qx.Bootstrap.bind(this.__getAllModelsWithEnvId2, this, bufferName, dataBase, modelCreator, callback, modelMap));
			}
			else this.__getAllModelsWithDataBase2(bufferName, modelCreator, callback, modelMap);
		},
		
		__getAllModelsWithEnvId2 : function(bufferName, dataBase, modelCreator, callback, modelMap)
		{
			dataBase.close();
			this.__getAllModelsWithDataBase2(bufferName, modelCreator, callback, modelMap);
		},

		__getAllModelsWithDataBase : function(bufferName, dataBase, modelCreator, callback)
		{
			var modelMap = this.getBufferMap(dataBase.getEnvId(), bufferName);
			if(callback) this.getModelCount(bufferName, dataBase.getEnvId(), qx.Bootstrap.bind(this.__getAllModelsWithDataBase1, this, bufferName, dataBase, modelCreator, callback, modelMap));
			else
			{
				var allCount = this.getModelCount(bufferName, dataBase.getEnvId());
				if(allCount==-1||modelMap.size()<allCount) this.initAllModels(bufferName, dataBase, modelCreator);
				var modelList = new xlsys.java.util.ArrayList();
				var valueArray = modelMap.values();
				for(var i=0;i<valueArray.length;++i)
				{
					modelList.add(valueArray[i]);
				}
				return modelList;
			}
		},
		
		__getAllModelsWithDataBase1 : function(bufferName, dataBase, modelCreator, callback, modelMap, allCount)
		{
			if(allCount==-1||modelMap.size()<allCount) this.initAllModels(bufferName, dataBase, modelCreator, qx.Bootstrap.bind(this.__getAllModelsWithDataBase2, this, bufferName, modelCreator, callback, modelMap));
			else this.__getAllModelsWithDataBase2(bufferName, modelCreator, callback, modelMap);
		},
		
		__getAllModelsWithDataBase2 : function(bufferName, modelCreator, callback, modelMap)
		{
			var modelList = new xlsys.java.util.ArrayList();
			var valueArray = modelMap.values();
			for(var i=0;i<valueArray.length;++i)
			{
				modelList.add(valueArray[i]);
			}
			callback(modelList);
		},
		
		/**
		 * 支持以下方法:
		 * getModel(bufferName, session, key, callback);
		 * getModel(bufferName, envId, key, callback);
		 * getModel(bufferName, dataBase, key, callback);
		 */
		getModel : function(bufferName, sed, key, modelCreator, callback)
		{
			var paramMap = new xlsys.java.util.HashMap();
			paramMap.put(this.self(arguments).BUFFER_KEY_MODEL_KEY, key);
			if(qx.lang.Type.isNumber(sed)) paramMap.put(this.self(arguments).BUFFER_KEY_ENV_ID, sed);
			else if(sed instanceof xlsys.base.session.Session) paramMap.put(this.self(arguments).BUFFER_KEY_SESSION, sed);
			else if(qx.Class.implementsInterface(sed, xlsys.base.database.IDataBase)) paramMap.put(this.self(arguments).BUFFER_KEY_DATABASE, sed);
			return this.getBufferData(bufferName, paramMap, callback);
		},
		
		_getTranslatedModel : function(srcModel, envId, bufferName, dataBase, language, callback)
		{
			if(srcModel==null) return null;
			// 查询翻译缓存中是否存在
			var envTransMap = this.modelTransMap.get(envId);
			if(envTransMap==null)
			{
				envTransMap = new xlsys.java.util.HashMap();
				this.modelTransMap.put(envId, envTransMap);
			}
			var bnTransMap = envTransMap.get(bufferName);
			if(bnTransMap==null)
			{
				bnTransMap = new xlsys.java.util.HashMap();
				envTransMap.put(bufferName, bnTransMap);
			}
			var mTransMap = bnTransMap.get(srcModel);
			if(mTransMap==null)
			{
				mTransMap = new xlsys.java.util.HashMap();
				bnTransMap.put(srcModel, mTransMap);
			}
			var transModel = mTransMap.get(language);
			if(transModel==null)
			{
				if(callback) xlsys.base.util.TranslateUtil.getInstance(language).translateModel(envId, dataBase, srcModel, qx.Bootstrap.bind(this.__getTranslatedModel1, this, envId, bufferName, dataBase, language, callback, mTransMap));
				else
				{
					// 先翻译当前Model
					transModel = xlsys.base.util.TranslateUtil.getInstance(language).translateModel(envId, dataBase, srcModel);
					// 查找当前Model的所有属性, 如果属性为List, 并且List中的元素为IModel, 则翻译该属性
					this._translateSubList(transModel, envId, bufferName, dataBase, language);
					mTransMap.put(language, transModel);
					return transModel;
				}
			}
			else
			{
				if(callback) callback(transModel);
				else return transModel;
			}
		},
		
		__getTranslatedModel1 : function(envId, bufferName, dataBase, language, callback, mTransMap, transModel)
		{
			// 查找当前Model的所有属性, 如果属性为List, 并且List中的元素为IModel, 则翻译该属性
			this._translateSubList(transModel, envId, bufferName, dataBase, language, qx.Bootstrap.bind(this.__getTranslatedModel2, this, mTransMap, language, transModel, callback));
		},
		
		__getTranslatedModel2 : function(mTransMap, language, transModel, callback)
		{
			mTransMap.put(language, transModel);
			callback(transModel);
		},
				
		__getModelWithSession : function(bufferName, session, key, modelCreator, callback)
		{
			var envId = session.getAttribute(xlsys.base.XLSYS.SESSION_ENV_ID);
			return this.__getModelWithEnvId(bufferName, envId, key, modelCreator, callback);
		},
				
		__getModelWithEnvId : function(bufferName, envId, key, modelCreator, callback)
		{
			if(key==null) return null;
			var modelMap = this.getBufferMap(envId, bufferName);
			var model = modelMap.get(key);
			if(model==null)
			{
				var dataBase = xlsys.base.database.ClientDataBase.getInstance();
				if(callback)
				{
					this.__getModelWithDataBase(bufferName, dataBase, key, modelCreator, qx.Bootstrap.bind(this.__getModelWithEnvId1, this, dataBase, callback));
					return;
				}
				else
				{
					model = this.__getModelWithDataBase(bufferName, dataBase, key, modelCreator);
					dataBase.close();
				}
			}
			else
			{
				if(callback)
				{
					callback(model);
					return;
				}
			}
			return model;
		},
		
		__getModelWithEnvId1 : function(dataBase, callback, model)
		{
			dataBase.close();
			callback(model);
		},

		__getModelWithDataBase : function(bufferName, dataBase, key, modelCreator, callback)
		{
			if(key==null)
			{
				if(callback) callback(null);
				return null;
			}
			var modelMap = this.getBufferMap(dataBase.getEnvId(), bufferName);
			var model = modelMap.get(key);
			if(model==null)
			{
				if(callback)
				{
					modelCreator.createModel(dataBase, key, qx.Bootstrap.bind(this.__getModelWithDataBase1, this, modelMap, key, callback));
					return;
				}
				else
				{
					model = modelCreator.createModel(dataBase, key);
					modelMap.put(key, model);
				}
			}
			else
			{
				if(callback)
				{
					callback(model);
					return;
				}
			}
			return model;
		},
		
		__getModelWithDataBase1 : function(modelMap, key, callback, model)
		{
			modelMap.put(key, model);
			callback(model);
		},
		
		getStorageObject : function(bufferName, mark)
		{
			var session = xlsys.base.session.SessionManager.getInstance().getCurrentSession();
			var envId = session.getAttribute(xlsys.base.XLSYS.SESSION_ENV_ID);
			var bufferMap = this.allEnvBufferMap.get(envId).get(bufferName);
			var modelCount = this.modelCountMap.get(envId).get(bufferName);
			var transMap = this.modelTransMap.get(envId).get(bufferName);
			var storageObject = new Array();
			storageObject.push(bufferMap);
			storageObject.push(transMap);
			if(qx.lang.String.startsWith(mark, xlsys.base.buffer.XlsysBuffer.MARK_ALL)) storageObject.push(modelCount);
			return storageObject;
		},
		
		isBufferComplete : function(bufferName, callback)
		{
			var session = xlsys.base.session.SessionManager.getInstance().getCurrentSession();
			var envId = session.getAttribute(xlsys.base.XLSYS.SESSION_ENV_ID);
			var envBufferMap = this.allEnvBufferMap.get(envId);
			var countMap = this.modelCountMap.get(envId);
			if(envBufferMap==null||countMap==null)
			{
				if(callback) callback(false);
				return false;
			}
			var bufferMap = envBufferMap.get(bufferName);
			var count = countMap.get(bufferName);
			if(bufferMap==null||count==null)
			{
				if(callback) callback(false);
				return false;
			}
			if(bufferMap.size()==count)
			{
				if(callback) callback(true);
				return true;
			}
			if(callback) callback(false);
			else return false;
		},
		
		reloadDataDirectly : function(bufferName, paramMap, forceLoad, callback)
		{
			if(forceLoad==null) forceLoad = false;
			var session = xlsys.base.session.SessionManager.getInstance().getCurrentSession();
			var envId = session.getAttribute(xlsys.base.XLSYS.SESSION_ENV_ID);
			var reloadAll = false;
			if(paramMap==null) reloadAll = true;
			else
			{
				var key = paramMap.get(this.self(arguments).BUFFER_KEY_MODEL_KEY);
				if(key==null) reloadAll = true;
				else
				{
					this.getBufferMap(envId, bufferName).remove(key);
					if(forceLoad) this.getModel(bufferName, envId, key, callback);
					else if(callback) callback();
					return;
				}
			}
			if(reloadAll)
			{
				this.getBufferMap(envId, bufferName).clear();
				var countMap = this.modelCountMap.get(envId);
				if(countMap!=null) countMap.remove(bufferName);
				var envTransMap = this.modelTransMap.get(envId);
				if(envTransMap!=null) envTransMap.remove(bufferName);
				if(forceLoad) this.getAllModels(bufferName, envId, callback);
				else callback();
			}
		},
		
		loadDataFromStorageObject : function(bufferName, storageObj, mark)
		{
			var session = xlsys.base.session.SessionManager.getInstance().getCurrentSession();
			var envId = session.getAttribute(xlsys.base.XLSYS.SESSION_ENV_ID);
			var objArr = storageObj;
			var bufferMap = objArr[0];
			var transMap = objArr[1];
			var curBufferMap = this.getBufferMap(envId, bufferName);
			curBufferMap.putAll(bufferMap);
			var envTransMap = this.modelTransMap.get(envId);
			if(envTransMap==null)
			{
				envTransMap = new xlsys.java.util.HashMap();
				this.modelTransMap.put(envId, envTransMap);
			}
			var curTransMap = envTransMap.get(bufferName);
			if(curTransMap==null)
			{
				curTransMap = new xlsys.java.util.HashMap();
				envTransMap.put(bufferName, curTransMap);
			}
			curTransMap.putAll(transMap);
			if(qx.lang.String.startsWith(mark, xlsys.base.buffer.XlsysBuffer.MARK_ALL))
			{
				var modelCount = objArr[2];
				var countMap = this.modelCountMap.get(envId);
				if(countMap==null)
				{
					countMap = this.modelCountMap.get(envId);
					if(countMap==null)
					{
						countMap = new xlsys.java.util.HashMap();
						this.modelCountMap.put(envId, countMap);
					}
				}
				countMap.put(bufferName, modelCount);
			}
			return true;
		},
		
		doGetBufferData : function(bufferName, paramMap, callback)
		{
			var session = xlsys.base.session.SessionManager.getInstance().getCurrentSession();
			var envId = session.getAttribute(xlsys.base.XLSYS.SESSION_ENV_ID);
			var language = session.getAttribute(xlsys.base.XLSYS.SESSION_LANGUAGE);
			var ret = null;
			var modelCreator = this.bindModelCreatorMap.get(bufferName);
			if(modelCreator==null) throw new Error("Please bind model creator first!");
			if(paramMap==null)
			{
				if(callback)
				{
					this.dispatchGetAllModels(bufferName, null, envId, null, modelCreator, qx.Bootstrap.bind(this.__doGetBufferData1, this, envId, bufferName, language, callback));
					return;
				}
				else
				{
					var modelList = this.dispatchGetAllModels(bufferName, null, envId, null, modelCreator);
					ret = this._getAllTranslatedModels(modelList, envId, bufferName, xlsys.base.database.ClientDataBase.getInstance(), language);
				}
			}
			else
			{
				var key = paramMap.get(this.self(arguments).BUFFER_KEY_MODEL_KEY);
				var session = paramMap.get(this.self(arguments).BUFFER_KEY_SESSION);
				var dataBase = paramMap.get(this.self(arguments).BUFFER_KEY_DATABASE);
				var tempEnvId = paramMap.get(this.self(arguments).BUFFER_KEY_ENV_ID);
				if(tempEnvId==null) tempEnvId = envId;
				if(key==null)
				{
					if(callback)
					{
						this.dispatchGetAllModels(bufferName, session, tempEnvId, dataBase, modelCreator, qx.Bootstrap.bind(this.__doGetBufferData1, this, envId, bufferName, language, callback));
						return;
					}
					else
					{
						var modelList = this.dispatchGetAllModels(bufferName, session, tempEnvId, dataBase, modelCreator);
						ret = this._getAllTranslatedModels(modelList, envId, bufferName, xlsys.base.database.ClientDataBase.getInstance(), language);
					}
				}
				else
				{
					if(callback)
					{
						this.dispatchGetModel(bufferName, session, tempEnvId, dataBase, key, modelCreator, qx.Bootstrap.bind(this.__doGetBufferData2, this, envId, bufferName, language, callback));
						return;
					}
					else
					{
						var model = this.dispatchGetModel(bufferName, session, tempEnvId, dataBase, key, modelCreator);
						ret = this._getTranslatedModel(model, envId, bufferName, xlsys.base.database.ClientDataBase.getInstance(), language);
					}
				}
			}
			return ret;
		},
		
		__doGetBufferData1 : function(envId, bufferName, language, callback, modelList)
		{
			this._getAllTranslatedModels(modelList, envId, bufferName, xlsys.base.database.ClientDataBase.getInstance(), language, callback);
		},
		
		__doGetBufferData2 : function(envId, bufferName, language, callback, model)
		{
			this._getTranslatedModel(model, envId, bufferName, xlsys.base.database.ClientDataBase.getInstance(), language, callback);
		},
		
		saveDataToLocalStorage : function(bufferName, storageObject, callback)
		{
			if(callback) callback(false);
			else return false;
		},
		
		reloadDataFromLocalStorage : function(bufferName, callback)
		{
			if(callback) callback(false);
			else return false;
		},
		
		getLocalStorageVersion : function(bufferName, callback)
		{
			if(callback) callback(-1);
			else return -1;
		},
		
		dispatchGetModel : function(bufferName, session, envId, dataBase, key, modelCreator, callback)
		{
			if(callback)
			{
				if(session!=null) this.__getModelWithSession(bufferName, session, key, modelCreator, callback);
				else if(dataBase!=null) this.__getModelWithDataBase(bufferName, dataBase, key, modelCreator, callback);
				else this.__getModelWithEnvId(bufferName, envId, key, modelCreator, callback);
			}
			else
			{
				var model = null;
				if(session!=null) model = this.__getModelWithSession(bufferName, session, key, modelCreator);
				else if(dataBase!=null) model = this.__getModelWithDataBase(bufferName, dataBase, key, modelCreator);
				else model = this.__getModelWithEnvId(bufferName, envId, key, modelCreator);
				return model;
			}
		},

		dispatchGetAllModels : function(bufferName, session, envId, dataBase, modelCreator, callback)
		{
			if(callback)
			{
				if(session!=null) this.__getAllModelsWithSession(bufferName, session, modelCreator, callback);
				else if(dataBase!=null) this.__getAllModelsWithDataBase(bufferName, dataBase, modelCreator, callback);
				else this.__getAllModelsWithEnvId(bufferName, envId, modelCreator, callback);
			}
			else
			{
				var modelList = null;
				if(session!=null) modelList = this.__getAllModelsWithSession(bufferName, session, modelCreator);
				else if(dataBase!=null) modelList = this.__getAllModelsWithDataBase(bufferName, dataBase, modelCreator);
				else modelList = this.__getAllModelsWithEnvId(bufferName, envId, modelCreator);
				return modelList;
			}
		}
	}
});