/**
 * ClientDataBase
 */
qx.Class.define("xlsys.base.database.ClientDataBase",
{
	extend : qx.core.Object,
	implement : [xlsys.base.database.IClientDataBase],
	
	statics :
	{
		getInstance : function()
		{
			return new xlsys.base.database.ClientDataBase();
		}
	},
	
	construct: function()
	{
		this.autoCommit = true;
		this.transactionList = new xlsys.java.util.ArrayList();
	},
	
	members : 
	{
		autoCommit : null,
		transactionList : null,

		getEnvId : function()
		{
			var session = xlsys.base.session.SessionManager.getInstance().getCurrentSession();
			return session.getAttribute(xlsys.base.XLSYS.SESSION_ENV_ID);
		},

		/**
		 * 需要重载以下方法
		 * getTableInfo(tableName);
		 * getTableInfo(tableName, callback);
		 */
		getTableInfo : function(tableName, callback)
		{
			var envId = this.getEnvId();
			var bufferName = "_ENV"+xlsys.base.XLSYS.BUFFER_TABLE_INFO_PREFIX+envId;
			var paramMap = new xlsys.java.util.HashMap();
			paramMap.put(xlsys.base.database.IDataBase.BUFFER_KEY_TABLE_NAME, tableName);
			if(callback==null) return xlsys.base.database.ClientTableInfoBuffer.getInstance().getBufferData(bufferName, paramMap);
			else xlsys.base.database.ClientTableInfoBuffer.getInstance().getBufferData(bufferName, paramMap, callback);
		},

		/**
		 * 需要重载以下方法
		 * getAllTableBaseInfo();
		 * getAllTableBaseInfo(callback);
		 */
		getAllTableBaseInfo : function()
		{
			if(arguments.length==0)
			{
				return xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_GET_ALL_TABLE_BASE_INFO, null);
			}
			else if(arguments.length==1)
			{
				xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_GET_ALL_TABLE_BASE_INFO, null, arguments[0]);
			}
		},

		/**
		 * 需重载以下方法
		 * getResultCount(paramBean);
		 * getResultCount(paramBean, callback);
		 */
		getResultCount : function(paramBean)
		{
			if(arguments.length==1)
			{
				return xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_GET_RESULT_COUNT, paramBean);
			}
			else if(arguments.length==2)
			{
				xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_GET_RESULT_COUNT, paramBean, arguments[1]);
			}
		},

		/**
		 * 需要重载以下方法
		 * sqlSelectAsOneValue(selectSql);
		 * sqlSelectAsOneValue(paramBean);
		 * sqlSelectAsOneValue(selectSql, callback);
		 * sqlSelectAsOneValue(paramBean, callback);
		 */
		sqlSelectAsOneValue : function(sqlOrPb)
		{
			var pb = null;
			if((typeof sqlOrPb)=="string") pb = new xlsys.base.database.bean.ParamBean(sqlOrPb);
			else pb = sqlOrPb;
			if(arguments.length==1)
			{
				return xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_SQL_SELECT_AS_ONE_VALUE, pb);
			}
			else if(arguments.length==2)
			{
				xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_SQL_SELECT_AS_ONE_VALUE, pb, arguments[1]);
			}
		},

		/**
		 * 需要重载以下方法
		 * sqlSelect(String selectSql);
		 * sqlSelect(String selectSql, callback);
		 * sqlSelect(ParamBean paramBean);
		 * sqlSelect(ParamBean paramBean, callback);
		 * sqlSelect(ParamBean paramBean, int beginRowNum, int endRowNum);
		 * sqlSelect(ParamBean paramBean, int beginRowNum, int endRowNum, callback);
		 * sqlSelect(ParamBean paramBean, Map<String, Integer> sortColMap, int beginRowNum, int endRowNum);
		 * sqlSelect(ParamBean paramBean, Map<String, Integer> sortColMap, int beginRowNum, int endRowNum, callback);
		 */
		sqlSelect : function()
		{
			if(arguments.length<=2)
			{
				var pb = null;
				if((typeof arguments[0])=="string") pb = new xlsys.base.database.bean.ParamBean(arguments[0]);
				else pb = arguments[0];
				if(arguments.length==1) return xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_SQL_SELECT, pb);
				else xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_SQL_SELECT, pb, arguments[1]);
			}
			else if(arguments.length==3)
			{
				var arr = new Array();
				arr.push(arguments[0]);
				arr.push(arguments[1]);
				arr.push(arguments[2]);
				return xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_SQL_SELECT, arr);
			}
			else if(arguments.length==4)
			{
				if((typeof arguments[3])=="number")
				{
					var arr = new Array();
					arr.push(arguments[0]);
					arr.push(arguments[1]);
					arr.push(arguments[2]);
					arr.push(arguments[3]);
					return xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_SQL_SELECT, arr);
				}
				else
				{
					var arr = new Array();
					arr.push(arguments[0]);
					arr.push(arguments[1]);
					arr.push(arguments[2]);
					xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_SQL_SELECT, arr, arguments[3]);
				}
			}
			else if(arguments.length==5)
			{
				var arr = new Array();
				arr.push(arguments[0]);
				arr.push(arguments[1]);
				arr.push(arguments[2]);
				arr.push(arguments[3]);
				xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_SQL_SELECT, arr, arguments[4]);
			}
		},

		/**
		 * 需要重载以下方法
		 * sqlExecute(String executeSql);
		 * sqlExecute(String executeSql, callback);
		 * sqlExecute(ExecuteBean executeBean);
		 * sqlExecute(ExecuteBean executeBean, callback);
		 * sqlExecute(List<? extends ISqlBean> sqlBeanList);
		 * sqlExecute(List<? extends ISqlBean> sqlBeanList, callback);
		 * sqlExecute(ParamBean paramBean);
		 * sqlExecute(ParamBean paramBean, callback);
		 */
		sqlExecute : function()
		{
			if((typeof arguments[0])=="string")
			{
				var eb = new xlsys.base.database.bean.ExecuteBean(arguments[0]);
				if(arguments.length==1) return this.__sqlExecute(eb);
				else if(arguments.length==2) this.__sqlExecute(eb, arguments[1]);
			}
			else
			{
				if(arguments.length==1) return this.__sqlExecute(arguments[0]);
				else if(arguments.length==2) this.__sqlExecute(arguments[0], arguments[1]);
			}
		},

		__sqlExecute : function(sqlBeanOrList)
		{
			if(this.autoCommit)
			{
				if(arguments.length==1) return xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_SQL_EXECUTE, sqlBeanOrList);
				else if(arguments.length==2) xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_SQL_EXECUTE, sqlBeanOrList, arguments[1]);
			}
			else
			{
				if(qx.Interface.objectImplements(sqlBeanOrList, xlsys.java.util.List)) this.transactionList.addAll(sqlBeanOrList);
				else this.transactionList.add(sqlBeanOrList);
				if(arguments.length==2) arguments[1](true);
				else return true;
			}
		},
		
		/**
		 * 需要重载以下方法
		 * getSplitedParamBean(paramBean, beginRowNum, endRowNum);
		 * getSplitedParamBean(paramBean, beginRowNum, endRowNum, callback);
		 */
		getSplitedParamBean : function(paramBean, beginRowNum, endRowNum)
		{
			var arr = new Array();
			arr.push(paramBean);
			arr.push(beginRowNum);
			arr.push(endRowNum);
			if(arguments.length==3) return xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_GET_SPLITED_PARAM_BEAN, arr);
			else if(arguments.length==4) xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_GET_SPLITED_PARAM_BEAN, arr, arguments[3]);
		},

		/**
		 * 需要重载以下方法
		 * disableConstraint(tableName, constraintName);
		 * disableConstraint(tableName, constraintName, callback);
		 */
		disableConstraint : function(tableName, constraintName)
		{
			var arr = new Array();
			arr.push(tableName);
			arr.push(constraintName);
			if(arguments.length==2) return xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_DISABLE_CONSTRAINT, arr);
			else if(arguments.length==3) xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_DISABLE_CONSTRAINT, arr, arguments[2]);
		},

		/**
		 * 需要重载以下方法
		 * enableConstraint(tableName, constraintName);
		 * enableConstraint(tableName, constraintName, callback);
		 */
		enableConstraint : function(tableName, constraintName)
		{
			var arr = new Array();
			arr.push(tableName);
			arr.push(constraintName);
			if(arguments.length==2) return xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_ENABLE_CONSTRAINT, arr);
			else if(arguments.length==3) xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_ENABLE_CONSTRAINT, arr, arguments[2]);
		},

		/**
		 * 需要重载以下方法
		 * getLowerFunc(srcExp);
		 * getLowerFunc(srcExp, callback);
		 */
		getLowerFunc : function(srcExp)
		{
			if(arguments.length==1) return xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_FUNC_LOWER, srcExp);
			else if(arguments.length==2) xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_FUNC_LOWER, srcExp, arguments[1]);
		},

		/**
		 * 需要重载以下方法
		 * getUpperFunc(srcExp);
		 * getUpperFunc(srcExp, callback);
		 */
		getUpperFunc : function(srcExp)
		{
			if(arguments.length==1) return xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_FUNC_UPPER, srcExp);
			else if(arguments.length==2) xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_FUNC_UPPER, srcExp, arguments[1]);
		},

		/**
		 * 需要重载以下方法
		 * getByteLengthFunc(srcExp);
		 * getByteLengthFunc(srcExp, callback);
		 */
		getByteLengthFunc : function(srcExp)
		{
			if(arguments.length==1) return xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_FUNC_BYTELENGTH, srcExp);
			else if(arguments.length==2) xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_FUNC_BYTELENGTH, srcExp, arguments[1]);
		},

		/**
		 * 需重载以下方法
		 * getSubStringFunc(String srcExp, int begin);
		 * getSubStringFunc(String srcExp, int begin, callback);
		 * getSubStringFunc(String srcExp, int begin, int end);
		 * getSubStringFunc(String srcExp, int begin, int end, callback);
		 */
		getSubStringFunc : function()
		{
			if(arguments.length==2)
			{
				var arr = new Array();
				arr.push(arguments[0]);
				arr.push(arguments[1]);
				return xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_FUNC_SUBSTRING, arr);
			}
			else if(arguments.length==3)
			{
				if((typeof arguments[2])=="number")
				{
					var arr = new Array();
					arr.push(arguments[0]);
					arr.push(arguments[1]);
					arr.push(arguments[2]);
					return xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_FUNC_SUBSTRING, arr);
				}
				else
				{
					var arr = new Array();
					arr.push(arguments[0]);
					arr.push(arguments[1]);
					xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_FUNC_SUBSTRING, arr, arguments[2]);
				}
			}
			else if(arguments.length==4)
			{
				var arr = new Array();
				arr.push(arguments[0]);
				arr.push(arguments[1]);
				arr.push(arguments[2]);
				xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_FUNC_SUBSTRING, arr, arguments[3]);
			}
		},

		/**
		 * 需重载以下方法
		 * getToDateFunc(date);
		 * getToDateFunc(date, callback);
		 */
		getToDateFunc : function(date)
		{
			if(arguments.length==1) return xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_FUNC_TODATE, date);
			else if(arguments.length==2) xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_FUNC_TODATE, date, arguments[1]);
		},

		getDataBaseId : function()
		{
			throw new xlsys.java.lang.Exception("Unsupported Operation!");
		},
		
		/**
		 * 需重载以下方法
		 * commit();
		 * commit(callback);
		 */
		commit : function()
		{
			if(!this.autoCommit&&this.transactionList.size()>0)
			{
				if(arguments.length==0)
				{
					var ret = xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_SQL_EXECUTE, this.transactionList);
					if(ret instanceof xlsys.java.lang.Exception) throw ret;
					else this.transactionList.clear();
				}
				else if(arguments.length==1)
				{
					xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_SQL_EXECUTE, this.transactionList, qx.Bootstrap.bind(this.__commitCallBack, this, arguments[0]));
				}
			}
			else
			{
				if(arguments.length==0) return true;
				else if(arguments.length==1) arguments[0](true);
			}
		},

		__commitCallBack : function(callback, data, status)
		{
			if(!(data instanceof xlsys.java.lang.Exception)) this.transactionList.clear();
			callback(data, status);
		},

		rollback : function()
		{
			if(!this.autoCommit&&this.transactionList.size()>0)
			{
				this.transactionList.clear();
			}
		},

		/**
		 * 需重载以下方法
		 * setAutoCommit(autoCommit);
		 * setAutoCommit(autoCommit, callback);
		 */
		setAutoCommit : function(autoCommit)
		{
			if(autoCommit&&!this.autoCommit)
			{
				if(arguments.length==1)
				{
					this.commit();
					this.autoCommit = autoCommit;
				}
				else if(arguments.length==2)
				{
					this.commit(qx.Bootstrap.bind(this.__setAutoCommitCallBack, this, autoCommit, arguments[1]));
				}
			}
			else
			{
				if(arguments.length==1) this.autoCommit = autoCommit;
				else if(arguments.length==2)
				{
					this.autoCommit = autoCommit;
					arguments[1](null, "success");
				}
			}
		},
		
		__setAutoCommitCallBack : function(autoCommit, callback, data, status)
		{
			this.autoCommit = autoCommit;
			callback(null, "success");
		},

		getAutoCommit : function()
		{
			return this.autoCommit;
		},

		/**
		 * 需重载以下方法
		 * close();
		 * close(callback);
		 */
		close : function()
		{
			if(!this.autoCommit)
			{
				if(arguments.length==1) this.commit();
				else if(arguments.length==2) this.commit(arguments[0]);
			}
		},

		isClose : function()
		{
			return false;
		}
	}
});