/**
 * ClientDataBase的表信息缓存对象
 */
qx.Class.define("xlsys.base.database.ClientTableInfoBuffer",
{
	type : "singleton",
	extend : xlsys.base.buffer.AbstractBuffer,
	
	construct : function()
	{
		this.base(arguments);
		this._tableInfoBuffer = new xlsys.java.util.HashMap();
		var session = xlsys.base.session.SessionManager.getInstance().getCurrentSession();
		var envId = session.getAttribute(xlsys.base.XLSYS.SESSION_ENV_ID);
		xlsys.base.buffer.BufferManager.getInstance().registerBuffer("_ENV"+xlsys.base.XLSYS.BUFFER_TABLE_INFO_PREFIX+envId, this);
	},
	
	members : 
	{
		_tableInfoBuffer : null,
		_tableCount : null,
		
		getStorageObject : function(bufferName, mark)
		{
			var arr = new Array();
			arr.push(this._tableInfoBuffer);
			if(mark==xlsys.base.buffer.XlsysBuffer.MARK_ALL) arr.push(this._tableCount);
			return arr;
		},
		
		isBufferComplete : function(bufferName, callback)
		{
			if(this._tableCount==null)
			{
				var dataBase = xlsys.base.database.ClientDataBase.getInstance();
				if(callback)
				{
					dataBase.getAllTableBaseInfo(qx.Bootstrap.bind(this.__isBufferComplete1, this, callback, dataBase));
					return;
				}
				else
				{
					this._tableCount = dataBase.getAllTableBaseInfo().size();
					dataBase.close();
				}
			}
			return this._tableInfoBuffer.size()==this._tableCount;
		},
		
		__isBufferComplete1 : function(callback, dataBase, allInfo)
		{
			this._tableCount = allInfo.size();
			dataBase.close();
			callback(this._tableInfoBuffer.size()==this._tableCount);
		},
		
		reloadDataDirectly : function(bufferName, paramMap, forceLoad, callback)
		{
			var tableName = null;
			if(paramMap!=null) tableName = paramMap.get(xlsys.base.database.IDataBase.BUFFER_KEY_TABLE_NAME);
			if(tableName!=null) this._tableInfoBuffer.remove(tableName);
			else
			{
				this._tableInfoBuffer.clear();
				this._tableCount = null;
			}
			if(forceLoad) this.getBufferData(bufferName, paramMap, callback);
			else callback();
		},
		
		loadDataFromStorageObject : function(bufferName, storageObj, mark)
		{
			if(mark==xlsys.base.buffer.XlsysBuffer.MARK_ALL)
			{
				this._tableInfoBuffer = storageObj[0];
				this._tableCount = storageObj[1];
			}
			else this._tableInfoBuffer.putAll(storageObj);
			return true;
		},
		
		doGetBufferData : function(bufferName, paramMap, callback)
		{
			var ret = null;
			var tableName = null;
			if(paramMap!=null) tableName = paramMap.get(xlsys.base.database.IDataBase.BUFFER_KEY_TABLE_NAME);
			if(tableName!=null) ret = this._tableInfoBuffer.get(tableName);
			else
			{
				ret = this._tableInfoBuffer;
				if(this._tableCount==null||this._tableInfoBuffer.size()!=this._tableCount) ret = null;
			}
			if(ret!=null)
			{
				if(callback) callback(ret);
				else return ret;
			}
			else
			{
				var session = xlsys.base.session.SessionManager.getInstance().getCurrentSession();
				var envId = session.getAttribute(xlsys.base.XLSYS.SESSION_ENV_ID);
				if(tableName!=null)
				{
					if(callback) xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_GET_TABLE_INFO, tableName, qx.Bootstrap.bind(this._getTableInfoRet, this, tableName, callback));
					else
					{
						var tableInfo = xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_GET_TABLE_INFO, tableName);
						this._tableInfoBuffer.put(tableName, tableInfo);
						return tableInfo;
					}
				}
				else
				{
					if(callback)
					{
						var dataBase = xlsys.base.database.ClientDataBase.getInstance();
						dataBase.getAllTableBaseInfo(qx.Bootstrap.bind(this._onLoadAllInfo1, this, dataBase, callback));
					}
					else
					{
						var dataBase = xlsys.base.database.ClientDataBase.getInstance();
						var allTableInfo = dataBase.getAllTableBaseInfo();
						this._tableCount = allTableInfo.size();
						var keys = allTableInfo.keys();
						for(var i=0;i<keys.length;++i)
						{
							var tempTable = keys[i];
							var tableInfo = xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_GET_TABLE_INFO, tempTable);
							this._tableInfoBuffer.put(tempTable, tableInfo);
						}
						dataBase.close();
						return this._tableInfoBuffer;
					}
				}
			}
		},
		
		_onLoadAllInfo1 : function(dataBase, callback, allTableInfo)
		{
			this._tableCount = allTableInfo.size();
			var keys = allTableInfo.keys();
			var counter = [0];
			for(var i=0;i<keys.length;++i)
			{
				var tempTable = keys[i];
				xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_DB_GET_TABLE_INFO, tempTable, qx.Bootstrap.bind(this._onLoadAllInfo2, this, dataBase, keys, callback, counter, tempTable));
			}
		},
		
		_onLoadAllInfo2 : function(dataBase, keys, callback, counter, tempTable, tableInfo)
		{
			this._tableInfoBuffer.put(tempTable, tableInfo);
			++counter[0];
			if(counter[0]==keys.length)
			{
				dataBase.close();
				callback(this._tableInfoBuffer);
			}
		},
		
		_getTableInfoRet : function(tableName, callback, data)
		{
			this._tableInfoBuffer.put(tableName, data);
			callback(data);
		},
		
		getBufferData : function(bufferName, paramMap, callback)
		{
			return this._getBufferData(bufferName, paramMap, callback);
		},
		
		getFinalStorageObject : function(bufferName, mark)
		{
			return this._getFinalStorageObject(bufferName, mark);
		},
		
		updateCurrentVersion : function(bufferName)
		{
			return this._updateCurrentVersion(bufferName);
		},
		
		getCurrentVersion : function(bufferName)
		{
			return this._getCurrentVersion(bufferName);
		},
		
		reloadAllData : function(bufferName, forceLoad)
		{
			this._reloadAllData(bufferName, forceLoad);
		},
		
		reloadData : function(bufferName, paramMap, forceLoad)
		{
			this._reloadData(bufferName, paramMap, forceLoad);
		},
		
		saveDataToLocalStorage : function(bufferName, storageObject, callback)
		{
			if(callback) callback(false);
			else return false;
		},
		
		reloadDataFromLocalStorage : function(bufferName, callback)
		{
			if(callback) callback(false);
			else return false;
		},
		
		getLocalStorageVersion : function(bufferName, callback)
		{
			if(callback) callback(-1);
			else return -1;
		}
	}
});