/**
 * DataSet
 */
qx.Class.define("xlsys.base.dataset.DataSet",
{
	extend : xlsys.java.io.Serializable,
	implement : [xlsys.base.dataset.IDataSet],
	
	construct: function()
	{
		this.name = null;
		this.style = null;
		this.columnMap = null;
		this.dataSetListenerSet = null;
		this.oldEditRow = null;
		this.regionChecking = null;
		
		this.sortColMap = new xlsys.java.util.LinkedHashMap();
		this.columns = new xlsys.java.util.ArrayList();
		this.rows = new xlsys.java.util.ArrayList();
		this.rowCursor = -1;
		this.colCursor = -1;
		this.expandPerCount = 32;
		this.optionMap = new xlsys.java.util.HashMap();
		
		var dataSetColumns = null;
		var style = null;
		if(arguments.length==0) style = xlsys.base.dataset.IDataSet.STYLE_NONE;
		else if(arguments.length==1)
		{
			if(qx.lang.Type.isNumber(arguments[0])) style = arguments[0];
			else
			{
				dataSetColumns = arguments[0];
				style = xlsys.base.dataset.IDataSet.STYLE_NONE;
			}
		}
		else if(arguments.length==2)
		{
			dataSetColumns = arguments[0];
			style = arguments[1];
		}
		this.style = style;
		if(dataSetColumns!=null)
		{
			for(var i=0;i<dataSetColumns.size();++i)
			{
				var dsc = dataSetColumns.get(i);
				this.insertNewColumnAfterLast().cloneOf(dsc);
			}
			this.buildColumnMap();
		}
	},
	
	members :
	{
		name : null,
		style : null,
		columnMap : null,
		dataSetListenerSet : null,
		oldEditRow : null,
		regionChecking : null,
		sortColMap : null,
		columns : null,
		rows : null,
		rowCursor : null,
		colCursor : null,
		expandPerCount : null,
		optionMap : null,
		
		getSerializableMembers : function()
		{
			var arr = new Array();
			arr.push("name");
			arr.push("style");
			arr.push("columnMap");
			arr.push("sortColMap");
			arr.push("columns");
			arr.push("rows");
			arr.push("expandPerCount");
			return arr;
		},
		
		getExpandPerCount : function()
		{
			return this.expandPerCount;
		},

		setExpandPerCount : function(expandPerCount)
		{
			this.expandPerCount = expandPerCount;
		},

		setRowCount : function(rowCount)
		{
			if(xlsys.base.util.ObjectUtil.hasBitProp(this.style, xlsys.base.dataset.IDataSet.STYLE_VIRTUAL))
			{
				var curRowCount = this.getRowCount();
				if(curRowCount<rowCount)
				{
					this.setEventLock(true);
					var oldCursor = this.getRowCursor();
					for(var i=0;i<rowCount-curRowCount;++i) this.insertNewRowAfterLast(true);
					this.setRowCursor(oldCursor);
					this.setEventLock(false);
				}
			}
		},

		setColumnCount : function(colCount)
		{
			if(xlsys.base.util.ObjectUtil.hasBitProp(this.style, xlsys.base.dataset.IDataSet.STYLE_VIRTUAL))
			{
				var curColCount = this.getColumnCount();
				if(curColCount<colCount)
				{
					this.setEventLock(true);
					var oldCursor = this.getColCursor();
					for(var i=0;i<colCount-curColCount;++i) this.insertNewColumnAfterLast();
					this.setColCursor(oldCursor);
					this.setEventLock(false);
				}
			}
		},

		buildColumnMap : function()
		{
			if(this.columnMap==null) this.columnMap = new xlsys.java.util.HashMap();
			else this.columnMap.clear();
			for(var i=0;i<this.columns.size();++i)
			{
				var column = this.columns.get(i);
				this.columnMap.put(column.getColumnName().toLowerCase(), i);
			}
		},

		getColumnIndex : function(columnName)
		{
			if(columnName==null) return -1;
			if(this.columnMap==null||this.columnMap.isEmpty()||!this.columnMap.containsKey(columnName.toLowerCase()))
			{
				this.buildColumnMap();
			}
			var i = this.columnMap.get(columnName.toLowerCase());
			return i==null?-1:i;
		},

		removeColumn : function(colNameOrColIdx)
		{
			var colIdx = null;
			if(qx.lang.Type.isString(colNameOrColIdx)) colIdx = this.getColumnIndex(colNameOrColIdx);
			else colIdx = colNameOrColIdx;
			if(!this.colRegionCheck(colIdx)) return;
			var toRemoveCol = this.getColumn(colIdx);
			var event = new xlsys.base.dataset.DataSetEvent(this);
			event.col = toRemoveCol;
			this.fireBeforeRemoveCol(event);
			if(event.doit)
			{
				for(var i=0;i<this.rows.size();++i)
				{
					var row = this.rows.get(i);
					row.removeCell(colIdx);
				}
				this.columns.removeAt(colIdx);
				this.buildColumnMap();
				var cursor = this.getColCursor();
				if(colIdx<=cursor) this.setColCursor(cursor-1);
				event = new xlsys.base.dataset.DataSetEvent(this);
				event.col = toRemoveCol;
				this.fireAfterRemoveCol(event);
			}
		},

		getRowCount : function()
		{
			return this.rows.size();
		},

		getColumnCount : function()
		{
			return this.columns.size();
		},

		getRowCursor : function()
		{
			return this.rowCursor;
		},

		setRowCursor : function(rowCursor)
		{
			var success = false;
			if(this.rowRegionCheck(rowCursor))
			{
				// 扩展到需要的行数
				success = this.expandToRow(rowCursor);
			}
			else if(rowCursor==-1) success = true;
			if(success) this.rowCursor = rowCursor;
			return success;
		},

		getColCursor : function()
		{
			return this.colCursor;
		},

		setColCursor : function(colCursor)
		{
			var success = false;
			if(this.colRegionCheck(colCursor)||colCursor==-1) success = true;
			if(success) this.colCursor = colCursor;
			return success;
		},

		getPageRangeOfRowIndex : function(wantRowIndex)
		{
			// 注意这里不能使用regionCheck，否则会触发两次expandRow
			if(wantRowIndex<0||wantRowIndex>=this.getRowCount()) return null;
			var beginRowIdx = Math.floor(wantRowIndex / this.expandPerCount) * this.expandPerCount;
			var endRowIdx = Math.min(beginRowIdx + this.expandPerCount-1, this.getRowCount()-1);
			var ret = new Array();
			ret.push(beginRowIdx);
			ret.push(endRowIdx);
			return ret;
		},

		expandToRow : function(wantRowIndex)
		{
			if(arguments.length==1) return this._expandToRow(wantRowIndex);
			else this._expandToRowWithCallback(wantRowIndex, arguments[1]);
		},

		_expandToRow : function(wantRowIndex)
		{
			var success = true;
			if(wantRowIndex<0||wantRowIndex>=this.getRowCount()) return success;
			if(xlsys.base.util.ObjectUtil.hasBitProp(this.style, xlsys.base.dataset.IDataSet.STYLE_VIRTUAL)&&this.isVirtualRow(wantRowIndex))
			{
				var range = this.getPageRangeOfRowIndex(wantRowIndex);
				if(range!=null)
				{
					var event = new xlsys.base.dataset.DataSetEvent(this);
					event.beginIdxOfExpand = range[0];
					event.endIdxOfExpand = range[1];
					success = this.fireExpandRows(event);
					if(success)
					{
						// 注意这里不能使用getRow来获取行，因为getRow里面还会触发expandToRow
						for(var i=event.beginIdxOfExpand;i<=event.endIdxOfExpand;++i)
						{
							this.rows.get(i).setChangeStatus(xlsys.base.dataset.DataSetRow.STATUS_COMMON);
						}
					}
				}
			}
			return success;
		},

		_expandToRowWithCallback : function(wantRowIndex, callback)
		{
			var success = true;
			if(wantRowIndex<0||wantRowIndex>=this.getRowCount()) callback(success, "success");
			if(xlsys.base.util.ObjectUtil.hasBitProp(this.style, xlsys.base.dataset.IDataSet.STYLE_VIRTUAL)&&this.isVirtualRow(wantRowIndex))
			{
				var range = this.getPageRangeOfRowIndex(wantRowIndex);
				if(range!=null)
				{
					var event = new xlsys.base.dataset.DataSetEvent(this);
					event.beginIdxOfExpand = range[0];
					event.endIdxOfExpand = range[1];
					this.fireExpandRows(event, qx.Bootstrap.bind(this._expandToRowWithCallback1, this, event, callback));
				}
				else callback(success, "success");
			}
			else callback(success, "success");
		},

		_expandToRowWithCallback1 : function(event, callback, success, status)
		{
			if(success)
			{
				// 注意这里不能使用getRow来获取行，因为getRow里面还会触发expandToRow
				for(var i=event.beginIdxOfExpand;i<=event.endIdxOfExpand;++i)
				{
					this.rows.get(i).setChangeStatus(xlsys.base.dataset.DataSetRow.STATUS_COMMON);
				}
			}
			callback(success, "success");
		},

		isVirtualRow : function(rowOrRowIndex)
		{
			var row = null;
			if(qx.lang.Type.isNumber(rowOrRowIndex)) row = this.rows.get(rowOrRowIndex);
			else row = rowOrRowIndex;
			return row.getChangeStatus()==xlsys.base.dataset.DataSetRow.STATUS_VIRTUAL;
		},

		expandAllRows : function()
		{
			for(var i=0;i<this.getRowCount();i+=this.expandPerCount) this.expandToRow(i);
		},

		getCurRow : function()
		{
			var row = null;
			var cursor = this.getRowCursor();
			if(cursor>=0) row = this.getRow(cursor);
			return row;
		},

		getCurColumn : function()
		{
			var col = null;
			var cursor = this.getColCursor();
			if(cursor>=0) col = this.getColumn(cursor);
			return col;
		},

		beforeFirst : function()
		{
			return this.gotoRow(-1);
		},

		/**
		 * 需要根据传入参数的类型来重载方法
		 */
		gotoRow : function(rowOrRowIndex)
		{
			var rowIndex = null;
			if(qx.lang.Type.isNumber(rowOrRowIndex)) rowIndex = rowOrRowIndex;
			else rowIndex = this.getRowIndex(rowOrRowIndex);
			var postLastEditRow = true;
			if(arguments.length==2) postLastEditRow = arguments[1];
			return this.gotoCell(rowIndex, this.getColCursor(), postLastEditRow);
		},

		gotoColumn : function(colNameOrColIndex)
		{
			var colIndex = null;
			if(qx.lang.Type.isString(colNameOrColIndex)) colIndex = this.getColumnIndex(colNameOrColIndex);
			else colIndex = colNameOrColIndex;
			return this.gotoCell(this.getRowCursor(), colIndex, false);
		},

		gotoCell : function(rowOrRowIndex, colNameOrColIndex)
		{
			var rowIndex = null;
			var colIndex = null;
			var postLastEditRow = true;
			if(qx.lang.Type.isNumber(rowOrRowIndex)) rowIndex = rowOrRowIndex;
			else rowIndex = this.getRowIndex(rowOrRowIndex);
			if(qx.lang.Type.isString(colNameOrColIndex)) colIndex = this.getColumnIndex(colNameOrColIndex);
			else colIndex = colNameOrColIndex;
			var success = false;
			if((!this.rowRegionCheck(rowIndex)&&rowIndex!=-1)||(!this.colRegionCheck(colIndex)&&colIndex!=-1)) return false;
			if(rowIndex==this.getRowCursor()&&colIndex==this.getColCursor()) return true;
			var postSuccess = !postLastEditRow||rowIndex==this.getRowCursor();
			if(postSuccess||this.postRow())
			{
				var oldRowCursor = this.getRowCursor();
				var oldColCursor = this.getColCursor();
				var oldRow = this.getRow(oldRowCursor);
				var oldCol = this.getColumn(oldColCursor);
				var toRow = this.getRow(rowIndex);
				var toCol = this.getColumn(colIndex);
				var event = new xlsys.base.dataset.DataSetEvent(this);
				event.oldRow = oldRow;
				event.oldCol = oldCol;
				event.row = toRow;
				event.col = toCol;
				this.fireBeforeCursorChange(event);
				if(event.doit&&this.setRowCursor(rowIndex)&&this.setColCursor(colIndex))
				{
					success = true;
					event = new xlsys.base.dataset.DataSetEvent(this);
					event.oldRow = oldRow;
					event.oldCol = oldCol;
					event.row = toRow;
					event.col = toCol;
					this.fireAfterCursorChange(event);
				}
				else
				{
					this.setRowCursor(oldRowCursor);
					this.setColCursor(oldColCursor);
				}
			}
			return success;
		},

		getRow : function(rowIndex)
		{
			if(!this.rowRegionCheck(rowIndex)) return null;
			if(xlsys.base.util.ObjectUtil.hasBitProp(this.style, xlsys.base.dataset.IDataSet.STYLE_VIRTUAL))
			{
				if(!this.expandToRow(rowIndex)) return null;
			}
			return this.rows.get(rowIndex);
		},

		getRowIndex : function(row)
		{
			if(row==null) return -1;
			return this.rows.indexOf(row);
		},

		getColumn : function(colNameOrColIndex)
		{
			var colIndex = null;
			if(qx.lang.Type.isNumber(colNameOrColIndex)) colIndex = colNameOrColIndex;
			else colIndex =  this.getColumnIndex(colNameOrColIndex);
			if(!this.colRegionCheck(colIndex)) return null;
			return this.columns.get(colIndex);
		},

		getColumnName : function(columnIndex)
		{
			var colName = null;
			var dsc = this.getColumn(columnIndex);
			if(dsc!=null) colName = dsc.getColumnName();
			return colName;
		},

		rowRegionCheck : function(rowAt)
		{
			if(this.regionChecking) return true;
			this.regionChecking = true;
			var success = true;
			if(rowAt<0||rowAt>=this.getRowCount()) success = false;
			if(success) success = this.expandToRow(rowAt);
			this.regionChecking = false;
			return success;
		},

		colRegionCheck : function(colAt)
		{
			if(this.regionChecking) return true;
			this.regionChecking = true;
			var success = true;
			if(colAt<0||colAt>=this.getColumnCount()) success = false;
			this.regionChecking = false;
			return success;
		},

		regionCheck : function(rowAt, colAt)
		{
			return this.rowRegionCheck(rowAt)&&this.colRegionCheck(colAt);
		},

		insertRow : function(row)
		{
			var clone = false;
			if(arguments.length==2) clone = arguments[1];
			return this.insertRowAt(this.getRowCursor(), row, clone);
		},

		insertRowAtFirst : function(row)
		{
			var clone = false;
			if(arguments.length==2) clone = arguments[1];
			return this.insertRowAt(0, row, clone);
		},

		insertRowAfterLast : function(row)
		{
			var clone = false;
			if(arguments.length==2) clone = arguments[1];
			return this.insertRowAt(this.rows.size(), row, clone);
		},

		insertRowAt : function(index, row)
		{
			var clone = false;
			if(arguments.length==3) clone = arguments[2];
			var newRow = null;
			if(this.postRow())
			{
				newRow = this.doInsertRow(index, row, clone);
				if(this.gotoRow(newRow)) this.oldEditRow = newRow;
			}
			return newRow;
		},

		doInsertRow : function(index, row, clone)
		{
			var newRow = null;
			if(clone)
			{
				newRow = new xlsys.base.dataset.DataSetRow();
				for(var i=0;i<row.getCells().size();++i)
				{
					var cell = new xlsys.base.dataset.DataSetCell();
					newRow.addCell(cell);
				}
				newRow.cloneOf(row);
			}
			else newRow = row;
			if(index<0) index = 0;
			else if(index>this.rows.size()) index = this.rows.size();
			var event = new xlsys.base.dataset.DataSetEvent(this);
			event.row = newRow;
			this.fireBeforeInsertRow(event);
			this.rows.add(index, newRow);
			if(index<=this.getRowCursor()) this.setRowCursor(index+1);
			event = new xlsys.base.dataset.DataSetEvent(this);
			event.row = newRow;
			this.fireAfterInsertRow(event);
			return newRow;
		},

		insertNewRow : function()
		{
			var virtual = false;
			if(arguments.length==1) virtual = arguments[0]; 
			return this.insertNewRowAt(this.getRowCursor(), virtual);
		},

		insertNewRowAtFirst : function()
		{
			var virtual = false;
			if(arguments.length==1) virtual = arguments[0]; 
			return this.insertNewRowAt(0, virtual);
		},

		insertNewRowAfterLast : function()
		{
			var virtual = false;
			if(arguments.length==1) virtual = arguments[0]; 
			return this.insertNewRowAt(this.rows.size(), virtual);
		},

		insertNewRowAt : function(index)
		{
			var virtual = false;
			if(arguments.length==2) virtual = arguments[1];
			var newRow = null;
			if(this.postRow())
			{
				newRow = this.doInsertNewRow(index, virtual);
				if(!virtual)
				{
					if(this.gotoRow(newRow)) this.oldEditRow = newRow;
				}
			}
			return newRow;
		},

		doInsertNewRow : function(index, virtual)
		{
			if(index<0) index = 0;
			else if(index>this.rows.size()) index = this.rows.size();
			var newRow = new xlsys.base.dataset.DataSetRow(virtual);
			for(var i=0;i<this.columns.size();++i)
			{
				var cell = new xlsys.base.dataset.DataSetCell();
				newRow.addCell(cell);
			}
			var event = new xlsys.base.dataset.DataSetEvent(this);
			event.row = newRow;
			this.fireBeforeInsertRow(event);
			this.rows.add(index, newRow);
			if(index<=this.getRowCursor()) this.setRowCursor(index+1);
			event = new xlsys.base.dataset.DataSetEvent(this);
			event.row = newRow;
			this.fireAfterInsertRow(event);
			return newRow;
		},

		insertNewColumnAtFirst : function()
		{
			return this.insertNewColumnAt(0);
		},

		insertNewColumnAfterLast : function()
		{
			return this.insertNewColumnAt(this.columns.size());
		},

		insertNewColumn : function()
		{
			return this.insertNewColumnAt(this.getColCursor());
		},

		insertNewColumnAt : function(index)
		{
			var newColumn = null;
			if(this.postRow())
			{
				if(index<0) index = 0;
				else if(index>this.columns.size()) index = this.columns.size();
				newColumn = new xlsys.base.dataset.DataSetColumn();
				var event = new xlsys.base.dataset.DataSetEvent(this);
				event.col = newColumn;
				this.fireBeforeInsertCol(event);
				this.columns.add(index, newColumn);
				for(var i=0;i<this.rows.size();++i)
				{
					var cell = new xlsys.base.dataset.DataSetCell();
					var row = this.rows.get(i);
					row.getCells().add(index, cell);
				}
				if(index<=this.getColCursor()) this.setColCursor(index+1);
				event = new xlsys.base.dataset.DataSetEvent(this);
				event.col = newColumn;
				this.fireAfterInsertCol(event);
			}
			return newColumn;
		},

		removeRow : function(rowOrRowIndex)
		{
			var rowIndex = null;
			if(qx.lang.Type.isNumber(rowOrRowIndex)) rowIndex = rowOrRowIndex;
			else rowIndex = this.getRowIndex(rowOrRowIndex);
			if(!this.rowRegionCheck(rowIndex)||!this.expandToRow(rowIndex)) return null;
			
			var toRemoveRow = this.rows.get(rowIndex);
			var event = new xlsys.base.dataset.DataSetEvent(this);
			event.row = toRemoveRow;
			this.fireBeforeRemoveRow(event);
			if(event.doit)
			{
				this.doRemoveRow(rowIndex);
				event = new xlsys.base.dataset.DataSetEvent(this);
				event.row = toRemoveRow;
				this.fireAfterRemoveRow(event);
			}
			else toRemoveRow = null;
			if(toRemoveRow!=null)
			{
				if(this.oldEditRow==toRemoveRow) this.oldEditRow = null;
				var cursor = this.getRowCursor();
				if(cursor>rowIndex) this.setRowCursor(cursor-1);
				else if(cursor==rowIndex)
				{
					if(cursor>=this.rows.size()) this.gotoRow(cursor-1);
					else
					{
						var oldCursor = cursor;
						this.setRowCursor(-1);
						this.gotoRow(oldCursor);
					}
				}
			}
			return toRemoveRow;
		},

		doRemoveRow : function(rowNum)
		{
			var toRemoveRow = this.rows.get(rowNum);
			this.rows.removeAt(rowNum);
			return toRemoveRow;
		},

		removeAllRow : function()
		{
			for(var i=this.rows.size()-1;i>=0;--i) this.removeRow(i);
			this.truncateAllRow();
		},
		
		/**
		 * 快速删除所有的行, 不会触发任务事件
		 */
		truncateAllRow : function()
		{
			this.rows.clear();
			this.oldEditRow = null;
			this.setRowCursor(-1);
		},

		/**
		 * 需要支持多种参数传递方式
		 * getValue();
		 * getValue(String colName);
		 * getValue(int colIndex);
		 * getValue(int rowIndex, String columnName);
		 * getValue(DataSetRow row, String columnName);
		 * getValue(DataSetRow row, int colIndex);
		 * getValue(int rowIndex, int colIndex);
		 */
		getValue : function()
		{
			return this._getValue(arguments);
		},

		_getValue : function(args)
		{
			var rowIndex = null;
			var colIndex = null;
			if(args.length==0)
			{
				rowIndex = this.getRowCursor();
				colIndex = this.getColCursor();
			}
			else if(args.length==1)
			{
				// 一个参数
				rowIndex = this.getRowCursor();
				if(qx.lang.Type.isNumber(args[0])) colIndex = args[0];
				else colIndex = this.getColumnIndex(args[0]);
			}
			else if(args.length>=2)
			{
				// 两个参数
				if(qx.lang.Type.isNumber(args[0])) rowIndex = args[0];
				else rowIndex = this.getRowIndex(args[0]);
				if(qx.lang.Type.isNumber(args[1])) colIndex = args[1];
				else colIndex = this.getColumnIndex(args[1]);
			}
			if(!this.regionCheck(rowIndex, colIndex)) return null;
			this.expandToRow(rowIndex);
			var value = null;
			var row = this.rows.get(rowIndex);
			if(row!=null)
			{
				var cell = row.getCells().get(colIndex);
				if(cell!=null) value = cell.getContent();
			}
			return value;
		},

		/**
		 * 需要支持多种参数传递方式
		 * setValue(Serializable value);
		 * setValue(String columnName, Serializable value);
		 * setValue(int colIndex, Serializable value);
		 * setValue(int rowIndex, String columnName, Serializable value);
		 * setValue(DataSetRow row, String columnName, Serializable value);
		 * setValue(DataSetRow row, int colIndex, Serializable value);
		 * setValue(int rowIndex, int colIndex, Serializable value);
		 */
		setValue : function()
		{
			var rowIndex = null;
			var colIndex = null;
			var value = null;
			if(arguments.length==1)
			{
				rowIndex = this.getRowCursor();
				colIndex = this.getColCursor();
				value = arguments[0];
			}
			else if(arguments.length==2)
			{
				// 两个参数
				rowIndex = this.getRowCursor();
				if(qx.lang.Type.isNumber(arguments[0])) colIndex = arguments[0];
				else colIndex = this.getColumnIndex(arguments[0]);
				value = arguments[1];
			}
			else if(arguments.length>=3)
			{
				// 三个参数
				if(qx.lang.Type.isNumber(arguments[0])) rowIndex = arguments[0];
				else rowIndex = this.getRowIndex(arguments[0]);
				if(qx.lang.Type.isNumber(arguments[1])) colIndex = arguments[1];
				else colIndex = this.getColumnIndex(arguments[1]);
				value = arguments[2];
			}
			var success = false;
			if(!this.regionCheck(rowIndex, colIndex)) return success;
			var row = this.rows.get(rowIndex);
			if(row!=null)
			{
				var cell = row.getCells().get(colIndex);
				if(cell!=null)
				{
					var oldValue = cell.getContent();
					var needSet = false;
					if(xlsys.base.util.ObjectUtil.hashcode(value)!=xlsys.base.util.ObjectUtil.hashcode(oldValue)) needSet = true;
					if(needSet)
					{
						var event = new xlsys.base.dataset.DataSetEvent(this);
						event.oldValue = oldValue;
						event.newValue = value;
						event.row = row;
						event.col = this.getColumn(colIndex);
						// 当旧值和新值不一样时，设置值
						var postRowSuccess = true;
						if(this.oldEditRow!=row)
						{
							// 当设置值的行和之前编辑的行不一致时，先提交之前编辑的行
							postRowSuccess = this.postRow();
						}
						if(postRowSuccess)
						{
							this.fireBeforePostValue(event);
							if(event.doit)
							{
								this.doSetValue(rowIndex, colIndex, event.newValue);
								this.oldEditRow = row;
								success = true;
								this.fireAfterPostValue(event);
							}
						}
					}
				}
				else
				{
					var event = new xlsys.base.dataset.DataSetEvent(this);
					event.errMsg = new xlsys.java.lang.Exception("Cell is not initialized. Row At : "+rowIndex+"     Column At : "+colIndex);
					event.logLevel = xlsys.base.log.Log.LOG_LEVEL_ERROR;
					this.fireShowMessage(event);
				}
			}
			return success;
		},

		doSetValue : function(rowIndex, colIndex, value)
		{
			var cell = this.rows.get(rowIndex).getCells().get(colIndex);
			cell.setContent(value);
		},

		postRow : function()
		{
			var rowIndex = null;
			if(arguments.length==0) rowIndex = this.getRowIndex(this.oldEditRow);
			else rowIndex = arguments[0];
			var success = false;
			if(!this.rowRegionCheck(rowIndex)) return true;
			var event = new xlsys.base.dataset.DataSetEvent(this);
			var row = this.getRow(rowIndex);
			event.row = row;
			this.fireBeforePostRow(event);
			if(event.doit)
			{
				if(row==this.oldEditRow) this.oldEditRow = null;
				success = true;
				event = new xlsys.base.dataset.DataSetEvent(this);
				event.row = row;
				this.fireAfterPostRow(event);
			}
			return success;
		},

		setOption : function(option, value)
		{
			var useCount = false;
			if(arguments.length==3) useCount = arguments[2];
			if(useCount)
			{
				var oldCount = this.optionMap.get(option);
				if(oldCount==null) oldCount = 0;
				if(value) this.optionMap.put(option, ++oldCount);
				else
				{
					this.optionMap.put(option, --oldCount);
					if(oldCount<=0) this.optionMap.remove(option);
				}
			}
			else
			{
				if(value) this.optionMap.put(option, 0);
				else this.optionMap.remove(option);
			}
		},

		isOption : function(option)
		{
			return this.optionMap.containsKey(option);
		},

		refDataSet : function(toRefDataSet)
		{
			var refRows = toRefDataSet.getRows();
			var refColumns = toRefDataSet.getColumns();
			if(xlsys.base.util.ObjectUtil.hasBitProp(this.style, xlsys.base.dataset.IDataSet.STYLE_VIRTUAL))
			{
				this.setRowCount(refRows.size());
				this.setColumnCount(refColumns.size());
				for(var i=0;i<refRows.size();++i)
				{
					this.rows.get(i).cloneOf(refRows.get(i));
				}
				for(var i=0;i<refColumns.size();++i)
				{
					this.columns.get(i).cloneOf(refColumns.get(i));
				}
			}
			else
			{
				this.rows = refRows;
				this.columns = refColumns;
			}
			this.buildColumnMap();
		},

		setSortColumn : function()
		{
			this._setSortColumn(arguments);
		},

		_setSortColumn : function(args)
		{
			if(args.length==1&&qx.Class.implementsInterface(args[0], xlsys.java.util.Map))
			{
				this.setSortColMap(args[0]);
			}
			else
			{
				var colNames = args[0];
				var sortTypes = null;
				if(args.length==2) sortTypes = args[1];
				var sortColMap = new xlsys.java.util.LinkedHashMap();
				for(var i=0;i<colNames.length;++i)
				{
					var sortType = xlsys.base.dataset.IDataSet.SORT_TYPE_ASC;
					if(sortTypes!=null&&i<sortTypes.length) sortType = sortTypes[i];
					sortColMap.put(colNames[i], sortType);
				}
				this.setSortColMap(sortColMap);
			}
		},

		setSortColMap : function(sortColMap)
		{
			this.sortColMap.clear();
			this.sortColMap.putAll(sortColMap);
		},

		sort : function()
		{
			this.expandAllRows();
			var sortColIdxMap = new xlsys.java.util.LinkedHashMap();
			var sortCols = this.sortColMap.keys();
			for(var i=0;i<sortCols.length;++i)
			{
				var sortColName = sortCols[i];
				var colIdx = this.getColumnIndex(sortColName);
				if(colIdx!=-1)
				{
					var sortType = this.sortColMap.get(sortColName);
					sortColIdxMap.put(colIdx, sortType);
				}
			}
			var rowArray = this.rows.toArray();
			rowArray.sort(qx.Bootstrap.bind(this._sortRow, this, sortColIdxMap));
			// rebuilt rows and cols
			var curRow = this.getCurRow();
			this.setEventLock(true);
			this.rows.clear();
			for(var i=0;i<rowArray.length;++i)
			{
				this.rows.add(rowArray[i]);
			}
			// redirect row cursor
			this.setRowCursor(this.getRowIndex(curRow));
			this.setEventLock(false);
		},

		_sortRow : function(sortColIdxMap, row1, row2)
		{
			var result = 0;
			var toSortColIndices = sortColIdxMap.keys();
			for(var i=0;i<toSortColIndices.length;++i)
			{
				var colIdx = toSortColIndices[i];
				var sortType = sortColIdxMap.get(colIdx);
				var column = this.getColumn(colIdx);
				var content = row1.getCell(colIdx).content;
				var oContent = row2.getCell(colIdx).content;
				if(content==null&&oContent==null) result = 0;
				else if(content!=null&&oContent==null) result = (sortType==xlsys.base.dataset.IDataSet.SORT_TYPE_ASC?1:-1);
				else if(content==null&&oContent!=null) result = (sortType==xlsys.base.dataset.IDataSet.SORT_TYPE_ASC?-1:1);
				else
				{
					if(column.javaClass=="java.lang.String")
					{
						result = content.localeCompare(oContent);
					}
					else if(column.javaClass=="java.math.BigDecimal")
					{
						if(content<oContent) result = -1;
						else if(content>oContent) result = 1;
						else result = 0;
					}
					else if(column.javaClass=="java.util.Date")
					{
						var time1 = content.getTime();
						var time2 = oContent.getTime();
						if(time1<time2) result = -1;
						else if(time1>time2) result = 1;
						else result = 0;
					}
					else if(column.javaClass=="[B")
					{
						var byteStr1 = content.toString();
						var byteStr2 = oContent.toString();
						result = byteStr1.localeCompare(byteStr2);
					}
					else
					{
						if(content<oContent) result = -1;
						else if(content>oContent) result = 1;
						else result = 0;
					}
					result = (sortType==xlsys.base.dataset.IDataSet.SORT_TYPE_ASC?result:result*-1);
				}
				if(result!=0) break;
			}
			return result;
		},

		swapRow : function(i, j)
		{
			if(i==j) return;
			var min = i<j?i:j;
			var max = i<j?j:i;
			this.setEventLock(true);
			// 先删除大的行
			var maxRow = this.removeRow(max);
			// 再删除小的行
			var minRow = this.removeRow(min);
			// 先插入大行到小的序号中
			this.insertRowAt(min, maxRow, false);
			// 再插入小行到大的序号中
			this.insertRowAt(max, minRow, false);
			this.setEventLock(false);
		},

		addDataSetListener : function(dataSetListener)
		{
			if(this.dataSetListenerSet==null) this.dataSetListenerSet = new xlsys.java.util.LinkedHashSet();
			this.dataSetListenerSet.add(dataSetListener);
			var event = new xlsys.base.dataset.DataSetEvent(this);
			this.fireAfterAddToDataSet(dataSetListener, event);
		},

		removeDataSetListener : function(dataSetListener)
		{
			if(this.dataSetListenerSet!=null)
			{
				if(this.dataSetListenerSet.remove(dataSetListener))
				{
					var event = new xlsys.base.dataset.DataSetEvent(this);
					this.fireBeforeRemoveFromDataSet(dataSetListener, event);
				}
			}
		},

		getDataSetListeners : function()
		{
			return this.dataSetListenerSet;
		},

		refresh : function()
		{
			var event = new xlsys.base.dataset.DataSetEvent(this);
			this.fireBeforeRefresh(event);
			this.refreshing();
			event = new xlsys.base.dataset.DataSetEvent(this);
			this.fireAfterRefresh(event);
		},

		refreshing : function()
		{
			// Do nothing in this class
		},

		setEventLock : function()
		{
			var dataSet = this;
			var lock = false;
			if(arguments.length==1) lock = arguments[0];
			else
			{
				dataSet = arguments[0];
				lock = arguments[1];
			}
			var thisClass = xlsys.base.util.ObjectUtil.getClass(this);
			var staticFieldMap = xlsys.base.util.ReflectUtil.getAllStaticFieldMap(thisClass);
			var staticFields = staticFieldMap.keys();
			for(var i=0;i<staticFields.length;++i)
			{
				var fieldName = staticFields[i];
				if(qx.lang.String.startsWith(fieldName, "OPTION_LOCK_"))
				{
					this.setOption(staticFieldMap.get(fieldName), lock, true);
				}
			}
		},
		
		fireBeforeInsertRow : function(event)
		{
			if(this.isOption(xlsys.base.dataset.IDataSet.OPTION_LOCK_INSERT_ROW_EVENT)) return;
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						listener.beforeInsertRow(event);
						if(event.interrupt) break;
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		fireAfterInsertRow : function(event)
		{
			if(this.isOption(xlsys.base.dataset.IDataSet.OPTION_LOCK_INSERT_ROW_EVENT)) return;
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						listener.afterInsertRow(event);
						if(event.interrupt) break;
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},
		
		fireBeforeInsertCol : function(event)
		{
			if(this.isOption(xlsys.base.dataset.IDataSet.OPTION_LOCK_INSERT_COL_EVENT)) return;
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						listener.beforeInsertCol(event);
						if(event.interrupt) break;
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		fireAfterInsertCol : function(event)
		{
			if(this.isOption(xlsys.base.dataset.IDataSet.OPTION_LOCK_INSERT_COL_EVENT)) return;
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						listener.afterInsertCol(event);
						if(event.interrupt) break;
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		fireBeforeRemoveRow : function(event)
		{
			if(this.isOption(xlsys.base.dataset.IDataSet.OPTION_LOCK_REMOVE_ROW_EVENT)) return;
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						listener.beforeRemoveRow(event);
						if(event.interrupt) break;
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		fireAfterRemoveRow : function(event)
		{
			if(this.isOption(xlsys.base.dataset.IDataSet.OPTION_LOCK_REMOVE_ROW_EVENT)) return;
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						listener.afterRemoveRow(event);
						if(event.interrupt) break;
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		fireBeforeRemoveCol : function(event)
		{
			if(this.isOption(xlsys.base.dataset.IDataSet.OPTION_LOCK_REMOVE_COL_EVENT)) return;
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						listener.beforeRemoveCol(event);
						if(event.interrupt) break;
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		fireAfterRemoveCol : function(event)
		{
			if(this.isOption(xlsys.base.dataset.IDataSet.OPTION_LOCK_REMOVE_COL_EVENT)) return;
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						listener.afterRemoveCol(event);
						if(event.interrupt) break;
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		fireBeforeRefresh : function(event)
		{
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						listener.beforeRefresh(event);
						if(event.interrupt) break;
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		fireAfterRefresh : function(event)
		{
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						listener.afterRefresh(event);
						if(event.interrupt) break;
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		fireAfterAddToDataSet : function(dataSetListener, event)
		{
			try
			{
				if(dataSetListener!=null)
				{
					dataSetListener.afterAddToDataSet(event);
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		fireBeforeRemoveFromDataSet : function(dataSetListener, event)
		{
			try
			{
				if(dataSetListener!=null)
				{
					dataSetListener.beforeRemoveFromDataSet(event);
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		fireShowMessage : function(event)
		{
			if(this.dataSetListenerSet!=null)
			{
				var listeners = this.dataSetListenerSet.toArray();
				for(var i=0;i<listeners.length;++i)
				{
					var listener = listeners[i];
					listener.showMessage(event);
					if(event.interrupt) break;
				}
			}
		},

		fireBeforePostValue : function(event)
		{
			if(this.isOption(xlsys.base.dataset.IDataSet.OPTION_LOCK_POST_VALUE_EVENT)) return;
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						listener.beforePostValue(event);
						if(event.interrupt) break;
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		fireAfterPostValue : function(event)
		{
			if(this.isOption(xlsys.base.dataset.IDataSet.OPTION_LOCK_POST_VALUE_EVENT)) return;
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						listener.afterPostValue(event);
						if(event.interrupt) break;
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		fireBeforePostRow : function(event)
		{
			if(this.isOption(xlsys.base.dataset.IDataSet.OPTION_LOCK_POST_ROW_EVENT)) return;
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						listener.beforePostRow(event);
						if(event.interrupt) break;
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		fireAfterPostRow : function(event)
		{
			if(this.isOption(xlsys.base.dataset.IDataSet.OPTION_LOCK_POST_ROW_EVENT)) return;
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						listener.afterPostRow(event);
						if(event.interrupt) break;
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		fireBeforeCursorChange : function(event)
		{
			if(this.isOption(xlsys.base.dataset.IDataSet.OPTION_LOCK_CURSOR_CHANGE)) return;
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						listener.beforeCursorChange(event);
						if(event.interrupt) break;
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		fireAfterCursorChange : function(event)
		{
			if(this.isOption(xlsys.base.dataset.IDataSet.OPTION_LOCK_CURSOR_CHANGE)) return;
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						listener.afterCursorChange(event);
						if(event.interrupt) break;
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		fireExpandRows : function(event)
		{
			if(arguments.length==1) return this._fireExpandRows(event);
			else this._fireExpandRowsWithCallback(event, arguments[1]);
		},

		_fireExpandRows : function(event)
		{
			try
			{
				this.setEventLock(true);
				var oldRowCursor = this.getRowCursor();
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						listener.expandRows(event);
						if(event.interrupt) break;
					}
				}
				this.setRowCursor(oldRowCursor);
				this.setEventLock(false);
				if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
			return event.doit;
		},

		_fireExpandRowsWithCallback : function(event, callback)
		{
			this.setEventLock(true);
			var oldRowCursor = this.getRowCursor();
			if(this.dataSetListenerSet!=null)
			{
				var listeners = this.dataSetListenerSet.toArray();
				if(listeners.length>0)
				{
					var listener = listeners[0];
					listener.expandRows(event, qx.Bootstrap.bind(this._expandCallBack1, this, callback, event, oldRowCursor, listeners, 0));
				}
			}
			else this._expandCallback2(callback, event, oldRowCursor);
		},

		_expandCallBack1 : function(callback, event, oldRowCursor, listeners, idx, data, status)
		{
			var finish = false;
			if(!event.interrupt)
			{
				var nextIdx = idx + 1;
				if(nextIdx<listeners.length)
				{
					var listener = listeners[nextIdx];
					listener.expandRows(event, qx.Bootstrap.bind(this._expandCallBack1, this, callback, event, oldRowCursor, listeners, nextIdx));
				}
				else finish = true;
			}
			else finish = true;
			if(finish) this._expandCallback2(callback, event, oldRowCursor);
		},

		_expandCallback2 : function(callback, event, oldRowCursor)
		{
			this.setRowCursor(oldRowCursor);
			this.setEventLock(false);
			if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
			callback(event.doit, "success");
		},

		fireCustomTrigger : function(event)
		{
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						listener.customTrigger(event);
						if(event.interrupt) break;
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		getColumns : function()
		{
			return this.columns;
		},

		getRows : function()
		{
			return this.rows;
		},

		getName : function()
		{
			return this.name;
		},

		setName : function(name)
		{
			this.name = name;
		},

		getStyle : function()
		{
			return this.style;
		},

		getSortColMap : function()
		{
			return this.sortColMap;
		}
	}
});
	