/**
 * 数据集接口
 */
qx.Interface.define("xlsys.base.dataset.IDataSet",
{
	statics :
	{
		STYLE_NONE : 0,
		STYLE_VIRTUAL : 1,
		SORT_TYPE_ASC : 0,
		SORT_TYPE_DESC : 1,
		OPTION_LOCK_POST_VALUE_EVENT : 0,
		OPTION_LOCK_POST_ROW_EVENT : 1,
		OPTION_LOCK_CURSOR_CHANGE : 2,
		OPTION_LOCK_REMOVE_ROW_EVENT : 3,
		OPTION_LOCK_REMOVE_COL_EVENT : 4,
		OPTION_LOCK_INSERT_ROW_EVENT : 5,
		OPTION_LOCK_INSERT_COL_EVENT : 6
	},
	
	members:
	{
		setRowCount : function(rowCount) {this.assertArgumentsCount(arguments, 1, 1);},

		getStyle : function() {},

		getExpandPerCount : function() {},

		setExpandPerCount : function(expandPerCount) {this.assertArgumentsCount(arguments, 1, 1);},

		getPageRangeOfRowIndex : function(wantRowIndex) {this.assertArgumentsCount(arguments, 1, 1);},

		/**
		 * 需重载以下方法
		 * expandToRow(wantRowIndex);
		 * expandToRow(wantRowIndex, callback);
		 */
		expandToRow : function(wantRowIndex) {this.assertArgumentsCount(arguments, 1, 2);},

		isVirtualRow : function(rowOrRowIndex) {this.assertArgumentsCount(arguments, 1, 1);},

		buildColumnMap : function() {},

		getColumnIndex : function(columnName) {this.assertArgumentsCount(arguments, 1, 1);},

		removeColumn : function(colNameOrColIdx) {this.assertArgumentsCount(arguments, 1, 1);},

		getRowCount : function() {},

		getColumnCount : function() {},

		getRowCursor : function() {},

		getColCursor : function() {},

		getCurRow : function() {},

		getCurColumn : function() {},

		beforeFirst : function() {},

		/**
		 * 需要根据传入参数的类型来重载方法
		 * gotoRow(rowOrRowIndex)
		 * gotoRow(rowOrRowIndex, boolean postLastEditRow)
		 */
		gotoRow : function() {this.assertArgumentsCount(arguments, 1, 2);},

		gotoColumn : function(colNameOrColIndex) {this.assertArgumentsCount(arguments, 1, 1);},

		/**
		 * 需重载以下方法
		 * gotoCell(rowOrRowIndex, colNameOrColIndex);
		 * gotoCell(rowOrRowIndex, colNameOrColIndex, boolean postLastEditRow);
		 */
		gotoCell : function(rowOrRowIndex, colNameOrColIndex) {this.assertArgumentsCount(arguments, 2, 3);},

		getRow : function(rowIndex) {this.assertArgumentsCount(arguments, 1, 1);},

		getRowIndex : function(row) {this.assertArgumentsCount(arguments, 1, 1);},

		/**
		 * 需要根据传入参数的类型来重载方法
		 */
		getColumn : function(colNameOrColIndex) {this.assertArgumentsCount(arguments, 1, 1);},

		getColumnName : function(colIndex) {this.assertArgumentsCount(arguments, 1, 1);},

		/**
		 * 需重载以下方法
		 * insertRow(DataSetRow row);
		 * insertRow(DataSetRow row, boolean clone);
		 */
		insertRow : function(row) {this.assertArgumentsCount(arguments, 1, 2);},

		/**
		 * 需重载以下方法
		 * insertRowAtFirst(DataSetRow row);
		 * insertRowAtFirst(DataSetRow row, boolean clone);
		 */
		insertRowAtFirst : function(row) {this.assertArgumentsCount(arguments, 1, 2);},

		/**
		 * 需重载以下方法
		 * insertRowAfterLast(DataSetRow row);
		 * insertRowAfterLast(DataSetRow row, boolean clone);
		 */
		insertRowAfterLast : function(row) {this.assertArgumentsCount(arguments, 1, 2);},

		/**
		 * 需重载以下方法
		 * insertRowAt(int index, DataSetRow row);
		 * insertRowAt(int index, DataSetRow row, boolean clone);
		 */
		insertRowAt : function(index, row) {this.assertArgumentsCount(arguments, 2, 3);},

		/**
		 * 需重载以下方法
		 * insertNewRow();
		 * insertNewRow(boolean virtual);
		 */
		insertNewRow : function() {this.assertArgumentsCount(arguments, 0, 1);},

		/**
		 * 需重载以下方法
		 * insertNewRowAtFirst();
		 * insertNewRowAtFirst(boolean virtual);
		 */
		insertNewRowAtFirst : function() {this.assertArgumentsCount(arguments, 0, 1);},

		/**
		 * 需重载以下方法
		 * insertNewRowAfterLast();
		 * insertNewRowAfterLast(boolean virtual);
		 */
		insertNewRowAfterLast : function() {this.assertArgumentsCount(arguments, 0, 1);},

		/**
		 * 需重载以下方法
		 * insertNewRowAt(int index);
		 * insertNewRowAt(int index, boolean virtual);
		 */
		insertNewRowAt : function(index) {this.assertArgumentsCount(arguments, 1, 2);},

		insertNewColumnAtFirst : function() {},

		insertNewColumnAfterLast : function() {},

		insertNewColumn : function() {},

		insertNewColumnAt : function(index) {this.assertArgumentsCount(arguments, 1, 1);},

		removeRow : function(rowOrRowIndex) {this.assertArgumentsCount(arguments, 1, 1);},

		removeAllRow : function() {},
		
		truncateAllRow : function() {},

		/**
		 * 需要支持多种参数传递方式
		 * getValue();
		 * getValue(String colName);
		 * getValue(int colIndex);
		 * getValue(int rowIndex, String columnName);
		 * getValue(DataSetRow row, String columnName);
		 * getValue(DataSetRow row, int colIndex);
		 * getValue(int rowIndex, int colIndex);
		 */
		getValue : function() {this.assertArgumentsCount(arguments, 0, 2);},

		/**
		 * 需要支持多种参数传递方式
		 * setValue(Serializable value);
		 * setValue(String columnName, Serializable value);
		 * setValue(int colIndex, Serializable value);
		 * setValue(int rowIndex, String columnName, Serializable value);
		 * setValue(DataSetRow row, String columnName, Serializable value);
		 * setValue(DataSetRow row, int colIndex, Serializable value);
		 * setValue(int rowIndex, int colIndex, Serializable value);
		 */
		setValue : function() {this.assertArgumentsCount(arguments, 1, 3);},

		/**
		 * 需重载以下方法
		 * postRow();
		 * postRow(int rowIndex);
		 */
		postRow : function() {this.assertArgumentsCount(arguments, 0, 1);},

		/**
		 * 需重载以下方法
		 * setOption(option, value);
		 * setOption(option, value, useCount);
		 */
		setOption : function(option, value) {this.assertArgumentsCount(arguments, 2, 3);},

		isOption : function(option) {this.assertArgumentsCount(arguments, 1, 1);},

		/**
		 * 需重载以下方法
		 * setSortColumn(String[] colNames);
		 * setSortColumn(String[] colNames, int[] sortTypes);
		 * setSortColumn(Map<String, Integer> sortColMap);
		 */
		setSortColumn : function() {this.assertArgumentsCount(arguments, 1, 2);},

		setSortColMap : function(sortColMap) {this.assertArgumentsCount(arguments, 1, 1);},

		getSortColMap : function() {},

		sort : function() {},

		swapRow : function(i, j) {this.assertArgumentsCount(arguments, 2, 2);},

		addDataSetListener : function(dataSetListener) {this.assertArgumentsCount(arguments, 1, 1);},

		removeDataSetListener : function(dataSetListener) {this.assertArgumentsCount(arguments, 1, 1);},

		getDataSetListeners : function() {},

		/**
		 * 需重载以下方法
		 * setEventLock(boolean lock);
		 * setEventLock(dataSet, boolean lock);
		 */
		setEventLock : function() {this.assertArgumentsCount(arguments, 1, 2);},

		fireShowMessage : function(event) {this.assertArgumentsCount(arguments, 1, 1);},

		fireBeforePostValue : function(event) {this.assertArgumentsCount(arguments, 1, 1);},

		fireAfterPostValue : function(event) {this.assertArgumentsCount(arguments, 1, 1);},

		fireBeforePostRow : function(event) {this.assertArgumentsCount(arguments, 1, 1);},

		fireAfterPostRow : function(event) {this.assertArgumentsCount(arguments, 1, 1);},

		fireBeforeCursorChange : function(event) {this.assertArgumentsCount(arguments, 1, 1);},

		fireAfterCursorChange : function(event) {this.assertArgumentsCount(arguments, 1, 1);},

		fireCustomTrigger : function(event) {this.assertArgumentsCount(arguments, 1, 1);},

		getColumns : function() {},

		getRows : function() {},

		getName : function() {},

		setName : function(name) {this.assertArgumentsCount(arguments, 1, 1);},

		refresh : function() {}
	}
});