/**
 * 可存储数据集.
 */
qx.Class.define("xlsys.base.dataset.StorableDataSet",
{
	extend: xlsys.base.dataset.DataSet,
	implement : [xlsys.base.dataset.IStorableDataSet],
	
	/**
	 * 需要重载以下方法:
	 * StorableDataSet(IDataBase dataBase, String tableName)
	 * StorableDataSet(IDataBase dataBase, String tableName, int style)
	 * StorableDataSet(IDataBase dataBase, String selectSql, String tableName)
	 * StorableDataSet(IDataBase dataBase, String selectSql, String tableName, int style)
	 * StorableDataSet(IDataBase dataBase, ParamBean selectBean, String tableName)
	 * StorableDataSet(IDataBase dataBase, ParamBean selectBean, String tableName, int style)
	 */
	construct: function()
	{
		this.base(arguments);

		this.dataBase = arguments[0];
		this.tableName = null;
		this.unSaveColNameSet = null;
		this.toBeDeleteRowList = null;
		this.changed = false;
		this.selectBean = null;
		this.sorted = null;
		this.saveTransaction = true;
		
		this.style = xlsys.base.dataset.IDataSet.STYLE_NONE;
		if(arguments.length==2)
		{
			this.tableName = arguments[1];
			var sql = "select * from " + this.tableName;
			this.selectBean = new xlsys.base.database.bean.ParamBean(sql);
		}
		else if(arguments.length==3)
		{
			if(arguments[1] instanceof xlsys.base.database.bean.ParamBean)
			{
				this.selectBean = arguments[1];
				this.tableName = arguments[2];
			}
			else if(qx.lang.Type.isNumber(arguments[2]))
			{
				this.tableName = arguments[1];
				this.selectBean = new xlsys.base.database.bean.ParamBean(this.getTableSelectSql(this.tableName));
				this.style = arguments[2];
			}
			else if(qx.lang.Type.isString(arguments[2]))
			{
				this.selectBean = arguments[1];
				this.tableName = arguments[2];
			}
		}
		else if(arguments.length==4)
		{
			if(qx.lang.Type.isString(arguments[1]))
			{
				this.selectBean = new xlsys.base.database.bean.ParamBean(arguments[1]);
				this.tableName = arguments[2];
				this.style = arguments[3];
			}
			else
			{
				this.selectBean = arguments[1];
				this.tableName = arguments[2];
				this.style = arguments[3];
			}
		}
		this.changed = false;
		this.sorted = false;
		this.saveTransaction = true;
		this.addExpandListener();
	},
	
	members :
	{
		dataBase : null,
		tableName : null,
		unSaveColNameSet : null,
		toBeDeleteRowList : null,
		changed : null,
		selectBean : null,
		sorted : null,
		saveTransaction : true,
		
		getSerializableMembers : function()
		{
			var arr = this.base(arguments);
			arr.push("tableName");
			arr.push("unSaveColNameSet");
			arr.push("toBeDeleteRowList");
			arr.push("changed");
			arr.push("selectBean");
			arr.push("sorted");
			return arr;
		},
		
		getTableSelectSql : function(tableName)
		{
			var tableInfo = this.dataBase.getTableInfo(tableName);
			var sb = new xlsys.java.lang.StringBuilder("select ");
			var dscList = tableInfo.getDataColumnList();
			for(var i=0;i<dscList.size();++i)
			{
				var dsCol = dscList.get(i);
				if(i!=0) sb.append(",");
				sb.append(dsCol.getColumnName());
			}
			sb.append(" from ").append(tableName);
			return sb.toString();
		},

		setSaveTransaction : function(saveTransaction)
		{
			this.saveTransaction = saveTransaction;
		},

		isSorted : function()
		{
			return this.sorted;
		},

		isSaveTransaction : function()
		{
			return this.saveTransaction;
		},

		addExpandListener : function()
		{
			if(xlsys.base.util.ObjectUtil.hasBitProp(this.getStyle(), xlsys.base.dataset.IDataSet.STYLE_VIRTUAL))
			{
				this.addDataSetListener(new xlsys.base.dataset.SdsExpandListener());
			}
		},

		getSql : function()
		{
			return this.selectBean.getSelectSql();
		},

		getSelectBean : function()
		{
			return this.selectBean;
		},

		save : function()
		{
			var event = new xlsys.base.dataset.StorableDataSetEvent(this);
			this.firePrepareSave(event);
			if(arguments.length==0) return this._save();
			else if(arguments.length==1) this._saveWithCallBack(arguments[0]);
		},

		_initInsertOrUpdateSqlBean : function()
		{
			var insertEb = null;
			var updateEb = null;
			var rows = this.getRows();
			var cols = this.getColumns();
			var rowCount = rows.size();
			var colCount = cols.size();
			for(var i=0;i<rowCount;++i)
			{
				var row = rows.get(i);
				if(row.isChanged())
				{
					if(row.getChangeStatus()==xlsys.base.dataset.DataSetRow.STATUS_REPLACE)
					{
						var updateDataMap = new xlsys.java.util.HashMap();
						var oldDataMap = new xlsys.java.util.HashMap();
						for(var j=0;j<colCount;++j)
						{
							var dsc = cols.get(j);
							var colName = dsc.getColumnName();
							if(this.unSaveColNameSet==null||!this.unSaveColNameSet.contains(colName))
							{
								updateDataMap.put(colName, this.getValue(row, colName));
							}
							if(dsc.isPrimaryKey())
							{
								var oldValue = row.getCell(this.getColumnIndex(colName)).getOldContent();
								if(oldValue==null)
								{
									oldValue = row.getCell(this.getColumnIndex(colName)).getContent();
								}
								oldDataMap.put(colName, oldValue);
							}
						}
						if(updateEb==null)
						{
							updateEb = new xlsys.base.database.bean.ExecuteBean(xlsys.base.database.bean.ExecuteBean.EXECUTE_TYPE_UPDATE, this.tableName);
						}
						if(!updateDataMap.isEmpty()) updateEb.addData(updateDataMap, oldDataMap);
					}
					else
					{
						var dataMap = new xlsys.java.util.HashMap();
						for(var j=0;j<colCount;++j)
						{
							var dsc = cols.get(j);
							var colName = dsc.getColumnName();
							if(this.unSaveColNameSet==null||!this.unSaveColNameSet.contains(colName))
							{
								dataMap.put(colName, row.getCell(this.getColumnIndex(colName)).getContent());
							}
						}
						if(row.getChangeStatus()==xlsys.base.dataset.DataSetRow.STATUS_FOR_NEW)
						{
							if(insertEb==null)
							{
								insertEb = new xlsys.base.database.bean.ExecuteBean(xlsys.base.database.bean.ExecuteBean.EXECUTE_TYPE_INSERT, this.tableName);
							}
							if(!dataMap.isEmpty()) insertEb.addData(dataMap);
						}
						else
						{
							if(updateEb==null)
							{
								updateEb = new xlsys.base.database.bean.ExecuteBean(xlsys.base.database.bean.ExecuteBean.EXECUTE_TYPE_UPDATE, this.tableName);
							}
							if(!dataMap.isEmpty()) updateEb.addData(dataMap);
						}
					}
				}
			}
			var ret = new Array();
			ret.push(insertEb);
			ret.push(updateEb);
			return ret;
		},

		_save : function()
		{
			var success = false;
			var oldAutoCommit = true;
			try
			{
				oldAutoCommit = this.dataBase.getAutoCommit();
				// 启动事务处理
				if(this.saveTransaction) this.dataBase.setAutoCommit(false);
				if(this.postRow())
				{
					if(this.changed)
					{
						var event = new xlsys.base.dataset.StorableDataSetEvent(this);
						this.fireBeforeSave(event);
						if(event.doit)
						{
							var sqlBeanList = new xlsys.java.util.ArrayList();
							var tempArr = this._initInsertOrUpdateSqlBean();
							var insertEb = tempArr[0];
							var updateEb = tempArr[1];
							var deleteEb = null;
							// 处理删除的列
							if(this.toBeDeleteRowList!=null)
							{
								var tableInfo = this.dataBase.getTableInfo(this.tableName);
								var tempSize = this.toBeDeleteRowList.size();
								var pkColArray = tableInfo.getPkColSet().toArray();
								for(var i=0;i<tempSize;++i)
								{
									var row = this.toBeDeleteRowList.get(i);
									if(deleteEb==null)
									{
										deleteEb = new xlsys.base.database.bean.ExecuteBean(xlsys.base.database.bean.ExecuteBean.EXECUTE_TYPE_DELETE, this.tableName);
									}
									var dataMap = new xlsys.java.util.HashMap();
									for(var j in pkColArray)
									{
										var colName = pkColArray[j];
										dataMap.put(colName, row.getCell(this.getColumnIndex(colName)).getContent());
									}
									deleteEb.addData(dataMap);
								}
							}
							if(insertEb!=null) sqlBeanList.add(insertEb);
							if(updateEb!=null) sqlBeanList.add(updateEb);
							if(deleteEb!=null) sqlBeanList.add(deleteEb);
							if(this.dataBase.sqlExecute(sqlBeanList))
							{
								success = true;
								this.fireAfterSave(new xlsys.base.dataset.StorableDataSetEvent(this));
							}
						}
						else
						{
							if(this.saveTransaction&&!this.dataBase.getAutoCommit()) this.dataBase.rollback();
						}
					}
					else success = true;
					if(this.saveTransaction&&!this.dataBase.getAutoCommit()) this.dataBase.commit();
					if(success)
					{
						this.changed = false;
						if(this.toBeDeleteRowList!=null) this.toBeDeleteRowList.clear();
						var rowsArray = this.getRows().toArray();
						for(var i in rowsArray)
						{
							var row = rowsArray[i];
							row.setChanged(false);
							if(!this.isVirtualRow(row))
							{
								row.setChangeStatus(xlsys.base.dataset.DataSetRow.STATUS_COMMON);
							}
						}
					}
					this.fireAfterSaveCommit(new xlsys.base.dataset.StorableDataSetEvent(this));
				}
			}
			catch(err)
			{
				if(this.saveTransaction&&!this.dataBase.getAutoCommit()) this.dataBase.rollback();
				this.fireSaveFinally(new xlsys.base.dataset.StorableDataSetEvent(this));
				if(this.saveTransaction) this.dataBase.setAutoCommit(oldAutoCommit);
				throw err;
			}
			this.fireSaveFinally(new xlsys.base.dataset.StorableDataSetEvent(this));
			if(this.saveTransaction) this.dataBase.setAutoCommit(oldAutoCommit);
			return success;
		},

		_saveWithCallBack : function(callback)
		{
			var oldAutoCommit = this.dataBase.getAutoCommit();
			// 启动事务处理
			if(this.saveTransaction) this.dataBase.setAutoCommit(false, qx.Bootstrap.bind(this._saveWithCallBack9, this, callback, oldAutoCommit));
			else this._saveWithCallBack9(callback, oldAutoCommit);
		},

		_saveWithCallBack9 : function(callback, oldAutoCommit)
		{
			if(this.postRow())
			{
				if(this.changed)
				{
					var event = new xlsys.base.dataset.StorableDataSetEvent(this);
					this.fireBeforeSave(event);
					if(event.doit)
					{
						var tempArr = this._initInsertOrUpdateSqlBean();
						// 处理删除的列
						if(this.toBeDeleteRowList!=null)
						{
							this.dataBase.getTableInfo(this.tableName, qx.Bootstrap.bind(this._saveWithCallBack1, this, callback, oldAutoCommit, tempArr[0], tempArr[1]));
						}
						else this._saveWithCallBack10(callback, oldAutoCommit, tempArr[0], tempArr[1], null);
					}
					else
					{
						if(this.saveTransaction&&!this.dataBase.getAutoCommit()) this.dataBase.rollback();
						this._saveWithCallBack6(callback, oldAutoCommit, false);
					}
				}
				else this._saveWithCallBack4(callback, oldAutoCommit, true);
			}
			else this._saveWithCallBack6(callback, oldAutoCommit, false);
		},

		_saveWithCallBack10 : function(callback, oldAutoCommit, insertEb, updateEb, deleteEb)
		{
			var sqlBeanList = new xlsys.java.util.ArrayList();
			if(insertEb!=null) sqlBeanList.add(insertEb);
			if(updateEb!=null) sqlBeanList.add(updateEb);
			if(deleteEb!=null) sqlBeanList.add(deleteEb);
			this.dataBase.sqlExecute(sqlBeanList, qx.Bootstrap.bind(this._saveWithCallBack3, this, callback, oldAutoCommit));
		},

		_saveWithCallBack1 : function(callback, oldAutoCommit, insertEb, updateEb, tableInfo, status)
		{
			var deleteEb = null;
			var tempSize = this.toBeDeleteRowList.size();
			for(var i=0;i<tempSize;++i)
			{
				var row = this.toBeDeleteRowList.get(i);
				if(deleteEb==null)
				{
					deleteEb = new xlsys.base.database.bean.ExecuteBean(xlsys.base.database.bean.ExecuteBean.EXECUTE_TYPE_DELETE, this.tableName);
				}
				var dataMap = new xlsys.java.util.HashMap();
				var pkColArray = tableInfo.getPkColSet().toArray();
				for(var j in pkColArray)
				{
					var colName = pkColArray[j];
					dataMap.put(colName, row.getCell(this.getColumnIndex(colName)).getContent());
				}
				deleteEb.addData(dataMap);
			}
			this._saveWithCallBack10(callback, oldAutoCommit, insertEb, updateEb, deleteEb);
		},

		_saveWithCallBack3 : function(callback, oldAutoCommit, success, status)
		{
			if(success)
			{
				this.fireAfterSave(new xlsys.base.dataset.StorableDataSetEvent(this));
			}
			this._saveWithCallBack4(callback, oldAutoCommit, success);
		},

		_saveWithCallBack4 : function(callback, oldAutoCommit, success)
		{
			if(this.saveTransaction&&!this.dataBase.getAutoCommit())
			{
				this.dataBase.commit(qx.Bootstrap.bind(this._saveWithCallBack6, this, callback, oldAutoCommit, success));
			}
			else this._saveWithCallBack6(callback, oldAutoCommit, success);
		},

		_saveWithCallBack6 : function(callback, oldAutoCommit, success)
		{
			if(success)
			{
				this.changed = false;
				if(this.toBeDeleteRowList!=null) this.toBeDeleteRowList.clear();
				var rowsArray = this.getRows().toArray();
				for(var i in rowsArray)
				{
					var row = rowsArray[i];
					row.setChanged(false);
					if(!this.isVirtualRow(row))
					{
						row.setChangeStatus(xlsys.base.dataset.DataSetRow.STATUS_COMMON);
					}
				}
			}
			this.fireAfterSaveCommit(new xlsys.base.dataset.StorableDataSetEvent(this));
			this._saveWithCallBackFinal1(callback, oldAutoCommit, success);
		},

		_saveWithCallBackFinal1 : function(callback, oldAutoCommit, success)
		{
			if(this.saveTransaction) this.dataBase.setAutoCommit(oldAutoCommit, qx.Bootstrap.bind(this._saveWithCallBackFinal3, this, callback, success));
			else this._saveWithCallBackFinal3(callback, success);
		},

		_saveWithCallBackFinal3 : function(callback, success)
		{
			this.fireSaveFinally(new xlsys.base.dataset.StorableDataSetEvent(this));
			callback(success);
		},

		open : function()
		{
			if(arguments.length==0) this._open();
			else if(arguments.length==1) this._openWithCallBack(arguments[0]);
		},

		_open : function()
		{
			var refds = null;
			try
			{
				var event = new xlsys.base.dataset.StorableDataSetEvent(this);
				event.selectBean = this.selectBean;
				this.fireBeforeOpen(event);
				this.selectBean = event.selectBean;
				if(event.refDataSet!=null)
				{
					refds = event.refDataSet;
				}
				else if(event.refDataSet==null&&this.selectBean!=null)
				{
					if(xlsys.base.util.ObjectUtil.hasBitProp(this.getStyle(), xlsys.base.dataset.IDataSet.STYLE_VIRTUAL))
					{
						var sortColMap = this.getSortColMap();
						if(sortColMap!=null&&!sortColMap.isEmpty()&&this.sorted)
						{
							// 使用带排序功能的查询加载
							refds = this.dataBase.sqlSelect(this.selectBean, sortColMap, 0, this.getExpandPerCount()-1);
						}
						else
						{
							refds = this.dataBase.sqlSelect(this.selectBean, 0, this.getExpandPerCount()-1);
						}
						this.setColumnCount(refds.getColumnCount());
						this.setRowCount(this.dataBase.getResultCount(this.selectBean));
					}
					else
					{
						refds = this.dataBase.sqlSelect(this.selectBean);
					}
				}
				if(refds!=null)
				{
					this.refDataSet(refds);
					this.setRowCursor(-1);
					event = new xlsys.base.dataset.StorableDataSetEvent(this);
					this.fireAfterOpen(event);
				}
			}
			catch(err)
			{
				if(this.dataBase!=null&&!this.dataBase.getAutoCommit()) this.dataBase.rollback();
				throw err;
			}
		},

		_openWithCallBack : function(callback)
		{
			var refds = null;
			var event = new xlsys.base.dataset.StorableDataSetEvent(this);
			event.selectBean = this.selectBean;
			this.fireBeforeOpen(event);
			this.selectBean = event.selectBean;
			if(event.refDataSet!=null)
			{
				refds = event.refDataSet;
			}
			else if(event.refDataSet==null&&this.selectBean!=null)
			{
				if(xlsys.base.util.ObjectUtil.hasBitProp(this.getStyle(), xlsys.base.dataset.IDataSet.STYLE_VIRTUAL))
				{
					var sortColMap = this.getSortColMap();
					var tempCallback = qx.Bootstrap.bind(this._openWithCallBack1, this, callback);
					if(sortColMap!=null&&!sortColMap.isEmpty()&&this.sorted)
					{
						// 使用带排序功能的查询加载
						this.dataBase.sqlSelect(this.selectBean, sortColMap, 0, this.getExpandPerCount()-1, tempCallback);
					}
					else
					{
						this.dataBase.sqlSelect(this.selectBean, 0, this.getExpandPerCount()-1, tempCallback);
					}
				}
				else
				{
					this.dataBase.sqlSelect(this.selectBean, qx.Bootstrap.bind(this._openWithCallBack3, this, callback));
				}
			}
			else this._openWithCallBack3(callback, this, refds);
		},

		_openWithCallBack1 : function(callback, refds, status)
		{
			this.setColumnCount(refds.getColumnCount());
			this.setRowCount(this.dataBase.getResultCount(this.selectBean));
			this._openWithCallBack3(callback, refds);
		},

		_openWithCallBack3 : function(callback, refds)
		{
			if(refds!=null)
			{
				this.refDataSet(refds);
				this.setRowCursor(-1);
				var event = new xlsys.base.dataset.StorableDataSetEvent(this);
				this.fireAfterOpen(event);
			}
			callback(null, "success");
		},

		addUnsaveCol : function(colName)
		{
			if(this.unSaveColNameSet==null) this.unSaveColNameSet = new xlsys.java.util.HashSet();
			this.unSaveColNameSet.add(colName);
		},

		addDeleteRow : function(row)
		{
			if(row==null) return;
			if(!this.isOption(xlsys.base.dataset.IStorableDataSet.OPTION_LOCK_TO_BE_DELETE_ROW))
			{
				if(this.toBeDeleteRowList==null) this.toBeDeleteRowList = new xlsys.java.util.ArrayList();
				this.toBeDeleteRowList.add(row);
			}
		},

		setNewRowStatus : function(row, virtual)
		{
			if(row!=null)
			{
				if(!this.isOption(xlsys.base.dataset.IStorableDataSet.OPTION_LOCK_CHANGE)&&!virtual)
				{
					row.setChanged(true);
					row.setChangeStatus(xlsys.base.dataset.DataSetRow.STATUS_FOR_NEW);
					this.changed = true;
				}
			}
		},

		doInsertRow : function(index, row, clone)
		{
			var newRow = this.base(arguments, index, row, clone);
			this.setNewRowStatus(newRow, false);
			return newRow;
		},

		doInsertNewRow : function(index, virtual)
		{
			var newRow = this.base(arguments, index, virtual);
			this.setNewRowStatus(newRow, virtual);
			return newRow;
		},

		doRemoveRow : function(rowNum)
		{
			var row = this.base(arguments, rowNum);
			this.addDeleteRow(row);
			if(!this.isOption(xlsys.base.dataset.IStorableDataSet.OPTION_LOCK_CHANGE)) this.changed = true;
			return row;
		},

		removeAllRow : function()
		{
			this.expandAllRows();
			if(!this.isOption(xlsys.base.dataset.IStorableDataSet.OPTION_LOCK_TO_BE_DELETE_ROW))
			{
				var rows = this.getRows();
				for(var i=0;i<rows.size();++i) this.addDeleteRow(rows.get(i));
				if(!this.isOption(xlsys.base.dataset.IStorableDataSet.OPTION_LOCK_CHANGE)) this.changed = true;
			}
			this.base(arguments);
		},

		doSetValue : function(rowIndex, colIndex, value)
		{
			var isPkCol = this.getColumn(colIndex).isPrimaryKey();
			var row = this.getRow(rowIndex);
			var needReplace = isPkCol&&row.getChangeStatus()!=xlsys.base.dataset.DataSetRow.STATUS_FOR_NEW;
			var oldContent = null;
			if(needReplace) oldContent = this.getValue(rowIndex, colIndex);
			this.base(arguments, rowIndex, colIndex, value);
			if(needReplace)
			{
				row.getCell(colIndex).setOldContent(oldContent);
				row.setChangeStatus(xlsys.base.dataset.DataSetRow.STATUS_REPLACE);
			}
			if(!this.isOption(xlsys.base.dataset.IStorableDataSet.OPTION_LOCK_CHANGE))
			{
				if(!this.changed) this.changed = true;
				this.getRow(rowIndex).setChanged(this.changed);
			}
		},

		getValue : function()
		{
			var value = this._getValue(arguments);
			if(arguments.length==2&&(arguments[0] instanceof xlsys.base.dataset.DataSetRow)&&qx.lang.Type.isNumber(arguments[1]))
			{
				var row = arguments[0];
				var colIndex = arguments[1];
				if(value==null&&this.toBeDeleteRowList!=null&&this.toBeDeleteRowList.contains(row))
				{
					value = row.getCell(colIndex).getContent();
				}
			}
			return value;
		},

		refresh : function()
		{
			var event = new xlsys.base.dataset.DataSetEvent(this);
			this.fireBeforeRefresh(event);
			if(arguments.length==0)
			{
				this.refreshing();
				event = new xlsys.base.dataset.DataSetEvent(this);
				this.fireAfterRefresh(event);
			}
			else
			{
				this.refreshing(arguments[0]);
			}
		},

		refreshing : function()
		{
			if(this.toBeDeleteRowList!=null) this.toBeDeleteRowList.clear();
			this.changed = false;
			this.sorted = false;
			this.getRows().clear();
			this.getColumns().clear();
			if(arguments.length==0)
			{
				// 同步处理
				try
				{
					this.open();
				}
				catch (err)
				{
					var event = new xlsys.base.dataset.DataSetEvent(this);
					event.errMsg = err;
					event.logLevel = xlsys.base.log.Log.LOG_LEVEL_ERROR;
					this.fireShowMessage(event);
				}
			}
			else
			{
				// 异步处理
				this.open(qx.Bootstrap.bind(this._refreshingCallback, this, arguments[0]));
			}
		},

		_refreshingCallback : function(callback, data, status)
		{
			var event = new xlsys.base.dataset.DataSetEvent(this);
			this.fireAfterRefresh(event);
			callback(null, status);
		},

		fireBeforeOpen : function(event)
		{
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						if(qx.Class.implementsInterface(listener, xlsys.base.dataset.StorableDataSetListener))
						{
							listener.beforeOpen(event);
							if(event.interrupt) break;
						}
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		fireAfterOpen : function(event)
		{
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						if(qx.Class.implementsInterface(listener, xlsys.base.dataset.StorableDataSetListener))
						{
							listener.afterOpen(event);
							if(event.interrupt) break;
						}
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},
		
		firePrepareSave : function(event)
		{
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						if(qx.Class.implementsInterface(listener, xlsys.base.dataset.StorableDataSetListener))
						{
							listener.prepareSave(event);
							if(event.interrupt) break;
						}
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		fireBeforeSave : function(event)
		{
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						if(qx.Class.implementsInterface(listener, xlsys.base.dataset.StorableDataSetListener))
						{
							listener.beforeSave(event);
							if(event.interrupt) break;
						}
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		fireAfterSave : function(event)
		{
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						if(qx.Class.implementsInterface(listener, xlsys.base.dataset.StorableDataSetListener))
						{
							listener.afterSave(event);
							if(event.interrupt) break;
						}
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		fireAfterSaveCommit : function(event)
		{
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						if(qx.Class.implementsInterface(listener, xlsys.base.dataset.StorableDataSetListener))
						{
							listener.afterSaveCommit(event);
							if(event.interrupt) break;
						}
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		fireSaveFinally : function(event)
		{
			try
			{
				if(this.dataSetListenerSet!=null)
				{
					var listeners = this.dataSetListenerSet.toArray();
					for(var i=0;i<listeners.length;++i)
					{
						var listener = listeners[i];
						if(qx.Class.implementsInterface(listener, xlsys.base.dataset.StorableDataSetListener))
						{
							listener.saveFinally(event);
							if(event.interrupt) break;
						}
					}
					if(!event.doit&&event.errMsg!=null) this.fireShowMessage(event);
				}
			}
			catch(err)
			{
				event.doit = false;
				event.errMsg = err;
				this.fireShowMessage(event);
			}
		},

		isChanged : function()
		{
			return this.changed;
		},

		setChanged : function(changed)
		{
			this.changed = changed;
		},

		getDataBase : function()
		{
			return this.dataBase;
		},

		getTableName : function()
		{
			return this.tableName;
		},

		getUnSaveColNameSet : function()
		{
			return this.unSaveColNameSet;
		},

		hasToBeDeleteRow : function()
		{
			return this.toBeDeleteRowList!=null&&!this.toBeDeleteRowList.isEmpty();
		},

		getToBeDeleteRowList : function()
		{
			return this.toBeDeleteRowList;
		},

		hasChangedRow : function()
		{
			var hasChangedRow = false;
			var rowsArray = this.getRows().toArray();
			for(var i in rowsArray)
			{
				var row = rowsArray[i];
				if(row.isChanged())
				{
					hasChangedRow = true;
					break;
				}
			}
			return hasChangedRow;
		},

		sort : function()
		{
			try
			{
				this.setEventLock(true);
				if(this.selectBean!=null&&xlsys.base.util.ObjectUtil.hasBitProp(this.getStyle(), xlsys.base.dataset.IDataSet.STYLE_VIRTUAL)&&qx.Class.implementsInterface(this.dataBase, xlsys.base.database.IClientDataBase)&&this.save())
				{
					this.getRows().clear();
					this.setRowCount(this.dataBase.getResultCount(this.selectBean));
				}
				else this.base(arguments);
				this.setEventLock(false);
			}
			catch(err)
			{
				var event = new xlsys.base.dataset.DataSetEvent(this);
				event.errMsg = err;
				event.logLevel = xlsys.base.log.Log.LOG_LEVEL_ERROR;
				this.fireShowMessage(event);
				this.base(arguments);
			}
			this.sorted = true;
		},

		setEventLock : function()
		{
			if(arguments.length==1) this.base(arguments, arguments[0]);
			else if(arguments.length==2) this.base(arguments, arguments[0], arguments[1]);
			var dataSet = this;
			var lock = false;
			if(arguments.length==1) lock = arguments[0];
			else
			{
				dataSet = arguments[0];
				lock = arguments[1];
			}
			if(qx.Class.implementsInterface(dataSet, xlsys.base.dataset.IStorableDataSet))
			{
				this.setOption(xlsys.base.dataset.IStorableDataSet.OPTION_LOCK_CHANGE, lock, true);
				this.setOption(xlsys.base.dataset.IStorableDataSet.OPTION_LOCK_TO_BE_DELETE_ROW, lock, true);
			}
		}
	}
});