/**
 * 可存储的树形结构数据集
 */
qx.Class.define("xlsys.base.dataset.StorableTreeDataSet",
{
	extend : xlsys.base.dataset.StorableDataSet,
	implement : [xlsys.base.dataset.ITreeDataSet],
	
	/**
	 * 需要重载以下方法:
	 * StorableTreeDataSet(IStorableDataSet sds);
	 * StorableTreeDataSet(IStorableDataSet sds, int style);
	 * StorableTreeDataSet(IDataBase dataBase, String tableName);
	 * StorableTreeDataSet(IDataBase dataBase, String tableName, int style);
	 * StorableTreeDataSet(IDataBase dataBase, String selectSql, String tableName);
	 * StorableTreeDataSet(IDataBase dataBase, String selectSql, String tableName, int style);
	 * StorableTreeDataSet(IDataBase dataBase, ParamBean selectBean, String tableName);
	 * StorableTreeDataSet(IDataBase dataBase, ParamBean selectBean, String tableName, int style);
	 */
	construct: function()
	{
		this.treeColIdx = null;
		var style = null;
		var sds = null;
		if(arguments.length==1)
		{
			sds = arguments[0];
			style = sds.getStyle();
		}
		else if(arguments.length==2)
		{
			if(qx.Class.implementsInterface(arguments[0], xlsys.base.dataset.IStorableDataSet))
			{
				sds = arguments[0];
				style = arguments[1];
				this.base(arguments, sds.getDataBase(), sds.getSelectBean(), sds.getTableName(), style);
				this.refDataSet(sds);
			}
			else
			{
				this.base(arguments, arguments[0], arguments[1]);
			}
		}
		else if(arguments.length==3)
		{
			this.base(arguments, arguments[0], arguments[1], arguments[2]);
		}
		else if(arguments.length==4)
		{
			this.base(arguments, arguments[0], arguments[1], arguments[2], arguments[3]);
		}
	},
	
	members :
	{
		treeColIdx : null,
		
		open : function()
		{
			this.base(arguments);
			if(this.getTreeColumn()!=-1)
			{
				this.sorted = false;
				this.sort();
			}
		},
		
		setTreeColumn : function(colNameOrIdx)
		{
			var colIdx = null;
			if(qx.lang.Type.isString(colNameOrIdx)) colIdx = this.getColumnIndex(colNameOrIdx);
			else colIdx = colNameOrIdx;
			var sortType = xlsys.base.dataset.IDataSet.SORT_TYPE_ASC;
			if(arguments.length==2) sortType = arguments[1];
			return xlsys.base.dataset.TreeDataSet.doSetTreeColumn(this, colIdx, sortType);
		},
		
		setTreeColIdx : function(treeColIdx)
		{
			this.treeColIdx = treeColIdx;
		},
		
		getTreeColumnName : function()
		{
			return xlsys.base.dataset.TreeDataSet.doGetTreeColumnName(this);
		},
		
		getTreeColumn : function()
		{
			return this.treeColIdx==null?-1:this.treeColIdx;
		},
		
		setSortColumn : function()
		{
			xlsys.base.dataset.TreeDataSet.doSetSortColumn(this);
		},
		
		swapRow : function(i, j)
		{
			xlsys.base.dataset.TreeDataSet.doSwapRow(this);
		},
		
		getRootRows : function()
		{
			return xlsys.base.dataset.TreeDataSet.getRowList(this, this.getRootRowNums());
		},
		
		getRootRowNums : function()
		{
			return xlsys.base.dataset.TreeDataSet.doGetRootRowNums(this);
		},
		
		getChildrenRows : function(rowOrRowAt)
		{
			return xlsys.base.dataset.TreeDataSet.getRowList(this, this.getChildrenRowNums(rowOrRowAt));
		},
		
		getChildrenRowNums : function(rowOrRowAt)
		{
			var rowAt = null;
			if(qx.lang.Type.isNumber(rowOrRowAt)) rowAt = rowOrRowAt;
			else rowAt = this.getRowIndex(rowOrRowAt);
			return xlsys.base.dataset.TreeDataSet.doGetChildrenRowNums(this, rowAt);
		},
		
		getAllChildrenRows : function(rowOrRowAt)
		{
			return xlsys.base.dataset.TreeDataSet.getRowList(this, this.getAllChildrenRowNums(rowOrRowAt));
		},
		
		getAllChildrenRowNums : function(rowOrRowAt)
		{
			var rowAt = null;
			if(qx.lang.Type.isNumber(rowOrRowAt)) rowAt = rowOrRowAt;
			else rowAt = this.getRowIndex(rowOrRowAt);
			return xlsys.base.dataset.TreeDataSet.doGetAllChildrenRowNums(this, rowAt);
		},
		
		getParentRow : function(rowOrRowAt)
		{
			var rowAt = null;
			if(qx.lang.Type.isNumber(rowOrRowAt)) rowAt = rowOrRowAt;
			else rowAt = this.getRowIndex(rowOrRowAt);
			return xlsys.base.dataset.TreeDataSet.doGetParentRow(this, rowAt);
		},
		
		getParentRowNum : function(rowOrRowAt)
		{
			var rowAt = null;
			if(qx.lang.Type.isNumber(rowOrRowAt)) rowAt = rowOrRowAt;
			else rowAt = this.getRowIndex(rowOrRowAt);
			return xlsys.base.dataset.TreeDataSet.doGetParentRowNum(this, rowAt);
		},
		
		getAllParent : function(rowOrRowAt)
		{
			var rowAt = null;
			if(qx.lang.Type.isNumber(rowOrRowAt)) rowAt = rowOrRowAt;
			else rowAt = this.getRowIndex(rowOrRowAt);
			return xlsys.base.dataset.TreeDataSet.doGetAllParent(this, rowAt);
		},
		
		removeRow : function(rowIndex)
		{
			// 删除子行
			var children = this.getChildrenRowNums(rowIndex);
			if(children!=null)
			{
				for(var i=children.size()-1;i>=0;--i)
				{
					this.removeRow(children.get(i));
				}
			}
			return this.base(arguments, rowIndex);
		}
	}
});