/**
 * 支持树形结构的数据集
 */
qx.Class.define("xlsys.base.dataset.TreeDataSet",
{
	extend : xlsys.base.dataset.DataSet,
	implement : [xlsys.base.dataset.ITreeDataSet],
	
	/**
	 * 需要重载以下方法:
	 * TreeDataSet();
	 * TreeDataSet(int style);
	 * TreeDataSet(IDataSet dataSet);
	 * TreeDataSet(IDataSet dataSet, int style);
	 */
	construct: function()
	{
		this.treeColIdx = null;
		var style = null;
		var refDataSet = null;
		if(arguments.length==0) this.base(arguments);
		else if(arguments.length==1)
		{
			if(qx.lang.Type.isNumber(arguments[0])) style = arguments[0];
			else
			{
				refDataSet = arguments[0];
				style = xlsys.base.dataset.IDataSet.STYLE_NONE;
			}
		}
		else if(arguments.length==2)
		{
			refDataSet = arguments[0];
			style = arguments[1];
		}
		if(style!=null) this.base(arguments, style);
		if(refDataSet!=null) this.refDataSet(refDataSet);
	},
	
	statics :
	{
		doSetTreeColumn : function(treeDataSet, columnIdx, sortType)
		{
			var success = false;
			if(columnIdx>=0&&columnIdx<treeDataSet.getColumnCount())
			{
				var sortColMap = new xlsys.java.util.HashMap();
				sortColMap.put(treeDataSet.getColumnName(columnIdx), sortType);
				treeDataSet.setSortColMap(sortColMap);
				treeDataSet.sort();
				treeDataSet.setTreeColIdx(columnIdx);
				success = true;
			}
			return success;
		},
		
		doGetTreeColumnName : function(treeDataSet)
		{
			var colName = null;
			var treeColIdx = treeDataSet.getTreeColumn();
			if(treeColIdx!=-1) colName = treeDataSet.getColumnName(treeColIdx);
			return colName;
		},
		
		doSetSortColumn : function(treeDataSet)
		{
			throw new xlsys.java.lang.Exception("This method is unsupported for ITreeDataSet, try method : setTreeColumn instead.");
		},
		
		doSwapRow : function(treeDataSet)
		{
			throw new xlsys.java.lang.Exception("This method is unsupported for ITreeDataSet.");
		},
		
		getRowList : function(treeDataSet, rowAtList)
		{
			var rowList = null;
			if(rowAtList!=null)
			{
				rowList = new xlsys.java.util.ArrayList();
				var rows = treeDataSet.getRows();
				for(var i=0;i<rowAtList.size();++i)
				{
					var rowAt = rowAtList.get(i);
					rowList.add(rows.get(rowAt));
				}
			}
			return rowList;
		},
		
		doGetRootRowNums : function(treeDataSet)
		{
			var root = null;
			var childrenSet = new xlsys.java.util.HashSet();
			var rows = treeDataSet.getRows();
			for(var i=0;i<rows.size();++i)
			{
				if(!childrenSet.contains(i))
				{
					if(root==null) root = new xlsys.java.util.ArrayList();
					root.add(i);
					var children = treeDataSet.getAllChildrenRowNums(i);
					if(children!=null) childrenSet.addAll(children);
				}
			}
			return root;
		},
		
		doGetChildrenRowNums : function(treeDataSet, rowAt)
		{
			var children = null;
			var treeColIdx = treeDataSet.getTreeColumn();
			if(treeColIdx!=-1)
			{
				var rows = treeDataSet.getRows();
				if(rowAt>=0&&rowAt<rows.size())
				{
					var row = rows.get(rowAt);
					var rowStr = "" + row.getCell(treeColIdx).getContent();
					var lastStr = null;
					for(var i=rowAt+1;i<rows.size();++i)
					{
						var otherRow = rows.get(i);
						var otherRowStr = "" + otherRow.getCell(treeColIdx).getContent();
						if((lastStr==null || (lastStr!=null&&!qx.lang.String.startsWith(otherRowStr, lastStr))) && qx.lang.String.startsWith(otherRowStr, rowStr)&&otherRowStr.length>rowStr.length)
						{
							if(children==null) children = new xlsys.java.util.ArrayList();
							children.add(i);
							lastStr = otherRowStr;
						}
						else if(otherRowStr.length<=rowStr.length)
						{
							break;
						}
					}
				}
			}
			return children;
		},
		
		doGetAllChildrenRowNums : function(treeDataSet, rowAt)
		{
			var children = null;
			var treeColIdx = treeDataSet.getTreeColumn();
			if(treeColIdx!=-1)
			{
				var rows = treeDataSet.getRows();
				if(rowAt>=0&&rowAt<rows.size())
				{
					var row = rows.get(rowAt);
					var rowStr = "" + row.getCell(treeColIdx).getContent();
					for(var i=rowAt+1;i<rows.size();++i)
					{
						var otherRow = rows.get(i);
						var otherRowStr = "" + otherRow.getCell(treeColIdx).getContent();
						if(qx.lang.String.startsWith(otherRowStr, rowStr)&&otherRowStr.length>rowStr.length)
						{
							if(children==null) children = new xlsys.java.util.ArrayList();
							children.add(i);
						}
						else
						{
							break;
						}
					}
				}
			}
			return children;
		},
		
		doGetParentRow : function(treeDataSet, rowAt)
		{
			var parentRow = null;
			var parentIdx = treeDataSet.getParentRowNum(rowAt);
			if(parentIdx!=-1) parentRow = treeDataSet.getRow(parentIdx);
			return parentRow;
		},
		
		doGetParentRowNum : function(treeDataSet, rowAt)
		{
			var parentIdx = -1;
			var treeColIdx = treeDataSet.getTreeColumn();
			if(treeColIdx!=-1)
			{
				var rows = treeDataSet.getRows();
				if(rowAt>=0&&rowAt<rows.size())
				{
					var row = rows.get(rowAt);
					var rowStr = "" + row.getCell(treeColIdx).getContent();
					for(var i=rowAt-1;i>=0;--i)
					{
						var otherRow = rows.get(i);
						var otherRowStr = "" + otherRow.getCell(treeColIdx).getContent();
						if(qx.lang.String.startsWith(rowStr, otherRowStr)&&otherRowStr.length<rowStr.length)
						{
							parentIdx = i;
							break;
						}
					}
				}
			}
			return parentIdx;
		},
		
		doGetAllParent : function(treeDataSet, rowAt)
		{
			var parentList = null;
			var treeColIdx = treeDataSet.getTreeColumn();
			if(treeColIdx!=-1)
			{
				var rows = treeDataSet.getRows();
				if(rowAt>=0&&rowAt<rows.size())
				{
					var row = rows.get(rowAt);
					var rowStr = "" + row.getCell(treeColIdx).getContent();
					for(var i=rowAt-1;i>=0;--i)
					{
						var otherRow = rows.get(i);
						var otherRowStr = "" + otherRow.getCell(treeColIdx).getContent();
						if(qx.lang.String.startsWith(rowStr, otherRowStr)&&otherRowStr.length<rowStr.length)
						{
							if(parentList==null) parentList = new xlsys.java.util.ArrayList();
							parentList.add(i);
						}
					}
				}
			}
			return parentList;
		}
	},
	
	members :
	{
		treeColIdx : null,
		
		setTreeColumn : function(colNameOrIdx)
		{
			var colIdx = null;
			if(qx.lang.Type.isString(colNameOrIdx)) colIdx = this.getColumnIndex(colNameOrIdx);
			else colIdx = colNameOrIdx;
			var sortType = xlsys.base.dataset.IDataSet.SORT_TYPE_ASC;
			if(arguments.length==2) sortType = arguments[1];
			return this.self(arguments).doSetTreeColumn(this, colIdx, sortType);
		},
		
		setTreeColIdx : function(treeColIdx)
		{
			this.treeColIdx = treeColIdx;
		},
		
		getTreeColumnName : function()
		{
			return this.self(arguments).doGetTreeColumnName(this);
		},
		
		getTreeColumn : function()
		{
			return this.treeColIdx==null?-1:this.treeColIdx;
		},
		
		setSortColumn : function()
		{
			this.self(arguments).doSetSortColumn(this);
		},
		
		swapRow : function(i, j)
		{
			this.self(arguments).doSwapRow(this);
		},
		
		getRootRows : function()
		{
			return this.self(arguments).getRowList(this, this.getRootRowNums());
		},
		
		getRootRowNums : function()
		{
			return this.self(arguments).doGetRootRowNums(this);
		},
		
		getChildrenRows : function(rowOrRowAt)
		{
			return this.self(arguments).getRowList(this, this.getChildrenRowNums(rowOrRowAt));
		},
		
		getChildrenRowNums : function(rowOrRowAt)
		{
			var rowAt = null;
			if(qx.lang.Type.isNumber(rowOrRowAt)) rowAt = rowOrRowAt;
			else rowAt = this.getRowIndex(rowOrRowAt);
			return this.self(arguments).doGetChildrenRowNums(this, rowAt);
		},
		
		getAllChildrenRows : function(rowOrRowAt)
		{
			return this.self(arguments).getRowList(this, this.getAllChildrenRowNums(rowOrRowAt));
		},
		
		getAllChildrenRowNums : function(rowOrRowAt)
		{
			var rowAt = null;
			if(qx.lang.Type.isNumber(rowOrRowAt)) rowAt = rowOrRowAt;
			else rowAt = this.getRowIndex(rowOrRowAt);
			return this.self(arguments).doGetAllChildrenRowNums(this, rowAt);
		},
		
		getParentRow : function(rowOrRowAt)
		{
			var rowAt = null;
			if(qx.lang.Type.isNumber(rowOrRowAt)) rowAt = rowOrRowAt;
			else rowAt = this.getRowIndex(rowOrRowAt);
			return this.self(arguments).doGetParentRow(this, rowAt);
		},
		
		getParentRowNum : function(rowOrRowAt)
		{
			var rowAt = null;
			if(qx.lang.Type.isNumber(rowOrRowAt)) rowAt = rowOrRowAt;
			else rowAt = this.getRowIndex(rowOrRowAt);
			return this.self(arguments).doGetParentRowNum(this, rowAt);
		},
		
		getAllParent : function(rowOrRowAt)
		{
			var rowAt = null;
			if(qx.lang.Type.isNumber(rowOrRowAt)) rowAt = rowOrRowAt;
			else rowAt = this.getRowIndex(rowOrRowAt);
			return this.self(arguments).doGetAllParent(this, rowAt);
		},
		
		removeRow : function(rowIndex)
		{
			// 删除子行
			var children = this.getChildrenRowNums(rowIndex);
			if(children!=null)
			{
				for(var i=children.size()-1;i>=0;--i)
				{
					this.removeRow(children.get(i));
				}
			}
			return this.base(arguments, rowIndex);
		}
	}
});
	