/**
 * 
 */
qx.Class.define("xlsys.base.io.util.ByteDataInputStream",
{
	extend : qx.core.Object,
	
	construct : function(bytes)
	{
		this.bytes = bytes;
		this.pos = 0;
	},
	
	members : 
	{
		bytes : null,
		pos : null,
		
		_atEnd : function()
		{
			return this.pos>=this.bytes.length; 
		},
		
		_checkRegion : function(count)
		{
			if(this._atEnd()&&count>0) throw new Error("EOF!");
			if(count!=null&&(this.pos+count>this.bytes.length)) throw new Error("EOF!");
		},
		
		read : function()
		{
			if(this._atEnd()) return -1;
			return this.readByte();
		},
		
		readByte : function()
		{
			this._checkRegion();
			return this.bytes[this.pos++];
		},
		
		readChar : function()
		{
			var charCode = this.readShort();
			return String.fromCharCode(charCode);
		},
		
		readShort : function()
		{
			this._checkRegion(2);
			var a = this.bytes[this.pos++];
			var b = this.bytes[this.pos++];
			return (((a & 0xff) << 8) | (b & 0xff));
		},
		
		readInt : function()
		{
			this._checkRegion(4);
			var a = this.bytes[this.pos++];
			var b = this.bytes[this.pos++];
			var c = this.bytes[this.pos++];
			var d = this.bytes[this.pos++];
			return (((a & 0xff) << 24) | ((b & 0xff) << 16) | ((c & 0xff) << 8) | (d & 0xff));
		},
		
		readLong : function()
		{
			this._checkRegion(8);
			var a = this.bytes[this.pos++];
			var b = this.bytes[this.pos++];
			var c = this.bytes[this.pos++];
			var d = this.bytes[this.pos++];
			var e = this.bytes[this.pos++];
			var f = this.bytes[this.pos++];
			var g = this.bytes[this.pos++];
			var h = this.bytes[this.pos++];
			return (((a & 0xff) << 56) |
					  ((b & 0xff) << 48) |
					  ((c & 0xff) << 40) |
					  ((d & 0xff) << 32) |
					  ((e & 0xff) << 24) |
					  ((f & 0xff) << 16) |
					  ((g & 0xff) <<  8) |
					  ((h & 0xff)));
		},
		
		readFully : function(b, off, len)
		{
			this._checkRegion(len);
			if(off==null) off = 0;
			if(len==null) len = this.bytes.length - this.pos;
			for(var i=0;i<len;++i)
			{
				b[i+off] = this.bytes[this.pos++];
			}
		},
		
		available : function()
		{
			return this.bytes.length - this.pos;
		}
	}
});