/**
 * 
 */
qx.Class.define("xlsys.base.io.util.ByteDataOutputStream",
{
	extend : qx.core.Object,
	
	construct : function()
	{
		this.bytes = new Array();
		this.pos = 0;
	},
	
	members : 
	{
		bytes : null,
		
		/**
		 * 支持以下方法:
		 * write(byte);
		 * write(bytes);
		 * write(b, off, len);
		 */
		write : function()
		{
			if(arguments.length==1)
			{
				if(qx.lang.Type.isArray(arguments[0]))
				{
					this._write(arguments[0], 0, arguments[0].length);
				}
				else this.writeByte(arguments[0]);
			}
			else
			{
				this._write(arguments[0], arguments[1], arguments[2]);
			}
		},
		
		_write : function(b, off, len)
		{
			for(var i=0;i<len;++i)
			{
				this.bytes.push(b[i+off] & 0xff);
			}
		},
		
		writeByte : function(v)
		{
			qx.core.Assert.assertInteger(v);
			this.bytes.push(v & 0xff);
		},
		
		writeChar : function(v)
		{
			this.writeShort(v);
		},
		
		writeShort : function(v)
		{
			qx.core.Assert.assertInteger(v);
			this.bytes.push(xlsys.base.util.NumberUtil.getInstance().unsignedRightShift(v, 8) & 0xff);
			this.bytes.push(v & 0xff);
		},
		
		writeInt : function(v)
		{
			qx.core.Assert.assertInteger(v);
			this.bytes.push(xlsys.base.util.NumberUtil.getInstance().unsignedRightShift(v, 24) & 0xff);
			this.bytes.push(xlsys.base.util.NumberUtil.getInstance().unsignedRightShift(v, 16) & 0xff);
			this.bytes.push(xlsys.base.util.NumberUtil.getInstance().unsignedRightShift(v, 8) & 0xff);
			this.bytes.push(v & 0xff);
		},
		
		writeLong : function(v)
		{
			qx.core.Assert.assertInteger(v);
			this.bytes.push(xlsys.base.util.NumberUtil.getInstance().unsignedRightShift(v, 56) & 0xff);
			this.bytes.push(xlsys.base.util.NumberUtil.getInstance().unsignedRightShift(v, 48) & 0xff);
			this.bytes.push(xlsys.base.util.NumberUtil.getInstance().unsignedRightShift(v, 40) & 0xff);
			this.bytes.push(xlsys.base.util.NumberUtil.getInstance().unsignedRightShift(v, 32) & 0xff);
			this.bytes.push(xlsys.base.util.NumberUtil.getInstance().unsignedRightShift(v, 24) & 0xff);
			this.bytes.push(xlsys.base.util.NumberUtil.getInstance().unsignedRightShift(v, 16) & 0xff);
			this.bytes.push(xlsys.base.util.NumberUtil.getInstance().unsignedRightShift(v, 8) & 0xff);
			this.bytes.push(v & 0xff);
		},
		
		toByteArray : function()
		{
			return this.bytes;
		}
	}
});