/**
 * io工具类
 */
qx.Class.define("xlsys.base.io.util.IOUtil",
{
	type : "static",
	
	statics :
	{
		__rRegExp : new RegExp("\r", "g"),
		__nRegExp : new RegExp("\n", "g"),
		__escapeRRegExp : new RegExp("&#13;", "g"),
		__escapeNRegExp : new RegExp("&#10;", "g"),
		
		/**
		 * 将对象转换成JSON字符串
		 * @param obj {Object}
		 * @return {String}
		 */
		getJSONStr : function(obj)
		{
			var hasWriteObjMap = new xlsys.java.util.LinkedHashMap();
			this.prepareJSONStr(obj, hasWriteObjMap);
			var sb = new xlsys.java.lang.StringBuilder();
			sb.append("[");
			var keyArray = hasWriteObjMap.keys();
			for(var i=0;i<keyArray.length;++i)
			{
				sb.append("{");
				// 写入hashcode
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_HASHCODE, keyArray[i]));
				// 写入内容
				sb.append(",").append(this.getDQMStr(xlsys.base.XLSYS.JSON_OBJ_CONTENT)).append(":").append(hasWriteObjMap.get(keyArray[i]));
				sb.append("}");
				if(i!=keyArray.length-1) sb.append(",");
			}
			sb.append("]");
			return sb.toString();
		},

		getDQMStr : function(str)
		{
			return "\""+str+"\"";
		},

		createJSONAttr : function(attrName, attrValue)
		{
			var sb = new xlsys.java.lang.StringBuilder();
			sb.append("\"").append(attrName).append("\":");
			if(attrValue==null) sb.append("null");
			else if((typeof attrValue)=="string") sb.append("\"").append(attrValue).append("\"");
			else sb.append(attrValue);
			return sb.toString();
		},

		prepareJSONStr : function(obj, hasWriteObjMap, dataType)
		{
			if(hasWriteObjMap.containsKey(xlsys.base.util.ObjectUtil.hashcode(obj))) return;
			// 先将当前obj放入,以免子属性递归时重复引用造成无限循环
			hasWriteObjMap.put(xlsys.base.util.ObjectUtil.hashcode(obj), "");
			var sb = new xlsys.java.lang.StringBuilder();
			sb.append("{");
			if(obj==undefined||obj==null) dataType = xlsys.base.XLSYS.DATA_TYPE_NULL;
			if(dataType==null)
			{
				if(qx.lang.Type.isString(obj)) dataType = xlsys.base.XLSYS.DATA_TYPE_STR;
				else if(qx.lang.Type.isArray(obj))
				{
					// 判断元素是数字、文本、还是对象
					dataType = xlsys.base.XLSYS.DATA_TYPE_BYTE_ARRAY;
					var typeofSet = new xlsys.java.util.HashSet();
					for(var i=0;i<obj.length;++i)
					{
						typeofSet.add(xlsys.base.util.ObjectUtil.className(obj[i]));
					}
					if(typeofSet.size()==1)
					{
						var typeNames = typeofSet.toArray();
						if(typeNames[0]=="java.math.BigDecimal") dataType = xlsys.base.XLSYS.DATA_TYPE_BYTE_ARRAY;
						else if(typeNames[0]=="java.lang.String") dataType = xlsys.base.XLSYS.DATA_TYPE_STR_ARRAY;
						else if(typeNames[0]=="java.util.AbstractMap.SimpleEntry") dataType = xlsys.base.XLSYS.DATA_TYPE_ENTRY_ARRAY;
						else dataType = xlsys.base.XLSYS.DATA_TYPE_OBJECT_ARRAY;
					}
					else dataType = xlsys.base.XLSYS.DATA_TYPE_OBJECT_ARRAY;
				}
				else if(qx.lang.Type.isNumber(obj)) dataType = xlsys.base.XLSYS.DATA_TYPE_BIGDECIMAL;
				else if(qx.lang.Type.isBoolean(obj)) dataType = xlsys.base.XLSYS.DATA_TYPE_BOOL;
				else if(qx.lang.Type.isDate(obj)) dataType = xlsys.base.XLSYS.DATA_TYPE_DATE;
				else if(qx.Interface.objectImplements(obj, xlsys.java.util.Entry)) dataType = xlsys.base.XLSYS.DATA_TYPE_ENTRY;
				else if(qx.Interface.objectImplements(obj, xlsys.java.util.Collection)) dataType = xlsys.base.XLSYS.DATA_TYPE_COLLECTION;
				else if(qx.Interface.objectImplements(obj, xlsys.java.util.Map)) dataType = xlsys.base.XLSYS.DATA_TYPE_MAP;
				else if(obj instanceof xlsys.java.io.Serializable) dataType = xlsys.base.XLSYS.DATA_TYPE_OBJECT;
				else throw new xlsys.java.lang.Exception("Unsupported Transfer Object!");
			}
			if(dataType==xlsys.base.XLSYS.DATA_TYPE_NULL)
			{
				// 写入类型
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_NULL));
			}
			else if(dataType==xlsys.base.XLSYS.DATA_TYPE_STR)
			{
				// 写入类型
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_STR));
				// 写入内容
				sb.append(",");
				// 替换\r和\n, 防止JSON格式出错
				if(obj.indexOf("\n")>=0||obj.indexOf("\r")>=0)
				{
					var temp = obj.replace(this.__nRegExp, "&#10;").replace(this.__rRegExp, "&#13;");
					sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_CONTENT, temp));
				}
				else sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_CONTENT, obj));
			}
			else if(dataType==xlsys.base.XLSYS.DATA_TYPE_SHORT)
			{
				// 写入类型
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_SHORT));
				// 写入内容
				sb.append(",");
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_CONTENT, ""+parseInt(obj)&0x0000ffff));
			}
			else if(dataType==xlsys.base.XLSYS.DATA_TYPE_INT)
			{
				// 写入类型
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_INT));
				// 写入内容
				sb.append(",");
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_CONTENT, ""+parseInt(obj)));
			}
			else if(dataType==xlsys.base.XLSYS.DATA_TYPE_LONG)
			{
				// 写入类型
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_LONG));
				// 写入内容
				sb.append(",");
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_CONTENT, ""+parseInt(obj)));
			}
			else if(dataType==xlsys.base.XLSYS.DATA_TYPE_CHAR)
			{
				// 写入类型
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_CHAR));
				// 写入内容
				sb.append(",");
				if(qx.lang.Type.isString(obj)) obj = obj.charCodeAt(0);
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_CONTENT, ""+parseInt(obj)&0x000000ff));
			}
			else if(dataType==xlsys.base.XLSYS.DATA_TYPE_FLOAT)
			{
				// 写入类型
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_FLOAT));
				// 写入内容
				sb.append(",");
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_CONTENT, obj.toString()));
			}
			else if(dataType==xlsys.base.XLSYS.DATA_TYPE_DOUBLE)
			{
				// 写入类型
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_DOUBLE));
				// 写入内容
				sb.append(",");
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_CONTENT, obj.toString()));
			}
			else if(dataType==xlsys.base.XLSYS.DATA_TYPE_BYTE)
			{
				// 写入类型
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_BYTE));
				// 写入内容
				sb.append(",");
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_CONTENT, ""+parseInt(obj)&0x000000ff));
			}
			else if(dataType==xlsys.base.XLSYS.DATA_TYPE_BYTE_ARRAY||
					dataType==xlsys.base.XLSYS.DATA_TYPE_STR_ARRAY||
					dataType==xlsys.base.XLSYS.DATA_TYPE_ENTRY_ARRAY||
					dataType==xlsys.base.XLSYS.DATA_TYPE_OBJECT_ARRAY)
			{
				// 写入类型
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, dataType));
				sb.append(",");
				sb.append(this.getDQMStr(xlsys.base.XLSYS.JSON_OBJ_CONTENT)).append(":[");
				if(dataType==xlsys.base.XLSYS.DATA_TYPE_BYTE_ARRAY)
				{
					// 字节数组
					// 写入内容
					for(var i=0;i<obj.length;++i)
					{
						sb.append(obj[i]);
						if(i!=(obj.length-1)) sb.append(",");
					}
				}
				else
				{
					// 数组对象
					// 写入内容
					for(var i=0;i<obj.length;++i)
					{
						// 写入元素的hashcode
						sb.append(xlsys.base.util.ObjectUtil.hashcode(obj[i]));
						if(i!=(obj.length-1)) sb.append(",");
						// 递归遍历元素
						this.prepareJSONStr(obj[i], hasWriteObjMap);
					}
				}
				sb.append("]");
			}
			else if(dataType==xlsys.base.XLSYS.DATA_TYPE_BIGDECIMAL)
			{
				// 写入类型
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_BIGDECIMAL));
				// 写入内容
				sb.append(",");
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_CONTENT, obj.toString()));
			}
			else if(dataType==xlsys.base.XLSYS.DATA_TYPE_BOOL)
			{
				// 写入类型
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_BOOL));
				// 写入内容
				sb.append(",");
				sb.append(this.getDQMStr(xlsys.base.XLSYS.JSON_OBJ_CONTENT)).append(":").append(obj.toString());
			}
			else if(dataType==xlsys.base.XLSYS.DATA_TYPE_DATE)
			{
				// 写入类型
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_DATE));
				// 写入内容
				sb.append(",");
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_CONTENT, obj.getTime()));
			}
			else if(dataType==xlsys.base.XLSYS.DATA_TYPE_ENTRY)
			{
				// 写入类型
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_ENTRY));
				// 写入内容
				sb.append(",");
				sb.append(this.getDQMStr(xlsys.base.XLSYS.JSON_OBJ_CONTENT)).append(":[");
				// 写入Key的hashcode
				var key = obj.getKey();
				sb.append(xlsys.base.util.ObjectUtil.hashcode(key));
				this.prepareJSONStr(key, hasWriteObjMap);
				// 写入Value的hashcode
				sb.append(",");
				var value = obj.getValue();
				sb.append(xlsys.base.util.ObjectUtil.hashcode(value));
				this.prepareJSONStr(value, hasWriteObjMap);
				sb.append("]");
			}
			else if(dataType==xlsys.base.XLSYS.DATA_TYPE_COLLECTION)
			{
				// 集合
				// 写入类型
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_COLLECTION));
				// 写入名称
				sb.append(",");
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_CLASS, xlsys.base.util.ObjectUtil.className(obj)));
				// 写入内容
				sb.append(",");
				sb.append(this.getDQMStr(xlsys.base.XLSYS.JSON_OBJ_CONTENT)).append(":[");
				var objArr = obj.toArray();
				for(var i=0;i<objArr.length;++i)
				{
					// 写入元素的hashcode
					sb.append(xlsys.base.util.ObjectUtil.hashcode(objArr[i]));
					if(i!=(objArr.length-1)) sb.append(",");
					// 递归遍历元素
					this.prepareJSONStr(objArr[i], hasWriteObjMap);
				}
				sb.append("]");
			}
			else if(dataType==xlsys.base.XLSYS.DATA_TYPE_MAP)
			{
				// Map
				// 写入类型
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_MAP));
				// 写入名称
				sb.append(",");
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_CLASS, xlsys.base.util.ObjectUtil.className(obj)));
				// 写入内容
				sb.append(",");
				sb.append(this.getDQMStr(xlsys.base.XLSYS.JSON_OBJ_CONTENT)).append(":[");
				var keyArr = obj.keys();
				for(var i=0;i<keyArr.length;++i)
				{
					// 写入Key的hashcode
					sb.append(xlsys.base.util.ObjectUtil.hashcode(keyArr[i])).append(",");
					this.prepareJSONStr(keyArr[i], hasWriteObjMap);
					// 写入value的hashcode
					var value = obj.get(keyArr[i]);
					sb.append(xlsys.base.util.ObjectUtil.hashcode(value));
					if(i!=(keyArr.length-1)) sb.append(",");
					this.prepareJSONStr(value, hasWriteObjMap);
				}
				sb.append("]");
			}
			else if(dataType==xlsys.base.XLSYS.DATA_TYPE_OBJECT)
			{
				// 为复杂对象类型
				// 写入类型
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_OBJECT));
				// 写入名称
				sb.append(",");
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_CLASS, xlsys.base.util.ObjectUtil.className(obj)));
				// 放入所有的Field属性对象
				sb.append(",");
				sb.append(this.getDQMStr(xlsys.base.XLSYS.JSON_OBJ_CONTENT)).append(":{");
				var isFirstField = true;
				var needMembers = obj.getSerializableMembers();
				for(var x in needMembers)
				{
					// 写入Field名称:hashcode
					var fieldName = needMembers[x];
					var fieldObj = obj[fieldName];
					if(isFirstField) isFirstField = false;
					else sb.append(",");
					sb.append(this.createJSONAttr(fieldName, xlsys.base.util.ObjectUtil.hashcode(fieldObj)));
					
					// 获取Field的类型
					var fieldDataType = obj.getMemberDataType(fieldName);
					this.prepareJSONStr(fieldObj, hasWriteObjMap, fieldDataType);
				}
				sb.append("}");
			}
			else throw new xlsys.java.lang.Exception("Unsupported Transfer Object!");
			sb.append("}");
			hasWriteObjMap.put(xlsys.base.util.ObjectUtil.hashcode(obj), sb.toString());
		},

		/** 
		 * 将JSON文本转换成JS对象
		 */
		getObjFromJSON : function(jsonStr)
		{
			var allJson = JSON.parse(jsonStr);
			return this.getObjFromJSONObj(allJson);
		},
		
		getObjFromJSONObj : function(allJson)
		{
			var obj = null;
			var firstHashcode = 0;
			// 获取所有引用的对象
			var hasWriteObjMap = new xlsys.java.util.HashMap();
			for(var i=0;i<allJson.length;++i)
			{
				// 读取hashcode
				var jsonObj = allJson[i];
				var hashcode = jsonObj[xlsys.base.XLSYS.JSON_OBJ_HASHCODE];
				if(firstHashcode==0) firstHashcode = hashcode;
				// 读取内容
				var content = jsonObj[xlsys.base.XLSYS.JSON_OBJ_CONTENT];
				hasWriteObjMap.put(hashcode, content);
			}
			// 生成第一个对象
			var rebuiltObjMap = new xlsys.java.util.HashMap();
			obj = this.rebuildJSONObject(firstHashcode, hasWriteObjMap, rebuiltObjMap);
			return obj;
		},

		rebuildJSONObject : function(hashcode, hasWriteObjMap, rebuiltObjMap)
		{
			var obj = null;
			if(rebuiltObjMap.containsKey(hashcode)) obj = rebuiltObjMap.get(hashcode);
			else
			{
				var jsonObj = hasWriteObjMap.get(hashcode);
				// 读取类型
				var type = jsonObj[xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE];
				if(type==xlsys.base.XLSYS.DATA_TYPE_NULL)
				{
					// Do Nothing
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_STR)
				{
					// 读取内容
					obj = jsonObj[xlsys.base.XLSYS.JSON_OBJ_CONTENT];
					// 还原\r和\n
					obj = obj.replace(this.__escapeNRegExp, "\n").replace(this.__escapeRRegExp, "\r");
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_STR_ARRAY)
				{
					// 数组对象
					// 读取内容
					var jsonArray = jsonObj[xlsys.base.XLSYS.JSON_OBJ_CONTENT];
					var strArr = new Array();
					for(var i=0;i<jsonArray.length;++i)
					{
						var tempHashCode = jsonArray[i];
						strArr.push(this.rebuildJSONObject(tempHashCode, hasWriteObjMap, rebuiltObjMap));
					}
					obj = strArr;
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_SHORT)
				{
					obj = jsonObj[xlsys.base.XLSYS.JSON_OBJ_CONTENT];
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_INT)
				{
					obj = jsonObj[xlsys.base.XLSYS.JSON_OBJ_CONTENT];
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_LONG)
				{
					obj = jsonObj[xlsys.base.XLSYS.JSON_OBJ_CONTENT];
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_CHAR)
				{
					obj = jsonObj[xlsys.base.XLSYS.JSON_OBJ_CONTENT];
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_BOOL)
				{
					obj = jsonObj[xlsys.base.XLSYS.JSON_OBJ_CONTENT];
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_FLOAT)
				{
					obj = jsonObj[xlsys.base.XLSYS.JSON_OBJ_CONTENT];
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_DOUBLE)
				{
					obj = jsonObj[xlsys.base.XLSYS.JSON_OBJ_CONTENT];
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_BYTE)
				{
					obj = jsonObj[xlsys.base.XLSYS.JSON_OBJ_CONTENT];
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_BYTE_ARRAY)
				{
					obj = jsonObj[xlsys.base.XLSYS.JSON_OBJ_CONTENT];
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_BIGDECIMAL)
				{
					obj = Number(jsonObj[xlsys.base.XLSYS.JSON_OBJ_CONTENT]);
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_DATE)
				{
					// 读取时间long
					var time = jsonObj[xlsys.base.XLSYS.JSON_OBJ_CONTENT];
					obj = new Date();
					obj.setTime(time);
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_ENTRY)
				{
					var jsonArr = jsonObj[xlsys.base.XLSYS.JSON_OBJ_CONTENT];
					// 读取key
					var tempHashCode = jsonArr[0];
					var key = this.rebuildJSONObject(tempHashCode, hasWriteObjMap, rebuiltObjMap);
					// 读取value
					tempHashCode = jsonArr[1];
					var value = this.rebuildJSONObject(tempHashCode, hasWriteObjMap, rebuiltObjMap);
					obj = new xlsys.java.util.SimpleEntry(key, value);
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_ENTRY_ARRAY)
				{
					// 读取内容
					var jsonArr = jsonObj[xlsys.base.XLSYS.JSON_OBJ_CONTENT];
					var entrys = new Array();
					for(var i=0;i<jsonArr.length;++i)
					{
						// 读取元素的hashcode
						var tempHashCode = jsonArr[i];
						// 添加元素
						entrys.push(this.rebuildJSONObject(tempHashCode, hasWriteObjMap, rebuiltObjMap));
					}
					obj = entrys;
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_OBJECT_ARRAY)
				{
					// 数组对象
					// 读取内容
					var jsonArr = jsonObj[xlsys.base.XLSYS.JSON_OBJ_CONTENT];
					var objArr = new Array();
					for(var i=0;i<jsonArr.length;++i)
					{
						// 读取元素的hashcode
						var tempHashCode = jsonArr[i];
						// 添加元素
						objArr.push(this.rebuildJSONObject(tempHashCode, hasWriteObjMap, rebuiltObjMap));
					}
					obj = objArr;
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_COLLECTION)
				{
					// 集合
					// 读取名称
					var objClassName = jsonObj[xlsys.base.XLSYS.JSON_OBJ_CLASS];
					obj = xlsys.base.util.ObjectUtil.tryInitObj(objClassName);
					if(obj==null) throw new xlsys.java.lang.Exception("Unsupported Class : " + objClassName);
					// 先将对象放入重建Map, 以免出现死循环
					rebuiltObjMap.put(hashcode, obj);
					// 读取内容
					var collection = obj;
					var jsonArr = jsonObj[xlsys.base.XLSYS.JSON_OBJ_CONTENT];
					for(var i=0;i<jsonArr.length;++i)
					{
						// 读取元素的hashcode
						var tempHashCode = jsonArr[i];
						// 添加元素
						collection.add(this.rebuildJSONObject(tempHashCode, hasWriteObjMap, rebuiltObjMap));
					}
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_MAP)
				{
					// Map
					// 读取名称
					var objClassName = jsonObj[xlsys.base.XLSYS.JSON_OBJ_CLASS];
					obj = xlsys.base.util.ObjectUtil.tryInitObj(objClassName);
					if(obj==null) throw "Unsupported Class : " + objClassName;
					// 先将对象放入重建Map, 以免出现死循环
					rebuiltObjMap.put(hashcode, obj);
					// 读取内容
					var map = obj;
					var jsonArr = jsonObj[xlsys.base.XLSYS.JSON_OBJ_CONTENT];
					for(var i=0;i<jsonArr.length;i+=2)
					{
						// 读取key
						var tempHashCode = jsonArr[i];
						var key = this.rebuildJSONObject(tempHashCode, hasWriteObjMap, rebuiltObjMap);
						// 读取value
						tempHashCode = jsonArr[i+1];
						var value = this.rebuildJSONObject(tempHashCode, hasWriteObjMap, rebuiltObjMap);
						// 放入map
						map.put(key, value);
					}
				}
				else
				{
					// 为复杂对象类型
					// 读取名称
					var objClassName = jsonObj[xlsys.base.XLSYS.JSON_OBJ_CLASS];
					obj = xlsys.base.util.ObjectUtil.tryInitObj(objClassName);
					if(obj==null) throw "Unsupported Class : " + objClassName;
					// 先将对象放入重建Map, 以免出现死循环
					rebuiltObjMap.put(hashcode, obj);
					// 读取内容
					var allField = jsonObj[xlsys.base.XLSYS.JSON_OBJ_CONTENT];
					for(var fieldName in allField)
					{
						// 获取Field对象
						var tempHashCode = allField[fieldName];
						var fieldObj = this.rebuildJSONObject(tempHashCode, hasWriteObjMap, rebuiltObjMap);
						obj[fieldName] = fieldObj;
					}
				}
				rebuiltObjMap.put(hashcode, obj);
			}
			return obj;
		},
		
		readInternalObject : function(bytes)
		{
			if(bytes==null||bytes.length==0) return null;
			var dis = new xlsys.base.io.util.ByteDataInputStream(bytes);
			var firstHashcode = 0;
			// 获取所有引用的对象
			var hasWriteObjMap = new xlsys.java.util.HashMap();
			// 读取总长度
			var totalLength = dis.readInt();
			// 读取所有内容
			var totalContent = new Array(totalLength);
			dis.readFully(totalContent, 0, totalLength);
			var tempDis = new xlsys.base.io.util.ByteDataInputStream(totalContent);
			while(tempDis.available()>0)
			{
				// 读取hashcode
				var hashcode = tempDis.readInt();
				if(firstHashcode==0) firstHashcode = hashcode;
				// 读取长度
				var length = tempDis.readInt();
				// 读取内容
				var content = new Array(length);
				tempDis.readFully(content, 0, length);
				hasWriteObjMap.put(hashcode, content);
			}
			// 生成第一个对象
			var rebuiltObjMap = new xlsys.java.util.HashMap();
			return this.rebuildInternalObject(firstHashcode, hasWriteObjMap, rebuiltObjMap);
		},
		
		rebuildInternalObject : function(hashcode, hasWriteObjMap, rebuiltObjMap)
		{
			var obj = null;
			if(rebuiltObjMap.containsKey(hashcode)) obj = rebuiltObjMap.get(hashcode);
			else
			{
				var content = hasWriteObjMap.get(hashcode);
				var dis = new xlsys.base.io.util.ByteDataInputStream(content);
				// 读取类型
				var type = dis.readByte();
				if(type==xlsys.base.XLSYS.DATA_TYPE_NULL)
				{
						// Do Nothing
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_STR)
				{
					// 读取长度
					var length = dis.readInt();
					// 读取内容
					var bytes = new Array(length);
					dis.readFully(bytes, 0, length);
					obj = xlsys.base.util.StringUtil.bytesToString(bytes);
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_STR_ARRAY)
				{
					// 数组对象
					// 读取长度
					var length = dis.readInt();
					// 读取内容
					obj = new Array();
					var bytes = new Array(length);
					dis.readFully(bytes, 0, length);
					var tempIn = new xlsys.base.io.util.ByteDataInputStream(bytes);
					while(tempIn.available()>0)
					{
						// 读取元素的hashcode
						var tempHashcode = tempIn.readInt();
						// 添加元素
						obj.push(this.rebuildInternalObject(tempHashcode, hasWriteObjMap, rebuiltObjMap));
					}
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_SHORT)
				{
					obj = dis.readShort();
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_INT)
				{
					obj = dis.readInt();
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_LONG)
				{
					obj = dis.readLong();
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_CHAR)
				{
					obj = dis.readChar();
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_BOOL)
				{
					obj = dis.readByte()==1?true:false;
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_FLOAT)
				{
					throw new Error("Unsupported!")
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_DOUBLE)
				{
					throw new Error("Unsupported!")
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_BYTE)
				{
					obj = dis.readByte();
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_BYTE_ARRAY)
				{
					// 读取长度
					var length = dis.readInt();
					// 读取内容
					var bytes = new Array(length);
					dis.readFully(bytes, 0, length);
					obj = bytes;
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_BIGDECIMAL)
				{
					// 读取数字字符串的hashcode
					var numHashcode = dis.readInt();
					var numStr = this.rebuildInternalObject(numHashcode, hasWriteObjMap, rebuiltObjMap);
					obj = Number(numStr);
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_DATE)
				{
					// 读取时间long
					var time = dis.readLong();
					obj = new Date(time);
					obj.setTime(time);
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_ENTRY)
				{
					// 读取长度
					var length = dis.readInt();
					// 读取内容
					var bytes = new Array(length);
					dis.readFully(bytes, 0, length);
					var tempIn = new xlsys.base.io.util.ByteDataInputStream(bytes);
					// 读取key
					var tempHashcode = tempIn.readInt();
					var key = this.rebuildInternalObject(tempHashcode, hasWriteObjMap, rebuiltObjMap);
					// 读取value
					tempHashcode = tempIn.readInt();
					var value = this.rebuildInternalObject(tempHashcode, hasWriteObjMap, rebuiltObjMap);
					obj = new xlsys.java.util.SimpleEntry(key, value);
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_ENTRY_ARRAY)
				{
					// 数组对象
					// 读取长度
					var length = dis.readInt();
					// 读取内容
					var obj = new Array();
					var bytes = new Array(length);
					dis.readFully(bytes, 0, length);
					var tempIn = new xlsys.base.io.util.ByteDataInputStream(bytes);
					while(tempIn.available()>0)
					{
						// 读取元素的hashcode
						var tempHashcode = tempIn.readInt();
						// 添加元素
						obj.push(this.rebuildInternalObject(tempHashcode, hasWriteObjMap, rebuiltObjMap));
					}
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_OBJECT_ARRAY)
				{
					// 数组对象
					// 读取长度
					var length = dis.readInt();
					// 读取内容
					var obj = new Array();
					var bytes = new Array(length);
					dis.readFully(bytes, 0, length);
					var tempIn = new xlsys.base.io.util.ByteDataInputStream(bytes);
					while(tempIn.available()>0)
					{
						// 读取元素的hashcode
						var tempHashcode = tempIn.readInt();
						// 添加元素
						obj.push(this.rebuildInternalObject(tempHashcode, hasWriteObjMap, rebuiltObjMap));
					}
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_COLLECTION)
				{
					// 集合
					// 读取名称
					var nameHashcode = dis.readInt();
					var objClassName = this.rebuildInternalObject(nameHashcode, hasWriteObjMap, rebuiltObjMap);
					obj = xlsys.base.util.ObjectUtil.tryInitObj(objClassName);
					if(obj==null) throw new Error("Unsupported");
					// 先将对象放入重建Map, 以免出现死循环
					rebuiltObjMap.put(hashcode, obj);
					// 读取长度
					var length = dis.readInt();
					// 读取内容
					var bytes = new Array(length);
					dis.readFully(bytes, 0, length);
					var tempIn = new xlsys.base.io.util.ByteDataInputStream(bytes);
					while(tempIn.available()>0)
					{
						// 读取元素的hashcode
						var tempHashcode = tempIn.readInt();
						// 添加元素
						obj.add(this.rebuildInternalObject(tempHashcode, hasWriteObjMap, rebuiltObjMap));
					}
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_MAP)
				{
					// Map
					// 读取名称
					var nameHashcode = dis.readInt();
					var objClassName = this.rebuildInternalObject(nameHashcode, hasWriteObjMap, rebuiltObjMap);
					obj = xlsys.base.util.ObjectUtil.tryInitObj(objClassName);
					if(obj==null) throw new Error("Unsupported");
					// 先将对象放入重建Map, 以免出现死循环
					rebuiltObjMap.put(hashcode, obj);
					// 读取长度
					var length = dis.readInt();
					// 读取内容
					var bytes = new Array(length);
					dis.readFully(bytes, 0, length);
					var tempIn = new xlsys.base.io.util.ByteDataInputStream(bytes);
					while(tempIn.available()>0)
					{
						// 读取key
						var tempHashcode = tempIn.readInt();
						var key = this.rebuildInternalObject(tempHashcode, hasWriteObjMap, rebuiltObjMap);
						// 读取value
						tempHashcode = tempIn.readInt();
						var value = this.rebuildInternalObject(tempHashcode, hasWriteObjMap, rebuiltObjMap);
						// 放入map
						obj.put(key, value);
					}
				}
				else if(type==xlsys.base.XLSYS.DATA_TYPE_EXCEPTION)
				{
					// Exception
					// 读取名称
					var nameHashcode = dis.readInt();
					var objClassName = this.rebuildInternalObject(nameHashcode, hasWriteObjMap, rebuiltObjMap);
					obj = xlsys.base.util.ObjectUtil.tryInitObj(objClassName);
					if(obj==null) throw new Error("Unsupported");
					// 读取长度
					var length = dis.readInt();
					// 读取内容
					var bytes = new Array(length);
					dis.readFully(bytes, 0, length);
					// 填充对象Field
					var tempIn = new xlsys.base.io.util.ByteDataInputStream(bytes);
					// 读取虚拟field : msg
					// 读取名称(这里不使用)
					var fieldNameHashcode = tempIn.readInt();
					this.rebuildInternalObject(fieldNameHashcode, hasWriteObjMap, rebuiltObjMap);
					// 读取内容
					var msgHashcode = tempIn.readInt();
					var message = this.rebuildInternalObject(msgHashcode, hasWriteObjMap, rebuiltObjMap);
					// 读取虚拟field : stackTrace
					// 读取名称(这里不使用)
					fieldNameHashcode = tempIn.readInt();
					this.rebuildInternalObject(fieldNameHashcode, hasWriteObjMap, rebuiltObjMap);
					// 读取内容
					var stackTraceHashcode = tempIn.readInt();
					var stackTrace = this.rebuildInternalObject(stackTraceHashcode, hasWriteObjMap, rebuiltObjMap);
					// 读取虚拟field : cause
					// 读取名称(这里不使用)
					fieldNameHashcode = tempIn.readInt();
					this.rebuildInternalObject(fieldNameHashcode, hasWriteObjMap, rebuiltObjMap);
					// 读取内容
					var causeHashcode = tempIn.readInt();
					var cause = this.rebuildInternalObject(causeHashcode, hasWriteObjMap, rebuiltObjMap);
					// 创建对象
					obj.msg = message;
					obj.stackTrace = stackTrace;
					obj.cause = cause;
					// 先将对象放入重建Map, 以免出现死循环
					rebuiltObjMap.put(hashcode, obj);
				}
				else
				{
					// 为复杂对象类型
					// 读取名称
					var nameHashcode = dis.readInt();
					var objClassName = this.rebuildInternalObject(nameHashcode, hasWriteObjMap, rebuiltObjMap);
					obj = xlsys.base.util.ObjectUtil.tryInitObj(objClassName);
					if(obj==null) throw new Error("Unsupported");
					// 先将对象放入重建Map, 以免出现死循环
					rebuiltObjMap.put(hashcode, obj);
					// 读取长度
					var length = dis.readInt();
					// 读取内容
					var bytes = new Array(length);
					dis.readFully(bytes, 0, length);
					// 填充对象Field
					var tempIn = new xlsys.base.io.util.ByteDataInputStream(bytes);
					while(tempIn.available()>0)
					{
						// 读取Field名称
						var fieldNameHashcode = tempIn.readInt();
						var fieldName = this.rebuildInternalObject(fieldNameHashcode, hasWriteObjMap, rebuiltObjMap);
						// 获取Field对象
						var tempHashcode = tempIn.readInt();
						var fieldObj = this.rebuildInternalObject(tempHashcode, hasWriteObjMap, rebuiltObjMap);
						// 设置field到obj对象
						obj[fieldName] = fieldObj;
					}
				}
				rebuiltObjMap.put(hashcode, obj);
			}
			return obj;
		},
		
		getInternalObjectBytes : function(obj)
		{
			// 获取参数的对象中包含的所有对象的字节数组Map,复杂类型均使用hashcode代表
			var hasWriteObjMap = new xlsys.java.util.LinkedHashMap();
			this.prepareInternalBytes(obj, hasWriteObjMap);
			// 将Map写成byte数组
			var baos = new xlsys.base.io.util.ByteDataOutputStream();
			var keys = hasWriteObjMap.keys();
			for(var i=0;i<keys.length;++i)
			{
				// 写入hashcode
				var hashcode = keys[i];
				baos.writeInt(hashcode);
				// 写入长度
				var content = hasWriteObjMap.get(hashcode);
				baos.writeInt(content.length);
				// 写入内容
				baos.write(content);
			}
			var out = new xlsys.base.io.util.ByteDataOutputStream();
			var temp = baos.toByteArray();
			// 写入总长度
			out.writeInt(temp.length);
			// 写入总内容
			out.write(temp);
			return out.toByteArray();
		},
		
		prepareInternalBytes : function(obj, hasWriteObjMap, dataType)
		{
			if(hasWriteObjMap.containsKey(xlsys.base.util.ObjectUtil.hashcode(obj))) return;
			// 先将当前obj放入,以免子属性递归时重复引用造成无限循环
			hasWriteObjMap.put(xlsys.base.util.ObjectUtil.hashcode(obj), null);
			var data = null;
			var out = new xlsys.base.io.util.ByteDataOutputStream();
			if(obj==null)
			{
				// 写入类型
				out.writeByte(xlsys.base.XLSYS.DATA_TYPE_NULL);
			}
			else
			{
				if(dataType==null)
				{
					// 没有dataType, 则先根据数据类型来尝试获取dataType
					if(qx.lang.Type.isString(obj)) dataType = xlsys.base.XLSYS.DATA_TYPE_STR;
					else if(qx.lang.Type.isArray(obj)) dataType = xlsys.base.XLSYS.DATA_TYPE_OBJECT_ARRAY;
					else if(qx.lang.Type.isNumber(obj)) dataType = xlsys.base.XLSYS.DATA_TYPE_BIGDECIMAL;
					else if(qx.lang.Type.isBoolean(obj)) dataType = xlsys.base.XLSYS.DATA_TYPE_BOOL;
					else if(qx.lang.Type.isDate(obj)) dataType = xlsys.base.XLSYS.DATA_TYPE_DATE;
					else if(qx.Class.implementsInterface(obj, xlsys.java.util.Entry)) dataType = xlsys.base.XLSYS.DATA_TYPE_ENTRY;
					else if(qx.Class.implementsInterface(obj, xlsys.java.util.Collection)) dataType = xlsys.base.XLSYS.DATA_TYPE_COLLECTION;
					else if(qx.Class.implementsInterface(obj, xlsys.java.util.Map)) dataType = xlsys.base.XLSYS.DATA_TYPE_MAP;
					else if(obj instanceof xlsys.java.lang.Exception) dataType = xlsys.base.XLSYS.DATA_TYPE_EXCEPTION;
					else dataType = xlsys.base.XLSYS.DATA_TYPE_OBJECT;
				}
				if(dataType==xlsys.base.XLSYS.DATA_TYPE_STR)
				{
					var bytes = xlsys.base.util.StringUtil.stringToBytes(obj);
					// 写入类型
					out.writeByte(xlsys.base.XLSYS.DATA_TYPE_STR);
					// 写入长度
					out.writeInt(bytes.length);
					// 写入内容
					out.write(bytes);
				}
				else if(dataType==xlsys.base.XLSYS.DATA_TYPE_SHORT)
				{
					out.writeByte(xlsys.base.XLSYS.DATA_TYPE_SHORT);
					out.writeShort(obj);
				}
				else if(dataType==xlsys.base.XLSYS.DATA_TYPE_INT)
				{
					out.writeByte(xlsys.base.XLSYS.DATA_TYPE_INT);
					out.writeInt(obj);
				}
				else if(dataType==xlsys.base.XLSYS.DATA_TYPE_LONG)
				{
					out.writeByte(xlsys.base.XLSYS.DATA_TYPE_LONG);
					out.writeLong(obj);
				}
				else if(dataType==xlsys.base.XLSYS.DATA_TYPE_CHAR)
				{
					out.writeByte(xlsys.base.XLSYS.DATA_TYPE_CHAR);
					out.writeChar(obj);
				}
				else if(dataType==xlsys.base.XLSYS.DATA_TYPE_BOOL)
				{
					out.writeByte(xlsys.base.XLSYS.DATA_TYPE_BOOL);
					out.writeByte(obj?1:0);
				}
				else if(dataType==xlsys.base.XLSYS.DATA_TYPE_BYTE)
				{
					out.writeByte(xlsys.base.XLSYS.DATA_TYPE_BYTE);
					out.writeByte(obj);
				}
				else if(dataType==xlsys.base.XLSYS.DATA_TYPE_BYTE_ARRAY)
				{
					out.writeByte(xlsys.base.XLSYS.DATA_TYPE_BYTE_ARRAY);
					out.writeInt(obj.length);
					out.write(obj);
				}
				else if(dataType==xlsys.base.XLSYS.DATA_TYPE_BIGDECIMAL)
				{
					out.writeByte(xlsys.base.XLSYS.DATA_TYPE_BIGDECIMAL);
					var numStr = "" + obj;
					// 写入数字文本的hashcode
					out.writeInt(xlsys.base.util.ObjectUtil.hashcode(numStr));
					this.prepareInternalBytes(numStr, hasWriteObjMap);
				}
				else if(dataType==xlsys.base.XLSYS.DATA_TYPE_DATE)
				{
					out.writeByte(xlsys.base.XLSYS.DATA_TYPE_DATE);
					// 写入时间long
					out.writeLong(obj.getTime());
				}
				else if(dataType==xlsys.base.XLSYS.DATA_TYPE_ENTRY)
				{
					// 写入类型
					out.writeByte(xlsys.base.XLSYS.DATA_TYPE_ENTRY);
					var tempOut = new xlsys.base.io.util.ByteDataOutputStream();
					// 写入Key
					var key = obj.getKey();
					tempOut.writeInt(xlsys.base.util.ObjectUtil.hashcode(key));
					this.prepareInternalBytes(key, hasWriteObjMap);
					// 写入Value
					var value = obj.getValue();
					tempOut.writeInt(xlsys.base.util.ObjectUtil.hashcode(value));
					this.prepareInternalBytes(value, hasWriteObjMap);
					var arrBytes = tempOut.toByteArray();
					// 写入长度
					out.writeInt(arrBytes.length);
					// 写入内容
					out.write(arrBytes);
				}
				else if(dataType==xlsys.base.XLSYS.DATA_TYPE_ENTRY_ARRAY||
						dataType==xlsys.base.XLSYS.DATA_TYPE_OBJECT_ARRAY||
						dataType==xlsys.base.XLSYS.DATA_TYPE_STR_ARRAY)
				{
					// 数组对象
					// 写入类型
					out.writeByte(dataType);
					var tempOut = new xlsys.base.io.util.ByteDataOutputStream();
					for(var i=0;i<obj.length;++i)
					{
						var temp = obj[i];
						// 写入元素的hashcode
						tempOut.writeInt(xlsys.base.util.ObjectUtil.hashcode(temp));
						// 递归遍历元素
						this.prepareInternalBytes(temp, hasWriteObjMap);
					}
					var arrBytes = tempOut.toByteArray();
					// 写入长度
					out.writeInt(arrBytes.length);
					// 写入内容
					out.write(arrBytes);
				}
				else if(dataType==xlsys.base.XLSYS.DATA_TYPE_COLLECTION)
				{
					// 集合
					// 写入类型
					out.writeByte(xlsys.base.XLSYS.DATA_TYPE_COLLECTION);
					// 写入名称
					var name = xlsys.base.util.ObjectUtil.className(obj);
					out.writeInt(xlsys.base.util.ObjectUtil.hashcode(name));
					this.prepareInternalBytes(name, hasWriteObjMap);
					var tempOut = new xlsys.base.io.util.ByteDataOutputStream();
					var collectionArr = obj.toArray();
					for(var i=0;i<collectionArr.length;++i)
					{
						var temp = collectionArr[i];
						// 写入元素的hashcode
						tempOut.writeInt(xlsys.base.util.ObjectUtil.hashcode(temp));
						// 递归遍历元素
						this.prepareInternalBytes(temp, hasWriteObjMap);
					}
					var arrBytes = tempOut.toByteArray();
					// 写入长度
					out.writeInt(arrBytes.length);
					// 写入内容
					out.write(arrBytes);
				}
				else if(dataType==xlsys.base.XLSYS.DATA_TYPE_MAP)
				{
					// Map
					// 写入类型
					out.writeByte(xlsys.base.XLSYS.DATA_TYPE_MAP);
					// 写入名称
					var name = xlsys.base.util.ObjectUtil.className(obj);
					out.writeInt(xlsys.base.util.ObjectUtil.hashcode(name));
					this.prepareInternalBytes(name, hasWriteObjMap);
					var tempOut = new xlsys.base.io.util.ByteDataOutputStream();
					var keys = obj.keys();
					for(var i=0;i<keys.length;++i)
					{
						var key = keys[i];
						var value = obj.get(key);
						// 写入Key
						tempOut.writeInt(xlsys.base.util.ObjectUtil.hashcode(key));
						this.prepareInternalBytes(key, hasWriteObjMap);
						// 写入Value
						tempOut.writeInt(xlsys.base.util.ObjectUtil.hashcode(value));
						this.prepareInternalBytes(value, hasWriteObjMap);
					}
					var arrBytes = tempOut.toByteArray();
					// 写入长度
					out.writeInt(arrBytes.length);
					// 写入内容
					out.write(arrBytes);
				}
				else if(dataType==xlsys.base.XLSYS.DATA_TYPE_EXCEPTION)
				{
					// 写入类型
					out.writeByte(xlsys.base.XLSYS.DATA_TYPE_EXCEPTION);
					// 写入名称
					var name = xlsys.base.util.ObjectUtil.className(obj);
					out.writeInt(xlsys.base.util.ObjectUtil.hashcode(name));
					this.prepareInternalBytes(name, hasWriteObjMap);
					var tempOut = new xlsys.base.io.util.ByteDataOutputStream();
					// 插入虚拟field : msg
					var fieldName = "msg";
					var message = obj.getMessage();
					if(message==null) message = "";
					// 写入Field名称
					tempOut.writeInt(xlsys.base.util.ObjectUtil.hashcode(fieldName));
					this.prepareInternalBytes(fieldName, hasWriteObjMap);
					// 写入Field的hashcode
					tempOut.writeInt(xlsys.base.util.ObjectUtil.hashcode(message));
					this.prepareInternalBytes(message, hasWriteObjMap);
					// 插入虚拟field : stackTrace
					fieldName = "stackTrace";
					var stackTrace = obj.getStackTrace();
					// 写入Field名称
					tempOut.writeInt(xlsys.base.util.ObjectUtil.hashcode(fieldName));
					this.prepareInternalBytes(fieldName, hasWriteObjMap);
					// 写入Field的hashcode
					tempOut.writeInt(xlsys.base.util.ObjectUtil.hashcode(stackTrace));
					this.prepareInternalBytes(stackTrace, hasWriteObjMap, xlsys.base.XLSYS.DATA_TYPE_STR_ARRAY);
					// 插入虚拟field : cause
					fieldName = "cause";
					var cause = obj.getCause();
					// 写入Field名称
					tempOut.writeInt(xlsys.base.util.ObjectUtil.hashcode(fieldName));
					this.prepareInternalBytes(fieldName, hasWriteObjMap);
					// 写入Field的hashcode
					tempOut.writeInt(xlsys.base.util.ObjectUtil.hashcode(cause));
					this.prepareInternalBytes(cause, hasWriteObjMap);
					// 写入长度
					var fieldsData = tempOut.toByteArray();
					out.writeInt(fieldsData.length);
					// 写入内容
					out.write(fieldsData);
				}
				else if(dataType==xlsys.base.XLSYS.DATA_TYPE_OBJECT)
				{
					// 为复杂对象类型
					// 写入类型
					out.writeByte(xlsys.base.XLSYS.DATA_TYPE_OBJECT);
					// 写入名称
					var name = xlsys.base.util.ObjectUtil.className(obj);
					out.writeInt(xlsys.base.util.ObjectUtil.hashcode(name));
					this.prepareInternalBytes(name, hasWriteObjMap);
					// 放入所有的member
					var tempOut = new xlsys.base.io.util.ByteDataOutputStream();
					var fields = obj.getSerializableMembers();
					for(var i=0;i<fields.length;++i)
					{
						var fieldName = fields[i];
						// 写入Field名称
						tempOut.writeInt(xlsys.base.util.ObjectUtil.hashcode(fieldName));
						this.prepareInternalBytes(fieldName, hasWriteObjMap);
						// 写入Field的hashcode
						var fieldObj = obj[fieldName];
						tempOut.writeInt(xlsys.base.util.ObjectUtil.hashcode(fieldObj));
						// 获取Field的类型
						var fieldDataType = obj.getMemberDataType(fieldName);
						this.prepareInternalBytes(fieldObj, hasWriteObjMap, fieldDataType);
					}
					// 写入长度
					var fieldsData = tempOut.toByteArray();
					out.writeInt(fieldsData.length);
					// 写入内容
					out.write(fieldsData);
				}
			}
			data = out.toByteArray();
			hasWriteObjMap.put(xlsys.base.util.ObjectUtil.hashcode(obj), data);
		},
		
		bytesToHexStr : function(bytes)
		{
			var sb = new xlsys.java.lang.StringBuilder();
			for(var i=0;i<bytes.length;++i)
			{
				var temp = bytes[i].toString(16);
				if(temp.length<2) temp = "0" + temp;
				sb.append(temp);
			}
			return sb.toString();
		},
		
		hexStrToBytes : function(hexStr)
		{
			var bytes = new Array();
			for(var i=0;i<hexStr.length;i+=2)
			{
				bytes.push(parseInt(hexStr.substring(i, i+2), 16));
			}
			return bytes;
		},
		
		getCommonJSONFromObj : function(seriObj, dataType)
		{
			var sb = new xlsys.java.lang.StringBuilder();
			if(seriObj==null) sb.append("null");
			else if(qx.lang.Type.isArray(seriObj))
			{
				if((dataType!=null&&dataType==xlsys.base.XLSYS.DATA_TYPE_BYTE_ARRAY)||
					(seriObj.length>0&&qx.lang.Type.isNumber(seriObj[0])&&qx.lang.Type.isNumber(seriObj[seriObj.length-1])))
				{
					sb.append("{");
					sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_BYTE_ARRAY)).append(",");
					sb.append(this.createJSONAttr("value", this.bytesToHexStr(seriObj)));
					sb.append("}");
				}
				else
				{
					sb.append("[");
					for(var i=0;i<seriObj.length;++i)
					{
						sb.append(this.getCommonJSONFromObj(seriObj[i]));
						if(i!=seriObj.length-1) sb.append(",");
					}
					sb.append("]");
				}
			}
			else if(qx.lang.Type.isBoolean(seriObj)) sb.append(seriObj);
			else if(qx.lang.Type.isDate(seriObj))
			{
				sb.append("{");
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_DATE)).append(",");
				sb.append(this.createJSONAttr("value", seriObj.getTime()));
				sb.append("}");
			}
			else if(qx.lang.Type.isError(seriObj)||seriObj instanceof xlsys.java.lang.Exception)
			{
				sb.append("{");
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_EXCEPTION)).append(",");
				if(qx.lang.Type.isError(seriObj)) sb.append(this.createJSONAttr("value", seriObj.message));
				else sb.append(this.createJSONAttr("value", seriObj.getMessage()));
				sb.append("}");
			}
			else if(qx.lang.Type.isNumber(seriObj)) sb.append(seriObj);
			else if(qx.lang.Type.isString(seriObj))
			{
				if(seriObj.indexOf("\n")>=0||seriObj.indexOf("\r")>=0)
				{
					var temp = seriObj.replace(this.__nRegExp, "&#10;").replace(this.__rRegExp, "&#13;");
					sb.append("\"").append(temp).append("\"");
				}
				else sb.append("\"").append(seriObj).append("\"");
			}
			else if(qx.Class.implementsInterface(seriObj, xlsys.java.util.Collection))
			{
				sb.append("{");
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_COLLECTION)).append(",");
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_CLASS, xlsys.base.util.ObjectUtil.className(seriObj))).append(",");
				sb.append("\"value\":").append("[");
				var tempArr = seriObj.toArray();
				for(var i=0;i<tempArr.length;++i)
				{
					sb.append(this.getCommonJSONFromObj(tempArr[i]));
					if(i!=tempArr.length-1) sb.append(",");
				}
				sb.append("]");
				sb.append("}");
			}
			else if(qx.Class.implementsInterface(seriObj, xlsys.java.util.Entry))
			{
				sb.append("{");
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_ENTRY)).append(",");
				sb.append("\"key\":").append(this.getCommonJSONFromObj(seriObj.getKey()));
				sb.append("\"value\":").append(this.getCommonJSONFromObj(seriObj.getValue()));
				sb.append("}");
			}
			else if(qx.Class.implementsInterface(seriObj, xlsys.java.util.Map))
			{
				sb.append("{");
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_MAP)).append(",");
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_CLASS, xlsys.base.util.ObjectUtil.className(seriObj))).append(",");
				sb.append("\"value\":").append("[");
				var keyArr = seriObj.keys();
				for(var i=0;i<keyArr.length;++i)
				{
					sb.append(this.getCommonJSONFromObj(keyArr[i])).append(",");
					sb.append(this.getCommonJSONFromObj(seriObj.get(keyArr[i])));
					if(i!=keyArr.length-1) sb.append(",");
				}
				sb.append("]");
				sb.append("}");
			}
			else if(seriObj instanceof xlsys.base.dataset.DataSetRow)
			{
				sb.append("{");
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_DATA_SET_ROW)).append(",");
				sb.append("\"value\":").append("[");
				var cellArr = seriObj.getCells().toArray();
				for(var i=0;i<cellArr.length;++i)
				{
					sb.append(this.getCommonJSONFromObj(cellArr[i].getContent()));
					if(i!=cellArr.length-1) sb.append(",");
				}
				sb.append("]");
				sb.append("}");
			}
			else if(seriObj instanceof xlsys.java.io.Serializable)
			{
				sb.append("{");
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE, xlsys.base.XLSYS.DATA_TYPE_OBJECT)).append(",");
				sb.append(this.createJSONAttr(xlsys.base.XLSYS.JSON_OBJ_CLASS, xlsys.base.util.ObjectUtil.className(seriObj))).append(",");
				var fields = seriObj.getSerializableMembers();
				for(var i=0;i<fields.length;++i)
				{
					var fieldName = fields[i];
					var fieldDataType = seriObj.getMemberDataType(fieldName);
					sb.append("\"").append(fieldName).append("\"").append(":").append(this.getCommonJSONFromObj(seriObj[fieldName], fieldDataType));
					if(i!=fields.length-1) sb.append(",");
				}
				sb.append("}");
			}
			return sb.toString();
		},
		
		getObjFromCommonJSON : function(jsonStr)
		{
			var allJson = JSON.parse(jsonStr);
			return this._rebuildObjFromCommonJSON(allJson);
		},
		
		_rebuildObjFromCommonJSON : function(jsonObj)
		{
			var obj = null;
			if(jsonObj==null) obj = null;
			if(qx.lang.Type.isArray(jsonObj))
			{
				obj = new Array();
				for(var i=0;i<jsonObj.length;++i)
				{
					obj.push(this._rebuildObjFromCommonJSON(jsonObj[i]));
				}
			}
			else if(qx.lang.Type.isBoolean(jsonObj)) obj = jsonObj;
			else if(qx.lang.Type.isNumber(jsonObj)) obj = jsonObj;
			else if(qx.lang.Type.isString(jsonObj))
			{
				obj = jsonObj;
				// 还原\r和\n
				obj = obj.replace("&#10;", "\n").replace("&#13;", "\r");
			}
			else if(qx.lang.Type.isObject(jsonObj))
			{
				// 获取类型
				var dataType = jsonObj[xlsys.base.XLSYS.JSON_OBJ_DATA_TYPE];
				var className = jsonObj[xlsys.base.XLSYS.JSON_OBJ_CLASS];
				if(dataType==xlsys.base.XLSYS.DATA_TYPE_DATE)
				{
					obj = new Date();
					obj.setTime(jsonObj["value"]);
				}
				else if(dataType==xlsys.base.XLSYS.DATA_TYPE_EXCEPTION)
				{
					obj = new xlsys.java.lang.Exception(jsonObj["value"]);
				}
				else if(dataType==xlsys.base.XLSYS.DATA_TYPE_COLLECTION)
				{
					obj = xlsys.base.util.ObjectUtil.tryInitObj(className);
					if(obj==null) throw new Error("Unsupported");
					var contentJsonArr = jsonObj["value"];
					for(var i=0;i<contentJsonArr.length;++i)
					{
						obj.add(this._rebuildObjFromCommonJSON(contentJsonArr[i]));
					}
				}
				else if(dataType==xlsys.base.XLSYS.DATA_TYPE_ENTRY)
				{
					var key = this._rebuildObjFromCommonJSON(jsonObj["key"]);
					var value = this._rebuildObjFromCommonJSON(jsonObj["value"]);
					obj = new xlsys.java.util.SimpleEntry(key, value);
				}
				else if(dataType==xlsys.base.XLSYS.DATA_TYPE_MAP)
				{
					obj = xlsys.base.util.ObjectUtil.tryInitObj(className);
					if(obj==null) throw new Error("Unsupported");
					var contentJsonArr = jsonObj["value"];
					for(var i=0;i<contentJsonArr.length;i+=2)
					{
						var key = this._rebuildObjFromCommonJSON(contentJsonArr[i]);
						var value = this._rebuildObjFromCommonJSON(contentJsonArr[i+1]);
						obj.put(key, value);
					}
				}
				else if(dataType==xlsys.base.XLSYS.DATA_TYPE_BYTE_ARRAY)
				{
					var hexStr = jsonObj["value"];
					obj = this.hexStrToBytes(hexStr);
				}
				else if(dataType==xlsys.base.XLSYS.DATA_TYPE_DATA_SET_ROW)
				{
					var row = new xlsys.base.dataset.DataSetRow();
					var contentJsonArr = jsonObj["value"];
					for(var i=0;i<contentJsonArr.length;++i)
					{
						var dsc = new xlsys.base.dataset.DataSetCell();
						dsc.setContent(this._rebuildObjFromCommonJSON(contentJsonArr[i]));
						row.getCells().add(dsc);
					}
					obj = row;
				}
				else if(dataType==xlsys.base.XLSYS.DATA_TYPE_OBJECT)
				{
					obj = xlsys.base.util.ObjectUtil.tryInitObj(className);
					if(obj==null) throw new Error("Unsupported");
					var fields = obj.getSerializableMembers();
					for(var i=0;i<fields.length;++i)
					{
						var fieldName = fields[i];
						obj[fieldName] = this._rebuildObjFromCommonJSON(jsonObj[fieldName]);
					}
				}
			}
			return obj;
		}
	}
});