/**
 * 日志基类
 */
qx.Class.define("xlsys.base.log.Log", 
{
	type : "abstract",
	extend : qx.core.Object,
	
	statics :
	{
		LOG_LEVEL_DEBUG : 50,
		LOG_LEVEL_INFO : 40,
		LOG_LEVEL_WARN : 30,
		LOG_LEVEL_ERROR : 20,
		LOG_LEVEL_FATAL : 10,
		LOG_LEVEL_NONE : 0
	},
	
	construct: function(logLevel)
	{
		this.logLevel = logLevel;
	},
	
	members :
	{
		logLevel : null,
		
		/**
		 * 需要重载以下方法:
		 * print(content);
		 * print(level, content)
		 */
		print : function()
		{
			var level = this.logLevel;
			var content = null;
			if(arguments.length==2)
			{
				level = arguments[0];
				content = arguments[1];
			}
			else content = arguments[0];
			if(level<=this.logLevel) return this.write(level, content);
			return 0;
		},
		
		printInfo : function(content)
		{
			return this.print(this.self(arguments).LOG_LEVEL_INFO, content);
		},
		
		printWarn : function(content)
		{
			return this.print(this.self(arguments).LOG_LEVEL_WARN, content);
		},
		
		printError : function(content)
		{
			return this.print(this.self(arguments).LOG_LEVEL_ERROR, content);
		},
		
		printFatal : function(content)
		{
			return this.print(this.self(arguments).LOG_LEVEL_FATAL, content);
		},
		
		write : function(level, content) {},
		
		/**
		 * 需要重载以下方法:
		 * println(content);
		 * println(level, content)
		 */
		println : function()
		{
			var level = this.logLevel;
			var content = null;
			if(arguments.length==2)
			{
				level = arguments[0];
				content = arguments[1];
			}
			else content = arguments[0];
			if(level<=this.logLevel) return this.writeln(level, content);
			return 0;
		},
		
		printlnInfo : function(content)
		{
			return this.println(this.self(arguments).LOG_LEVEL_INFO, content);
		},
		
		printlnWarn : function(content)
		{
			return this.println(this.self(arguments).LOG_LEVEL_WARN, content);
		},
		
		printlnError : function(content)
		{
			return this.println(this.self(arguments).LOG_LEVEL_ERROR, content);
		},
		
		printlnFatal : function(content)
		{
			return this.println(this.self(arguments).LOG_LEVEL_FATAL, content);
		},
		
		writeln : function(level, content)
		{
			var ret = this.write(level, content);
			if(ret>=0)
			{
				ret += this.write(level, "\n");
			}
			return ret;
		},

		setLogLevel : function(logLevel)
		{
			this.logLevel = logLevel;
		},

		getLogLevel : function()
		{
			return this.logLevel;
		},
		
		close : function() {}
	}
});