/**
 * 会话类
 */
qx.Class.define("xlsys.base.session.Session",
{
	extend : xlsys.java.io.Serializable,
	
	statics : 
	{
		TYPE_WHOLE : 0,
		TYPE_INCREMENTAL : 1
	},

	construct: function(sessionId)
	{
		this.sessionId = sessionId;
		this.sessionType = this.self(arguments).TYPE_WHOLE;
		this.sessionEnv = new xlsys.java.util.HashMap();
		this.tempEnv = new xlsys.java.util.HashMap();
		this.changedKeys = new xlsys.java.util.HashSet();
	},
	
	members :
	{
		sessionId : null,
		sessionType : null,
		sessionEnv : null,
		tempEnv : null,
		changedKeys : null,
		
		getSerializableMembers : function()
		{
			var arr = new Array();
			arr.push("sessionId");
			arr.push("sessionType");
			arr.push("sessionEnv");
			return arr;
		},
		
		/**
		 * 设置会话ID
		 * @param sessionId {String}
		 */
		setSessionId : function(sessionId)
		{
			this.sessionId = sessionId;
		},

		/**
		 * 获取会话ID
		 * @return {String}
		 */
		getSessionId : function()
		{
			return this.sessionId;
		},

		/**
		 * 获取会话类型
		 * @return {Number}
		 */
		getSessionType : function()
		{
			return this.sessionType;
		},

		/**
		 * 设置会话类型
		 * @param sessionType {Number}
		 */
		setSessionType : function(sessionType)
		{
			this.sessionType = sessionType;
		},
		
		/**
		 * 获取会话环境参数中的所有key
		 * @return {xlsys.java.util.Set}
		 */
		getEnvKeySet : function()
		{
			return this.sessionEnv.keySet();
		},
		
		/**
		 * 获取会话临时环境参数map
		 * @return {xlsys.java.util.Map}
		 */
		getTempEnv : function()
		{
			return this.tempEnv;
		},
		
		/**
		 * 获取所有发生过改变的环境参数
		 * @return {xlsys.java.util.Set}
		 */
		getChangedEnvKeys : function()
		{
			return this.changedKeys;
		},
		
		/**
		 * 设置环境变量
		 * @param key {String} 环境变量名
		 * @param value {xlsys.java.io.Serializable} 环境变量值
		 */
		setAttribute : function(key, value)
		{
			// 记录session数据的修改
			var oldValue = this.sessionEnv.get(key);
			if(oldValue==value) return;
			if((oldValue!=null&&value==null)||(value!=null&&oldValue==null)) this.changedKeys.add(key);
			else if(xlsys.base.util.ObjectUtil.hashcode(oldValue)!=xlsys.base.util.ObjectUtil.hashcode(value)) this.changedKeys.add(key);
			this.sessionEnv.put(key, value);
		},
		
		/**
		 * 获取环境变量
		 * @param key {String} 环境变量名
		 * @return {xlsys.java.io.Serializable} 环境变量值
		 */
		getAttribute : function(key)
		{
			return this.sessionEnv.get(key);
		},
		
		/**
		 * 设置临时环境变量
		 * @param key {String} 环境变量名
		 * @param value {Object} 环境变量值
		 */
		setTempAttribute : function(key, value)
		{
			this.tempEnv.put(key, value);
		},
		
		/**
		 * 获取临时环境变量
		 * @param key {String} 环境变量名
		 * @return {Object} 环境变量值
		 */
		getTempAttribute : function(key)
		{
			return this.tempEnv.get(key);
		},

		clone : function()
		{
			var cloneSession = new xlsys.base.session.Session(this.sessionId);
			cloneSession.sessionType = this.sessionType;
			cloneSession.sessionEnv.putAll(this.sessionEnv);
			cloneSession.tempEnv.putAll(this.tempEnv);
			cloneSession.changedKeys.addAll(this.changedKeys);
			return cloneSession;
		}
	}
});