/**
 * Object工具类
 */
qx.Class.define("xlsys.base.util.ObjectUtil",
{
	type : "static",
	
	statics :
	{
		__obj_id : 0,
		
		_getHashKey : function(obj)
		{
			var hashKey = null;
			if(obj==undefined||obj==null) hashKey = "_undefined_";
			else if(qx.lang.Type.isBoolean(obj)) hashKey = "_boolean_"+obj.toString();
			else if(qx.lang.Type.isNumber(obj)) hashKey = "_number_"+obj.toString();
			else if(qx.lang.Type.isString(obj)) hashKey = "_string_"+obj;
			else hashKey = ""+qx.core.ObjectRegistry.toHashCode(obj);
			return hashKey;
		},
		
		/**
		 * 将整形限制在32位整形类型内
		 * @param longValue {Number} 64位的整形
		 * @return {Number} 32位的整形
		 */
		__longToInt : function(longValue)
		{
			if(longValue>0x7fffffff||longValue<-0x80000000) longValue &= 0xFFFFFFFF;
			return longValue;
		},

		/**
		 * 返回字符串的hashcode
		 * @param str {String} 要计算hashcode的字符串
		 * @return {Number} 32位的hashcode
		 */
		__strHashcode : function(str)
		{
			var hash = 0;
			for(var i=0;i<str.length;++i)  
			{
				hash = hash * 31 + str.charCodeAt(i);
				hash = this.__longToInt(hash);  
			}
			return hash;
		},

		/**
		 * 合并hashcode
		 * @param seed {Number} 源hash值
		 * @param obj {Object} 需要合并的对象
		 * @return {Number} 32位的hashcode
		 */
		_hash_combine : function(seed, obj)
		{
			seed = seed * 31 + this.hashcode(obj);
			seed = this.__longToInt(seed);
			return seed;
		},
		
		/**
		 * 返回任意对象的hashcode
		 * @param obj {Object} 需要计算hashcode的对象
		 * @return {Number} 32位的hashcode
		 */
		hashcode : function(obj)
		{
			var hash = 0;
			if(obj==undefined||obj==null)
			{
				hash = this.__strHashcode("_undefined_");
			}
			else if(qx.lang.Type.isArray(obj))
			{
				// 是一个数组
				hash = this._hash_combine(hash, "_array_");
				for(var i=0;i<obj.length;++i)
				{
					hash = this._hash_combine(hash, obj[i]);
				}
			}
			else if(qx.lang.Type.isBoolean(obj))
			{
				hash = this.__strHashcode("_boolean_"+obj.toString());
			}
			else if(qx.lang.Type.isDate(obj))
			{
				hash = this.__strHashcode("_date_"+obj.toString());
			}
			else if(qx.lang.Type.isNumber(obj))
			{
				hash = this.__strHashcode("_number_"+obj.toString());
			}
			else if(qx.lang.Type.isString(obj))
			{
				hash = this.__strHashcode("_string_"+obj);
			}
			else if(qx.lang.Type.isRegExp(obj))
			{
				hash = this.__strHashcode("_regexp_"+obj.source);
			}
			else if(qx.lang.Type.isFunction(obj))
			{
				hash = this.__strHashcode("_function_"+obj.toString());
			}
			else if(qx.lang.Type.isError(obj))
			{
				hash = this.__strHashcode("_error_"+obj.toString());
			}
			else if(qx.lang.Type.isObject(obj))
			{
				if(obj instanceof qx.core.Object)
				{
					hash = obj.toHashCode();
				}
				else
				{
					// object 直接使用当前path+自增变量的形式给予唯一性字符串
					if(!obj.__hashcode)
					{
						++this.__obj_id;
						var uniqueObjStr = "_object_"+ location.href + this.__obj_id;
						hash = this.__strHashcode(uniqueObjStr);
						obj.__hashcode = hash;
					}
					else hash = obj.__hashcode;
				}
			}
			return hash;
		},
		
		/**
		 * 返回任意对象的className
		 * @param obj {Object} 要获取className的对象
		 * @return {String} 对象的className
		 */
		className : function(obj)
		{
			var name = null;
			if(obj==undefined||obj==null)
			{
				name = "undefined";
			}
			else if(qx.lang.Type.isArray(obj))
			{
				name = "[B";
			}
			else if(qx.lang.Type.isBoolean(obj))
			{
				name = "java.lang.Boolean";
			}
			else if(qx.lang.Type.isDate(obj))
			{
				name = "java.util.Date";
			}
			else if(qx.lang.Type.isNumber(obj))
			{
				name = "java.math.BigDecimal";
			}
			else if(qx.lang.Type.isString(obj))
			{
				name = "java.lang.String";
			}
			else if(qx.lang.Type.isRegExp(obj))
			{
				throw new Error("Unsupport!");
			}
			else if(qx.lang.Type.isFunction(obj))
			{
				throw new Error("Unsupport!");
			}
			else if(qx.lang.Type.isObject(obj))
			{
				if(obj instanceof xlsys.java.io.Serializable)
				{
					name = obj.getClassName();
				}
				else if(obj instanceof qx.core.Object)
				{
					name = obj.classname;
				}
				else
				{
					name = "java.lang.Object";
				}
			}
			return name;
		},
		
		getClass : function(obj)
		{
			if(obj==null||obj.classname==null) return null;
			return this.getClassByName(obj.classname);
		},
		
		getClassByName : function(className)
		{
			var objClass = qx.Bootstrap.getByName(className);
			if(objClass==null) objClass = qx.Interface.getByName(className);
			return objClass;
		},
		
		hasMixin : function(obj, mixin)
		{
			var has = false;
			var objClass = this.getClass(obj);
			if(objClass!=null) has = qx.Class.hasMixin(objClass, mixin);
			return has;
		},
		
		hasBitProp : function(prop, bitProp)
		{
			return (prop&bitProp)==bitProp;
		},
		
		objectToString : function(obj)
		{
			var str = null;
			if(obj==undefined||obj==null)
			{
				str = null;
			}
			else if(qx.lang.Type.isArray(obj))
			{
				try
				{
					str = xlsys.base.util.StringUtil.bytesToString(obj);
				}
				catch(err)
				{
					var sb = new xlsys.java.lang.StringBuilder();
					sb.append("[");
					for(var i=0;i<obj.length;++i)
					{
						sb.append(this.objectToString(obj[i]));
						if(i!=obj.length-1) sb.append(",");
					}
					sb.append("]");
					str = sb.toString();
				}
			}
			else if(qx.lang.Type.isBoolean(obj))
			{
				str = ""+obj;
			}
			else if(qx.lang.Type.isDate(obj))
			{
				var year = obj.getFullYear()+"";
				var month = obj.getMonth()+1+"";
				if(month.length<2) month = "0" + month;
				var day = obj.getDate()+"";
				if(day.length<2) day = "0" + day;
				var hour = obj.getHours()+"";
				if(hour.length<2) hour = "0" + hour;
				var minute = obj.getMinutes()+"";
				if(minute.length<2) minute = "0" + minute;
				var second = obj.getSeconds()+"";
				if(second.length<2) second = "0" + second;
				str = month+"/"+day+"/"+year+" "+hour+":"+minute+":"+second;
			}
			else if(qx.lang.Type.isNumber(obj))
			{
				str = ""+obj;
			}
			else if(qx.lang.Type.isString(obj))
			{
				str = obj;
			}
			else if(qx.lang.Type.isRegExp(obj))
			{
				str = obj.source;
			}
			else if(qx.lang.Type.isFunction(obj))
			{
				str = obj.toString();
			}
			else if(qx.lang.Type.isError(obj))
			{
				var sb = new xlsys.java.lang.StringBuilder();
				sb.append(obj.name+" : "+obj.message).append("\n");
				var stackTrace = qx.dev.StackTrace.getStackTraceFromError(obj);
				for(var i=0;i<stackTrace.length;++i)
				{
					sb.append(stackTrace[i]);
					if(i!=stackTrace.length-1) sb.append("\n");
				}
				str = sb.toString();
			}
			else if(obj instanceof xlsys.java.lang.Exception)
			{
				var stackTrace = obj.getStackTrace();
				var sb = new xlsys.java.lang.StringBuilder();
				sb.append(obj.getMessage()).append("\n");
				for(var i=0;i<stackTrace.length;++i)
				{
					sb.append(stackTrace[i]);
					if(i!=stackTrace.length-1) sb.append("\n");
				}
				str = sb.toString();
			}
			else if(qx.lang.Type.isObject(obj))
			{
				str = obj.toString();
			}
			return str;
		},
		
		objectToNumber : function(obj)
		{
			if(obj==null) return null;
			return Number(obj);
		},
		
		objectToInt : function(obj)
		{
			if(obj==null) return null;
			return parseInt(this.objectToString(obj));
		},
		
		objectToBoolean : function(obj)
		{
			if(obj==null) return false;
			else if(qx.lang.Type.isBoolean(obj)) return obj;
			else if(qx.lang.Type.isNumber(obj))
			{
				if(this.objectToInt(obj)==0) return false;
				else return true;
			}
			else
			{
				var objStr = this.objectToString(obj);
				if(objStr.toLowerCase()=="true") return true;
				else return false;
			}
		},
		
		objectToDate : function(obj)
		{
			if(obj==null) return null;
			else if(qx.lang.Type.isDate(obj)) return obj;
			else
			{
				// 为保持兼容性, 只支持 "月/日/年 小时:分钟:秒" 的格式
				var objStr = this.objectToString(obj);
				var time = Date.parse(objStr);
				var date = new Date();
				date.setTime(time);
				return date;
			}
		},
		
		objectToBlob : function(obj)
		{
			if(qx.lang.Type.isString(obj))
			{
				return xlsys.base.util.StringUtil.stringToBytes(obj);
			}
			else return obj;
		},
		
		/**
		 * 该文件内的方法用来进行动态创建对象
		 * @param className {String}
		 * @return {Object}
		 * @lint ignoreDeprecated(eval) 
		 */
		tryInitObj : function(className)
		{
			var obj = null;
			var invokeSb = new xlsys.java.lang.StringBuilder();
			invokeSb.append("new ");
			if(qx.lang.String.startsWith(className, "[")) invokeSb.append("Array");
			else if(qx.lang.String.endsWith(className, "Exception")) invokeSb.append("xlsys.java.lang.Exception");
			else if(qx.lang.String.startsWith(className, "java.")) invokeSb.append("xlsys.").append(className);
			else invokeSb.append(className);
			invokeSb.append("(");
			if(arguments.length>1)
			{
				for(var i=1;i<arguments.length;++i)
				{
					invokeSb.append(arguments[i]);
					if(i!=arguments.length-1) invokeSb.append(",");
				}
			}
			invokeSb.append(");");
			obj = eval(invokeSb.toString());
			return obj;
		}
	}
});