/**
 * 反射调用工具类
 */
qx.Class.define("xlsys.base.util.ReflectUtil",
{
	type : "static",
	
	statics :
	{
		/** 
		 * 获取要运行的指定类的实例.
		 * @param className {String} 类名
		 * @param params {Array|null} 要传入构造方法的参数
		 * @return {Object} 实例对象
		 * @lint ignoreDeprecated(eval)
		 */
		newInstance : function(className, params)
		{
			var targetClass = xlsys.base.util.ObjectUtil.getClassByName(className);
			var ret = null;
			var error = null;
			// 先尝试getInstance方法
			var sb = new xlsys.java.lang.StringBuilder();
			sb.append(className).append(".getInstance");
			if(eval(sb.toString())!=null)
			{
				sb.append("(");
				this.__addInstanceParams(sb, params);
				sb.append(");");
				try
				{
					ret = eval(sb.toString());
				}
				catch(err){error = err;}
			}
			if(ret==null)
			{
				// 再尝试使用new关键字
				sb = new xlsys.java.lang.StringBuilder();
				sb.append("new ").append(className).append("(");
				this.__addInstanceParams(sb, params);
				sb.append(");");
				try
				{
					ret = eval(sb.toString());
				}
				catch(err) {error = err;}
			}
			if(ret==null&&error!=null) throw error;
			return ret;
		},
		
		__addInstanceParams : function(sb, params)
		{
			if(params==null) return;
			if(qx.lang.Type.isArray(params))
			{
				for(var i=0;i<params.length;++i)
				{
					sb.append("params[").append(i).append("]");
					if(i!=params.length-1) sb.append(",");
				}
			}
			else
			{
				for(var i=1;i<arguments.length;++i)
				{
					sb.append("arguments[").append(i).append("]");
					if(i!=arguments.length-1) sb.append(",");
				}
			}
		},
		
		/**
		 * 执行形如以下文本的静态方法, 其中ccc和ddd将被放入map, 并将该map作为该方法的第一个参数传入
		 * xlsys.test.JavaInvokeTest.getTestOs?ccc=3&ddd=4
		 * @lint ignoreDeprecated(eval) 
		 */
		invokeStaticMethod : function(invokeStr)
		{
			var arr = invokeStr.split(xlsys.base.XLSYS.PARAM_QUESTION);
			var map = null;
			if(arr.length>1)
			{
				map = new xlsys.java.util.HashMap();
				var params = arr[1].split(xlsys.base.XLSYS.PARAM_AND);
				for(var i=0;i<params.length;++i)
				{
					var varParam = params[i].split(xlsys.base.XLSYS.PARAM_RELATION);
					map.put(varParam[0], varParam[1]);
				}
			}
			var sb = new xlsys.java.lang.StringBuilder();
			sb.append(arr[0]).append("(");
			if(map!=null) sb.append("map");
			sb.append(");");
			return eval(sb.toString());
		},
		
		/**
		 * 获取对象所有的静态属性(包含父类)
		 */
		getAllStaticFieldMap : function(objClass)
		{
			if(objClass==null) return null;
			var staticFieldMap = new xlsys.java.util.LinkedHashMap();
			var field = null;
			// 先获取所有的interface中的静态属性
			var interfaces = qx.Class.getInterfaces(objClass);
			for(var i=0;i<interfaces.length;++i)
			{
				var tempInterface = interfaces[i];
				for(field in tempInterface)
				{
					staticFieldMap.put(field, tempInterface[field]);
				}
			}
			// 获取所有mixin中的静态属性
			var mixins = qx.Class.getMixins(objClass);
			for(var i=0;i<mixins.length;++i)
			{
				var tempMixin = mixins[i];
				for(field in tempMixin)
				{
					staticFieldMap.put(field, tempMixin[field]);
				}
			}
			// 获取本类及其父类中的静态属性
			var tempClass = objClass;
			while(tempClass!=null)
			{
				for(field in tempClass)
				{
					staticFieldMap.put(field, tempClass[field]);
				}
				tempClass = tempClass.superclass;
			}
			return staticFieldMap;
		},
		
		/**
		 * 获取本对象所有的静态属性(不包含父类)
		 */
		getStaticFieldMap : function(objClass)
		{
			if(objClass==null) return null;
			var staticFieldMap = new xlsys.java.util.LinkedHashMap();
			var field = null;
			for(field in objClass)
			{
				staticFieldMap.put(field, objClass[field]);
			}
			return staticFieldMap;
		},
		
		getStaticFieldMapWithPrefix : function(objClass, fieldPrefix, containsParent)
		{
			var targetMap = new xlsys.java.util.LinkedHashMap();
			var staticFieldMap = null;
			if(containsParent) staticFieldMap = this.getAllStaticFieldMap(objClass);
			else staticFieldMap = this.getStaticFieldMap(objClass);
			var keys = staticFieldMap.keys();
			for(var i=0;i<keys.length;++i)
			{
				var fieldName = keys[i];
				if(!qx.lang.String.startsWith(fieldName, fieldPrefix)) continue;
				targetMap.put(fieldName, staticFieldMap.get(fieldName));
			}
			return targetMap;
		},
		
		/**
		 * 解析形如下行参数, 将其转换成Map.
		 * <li> aaa(String)=12;34;56&bbb=select * from xxx&ccc(Number)=23&ddd(Number)=1:a;2:b;3:c
		 * <li> 单个参数说明 : '属性名' + (可选)'(单个属性类型,支持String,Number和Boolean)' + 'xlsys.base.XLSYS.PARAM_RELATION' + '属性值(如果是数组,使用xlsys.base.XLSYS.KEY_CODE_SEPARATOR分割,如果是码名映射的Map,码名之间使用xlsys.base.XLSYS.CODE_NAME_RELATION分割)'
		 */
		getFieldParamMap : function(paramStr)
		{
			var paramMap = new xlsys.java.util.HashMap();
			if(paramStr!=null)
			{
				var paramArr = paramStr.split(xlsys.base.XLSYS.PARAM_AND);
				for(var i=0;i<paramArr.length;++i)
				{
					var pair = paramArr[i].split(xlsys.base.XLSYS.PARAM_RELATION);
					var beginIdx = pair[0].indexOf("(");
					var endIdx = pair[0].indexOf(")");
					var fieldName = pair[0];
					var fieldClass = "String";
					if(beginIdx!=-1)
					{
						fieldName = pair[0].substring(0, beginIdx);
						fieldClass = pair[0].substring(beginIdx+1, endIdx);
					}
					var containsKeyCodeSeparator = pair[1].indexOf(xlsys.base.XLSYS.KEY_CODE_SEPARATOR)!=-1;
					var containsCodeNameRelation = pair[1].indexOf(xlsys.base.XLSYS.CODE_NAME_RELATION)!=-1;
					var fieldValue = null;
					if(containsCodeNameRelation)
					{
						// 属性值为码名映射Map
						fieldValue = new xlsys.java.util.LinkedHashMap();
						var codeNameArr = pair[1].split(xlsys.base.XLSYS.KEY_CODE_SEPARATOR);
						for(var j=0;j<codeNameArr.length;++j)
						{
							var codeNamePair = codeNameArr[j].split(xlsys.base.XLSYS.CODE_NAME_RELATION);
							var code = this._getCodeDataWithFieldClass(fieldClass, codeNamePair[0]);
							fieldValue.put(code, codeNamePair[1]);
						}
					}
					else if(containsKeyCodeSeparator)
					{
						// 属性值为数组
						fieldValue = new Array();
						var codeArr = pair[1].split(xlsys.base.XLSYS.KEY_CODE_SEPARATOR);
						for(var j=0;j<codeArr.length;++j)
						{
							var code = this._getCodeDataWithFieldClass(fieldClass, codeArr[j]);
							fieldValue.push(code);
						}
					}
					else
					{
						// 单个属性值
						var code = this._getCodeDataWithFieldClass(fieldClass, pair[1]);
						fieldValue = code;
					}
					paramMap.put(fieldName, fieldValue);
				}
			}
			return paramMap;
		},
		
		_getCodeDataWithFieldClass : function(fieldClass, codeStr)
		{
			var code = null;
			if(fieldClass=="String") code = codeStr;
			else if(fieldClass=="Number") code = Number(codeStr);
			else if(fieldClass=="Boolean") code = xlsys.base.util.ObjectUtil.objectToBoolean(codeStr);
			return code;
		},
		
		/**
		 * 支持以下方法:
		 * getStaticFieldCodeAndNameMap(paramMap); 其中,paramMap中需要包含以下Key值className, fieldPrefix, containsParent
		 * getStaticFieldCodeAndNameMap(objClass, fieldPrefix, containsParent);
		 */
		getStaticFieldCodeAndNameMap : function()
		{
			var objClass = null;
			var fieldPrefix = null;
			var containsParent = null;
			if(arguments.length==1)
			{
				var paramMap = arguments[0];
				var className = paramMap.get("className");
				objClass = xlsys.base.util.ObjectUtil.getClassByName(className);
				fieldPrefix = paramMap.get("fieldPrefix");
				containsParent = xlsys.base.util.ObjectUtil.objectToBoolean(paramMap.get("containsParent"));
			}
			else if(arguments.length==3)
			{
				objClass = arguments[0];
				fieldPrefix = arguments[1];
				containsParent = arguments[2];
			}
			var codeNameMap = new xlsys.java.util.LinkedHashMap();
			var staticFieldMap = null;
			if(containsParent) staticFieldMap = this.getAllStaticFieldMap(objClass);
			else staticFieldMap = this.getStaticFieldMap(objClass);
			var keys = staticFieldMap.keys();
			for(var i=0;i<keys.length;++i)
			{
				var fieldName = keys[i];
				if(!qx.lang.String.startsWith(fieldName, fieldPrefix)) continue;
				var code = staticFieldMap.get(fieldName);
				var tempNameArr = fieldName.substring(fieldPrefix.length).toLowerCase().split("_");
				var name = tempNameArr[0];
				for(var j=1;j<tempNameArr.length;++j)
				{
					name += qx.lang.String.firstUp(tempNameArr[j]);
				}
				codeNameMap.put(code, name);
			}
			return codeNameMap;
		}
	}
});