/**
 * 翻译工具类
 */
qx.Class.define("xlsys.base.util.TranslateUtil",
{
	extend : xlsys.base.buffer.AbstractBuffer,
	
	statics :
	{
		DEFAULT_TABLENAME : "_default_table",
		
		tuMap : new xlsys.java.util.HashMap(),
		
		getInstance : function(language)
		{
			if(language==null) language = xlsys.base.util.SystemUtil.getUserLanguage();
			var tu = this.tuMap.get(language);
			if(tu==null)
			{
				tu = new xlsys.base.util.TranslateUtil(language);
				this.tuMap.put(language, tu);
			}
			return tu;
		}
	},
	
	construct : function(language)
	{
		this.base(arguments);
		if(language==null) language = xlsys.base.util.SystemUtil.getUserLanguage();
		this._language = language;
		xlsys.base.buffer.BufferManager.getInstance().registerBuffer(xlsys.base.XLSYS.BUFFER_TRANSLATION_PREFIX+language, this);
	},
	
	members : 
	{
		_language : null,
		_tableCount : null,
		
		/**
		 * 支持以下方法:
		 * getTranslateStr(dataBase, srcStr);
		 * getTranslateStr(dataBase, srcStr, callback);
		 * getTranslateStr(envId, dataBase, srcStr);
		 * getTranslateStr(envId, dataBase, srcStr, callback);
		 * getTranslateStr(dataBase, srcStr, tableNames);
		 * getTranslateStr(dataBase, srcStr, tableNames, callback);
		 * getTranslateStr(envId, dataBase, srcStr, tableNames);
		 * getTranslateStr(envId, dataBase, srcStr, tableNames, callback);
		 */
		getTranslateStr : function()
		{
			var envId = null;
			var dataBase = null;
			var srcStr = null;
			var tableNames = null;
			var callback = null;
			if(arguments.length==2)
			{
				// getTranslateStr(dataBase, srcStr);
				dataBase = arguments[0];
				srcStr = arguments[1];
				envId = dataBase.getEnvId();
				tableNames = this.self(arguments).DEFAULT_TABLENAME;
			}
			else if(arguments.length==3)
			{
				if(qx.lang.Type.isNumber(arguments[0]))
				{
					// getTranslateStr(envId, dataBase, srcStr);
					envId = arguments[0];
					dataBase = arguments[1];
					srcStr = arguments[2];
					tableNames = this.self(arguments).DEFAULT_TABLENAME;
				}
				else if(qx.lang.Type.isString(arguments[2]))
				{
					// getTranslateStr(dataBase, srcStr, tableName);
					dataBase = arguments[0];
					srcStr = arguments[1];
					tableNames = arguments[2];
					envId = dataBase.getEnvId();
				}
				else
				{
					// getTranslateStr(dataBase, srcStr, callback);
					dataBase = arguments[0];
					srcStr = arguments[1];
					envId = dataBase.getEnvId();
					tableNames = this.self(arguments).DEFAULT_TABLENAME;
					callback = arguments[2];
				}
			}
			else if(arguments.length==4)
			{
				if(qx.lang.Type.isNumber(arguments[0]))
				{
					if(qx.lang.Type.isString(arguments[3]))
					{
						// getTranslateStr(envId, dataBase, srcStr, tableName);
						envId = arguments[0];
						dataBase = arguments[1];
						srcStr = arguments[2];
						tableNames = arguments[3];
					}
					else
					{
						// getTranslateStr(envId, dataBase, srcStr, callback);
						envId = arguments[0];
						dataBase = arguments[1];
						srcStr = arguments[2];
						tableNames = this.self(arguments).DEFAULT_TABLENAME;
						callback = arguments[3];
					}
				}
				else
				{
					// getTranslateStr(dataBase, srcStr, tableNames, callback);
					dataBase = arguments[0];
					srcStr = arguments[1];
					tableNames = arguments[2];
					envId = dataBase.getEnvId();
					callback = arguments[3];
				}
			}
			else
			{
				// getTranslateStr(envId, dataBase, srcStr, tableNames, callback);
				envId = arguments[0];
				dataBase = arguments[1];
				srcStr = arguments[2];
				tableNames = arguments[3];
				callback = arguments[4];
			}
			if(tableNames==null) tableNames = this.self(arguments).DEFAULT_TABLENAME;
			else if(this.self(arguments).DEFAULT_TABLENAME!=tableNames&&tableNames.indexOf(this.self(arguments).DEFAULT_TABLENAME)==-1) tableNames += xlsys.base.XLSYS.KEY_CODE_SEPARATOR + this.self(arguments).DEFAULT_TABLENAME;
			var retStr = null;
			var paramMap = new xlsys.java.util.HashMap();
			paramMap.put(xlsys.base.database.IDataBase.BUFFER_KEY_TABLE_NAME, tableNames);
			var bufferName = xlsys.base.XLSYS.BUFFER_TRANSLATION_PREFIX+this._language;
			if(callback) this.getBufferData(bufferName, paramMap, qx.Bootstrap.bind(this.__getTranslateStr1, this, srcStr, callback));
			else
			{
				var tableMap = this.getBufferData(bufferName, paramMap);
				retStr = tableMap.get(srcStr);
				if(retStr==null) retStr = srcStr;
				return retStr;
			}
		},
		
		__getTranslateStr1 : function(srcStr, callback, tableMap)
		{
			var retStr = tableMap.get(srcStr);
			if(retStr==null) retStr = srcStr;
			callback(retStr);
		},
		
		/**
		 * 支持以下方法:
		 * getTranslateMap(dataBase, srcMap);
		 * getTranslateMap(dataBase, srcMap, callback);
		 * getTranslateMap(envId, dataBase, srcMap);
		 * getTranslateMap(envId, dataBase, srcMap, callback);
		 * getTranslateMap(dataBase, srcMap, tableNames);
		 * getTranslateMap(dataBase, srcMap, tableNames, callback);
		 * getTranslateMap(envId, dataBase, srcMap, tableNames);
		 * getTranslateMap(envId, dataBase, srcMap, tableNames, callback);
		 */
		getTranslateMap : function()
		{
			var envId = null;
			var dataBase = null;
			var srcMap = null;
			var tableNames = null;
			var callback = null;
			if(arguments.length==2)
			{
				// getTranslateMap(dataBase, srcMap);
				dataBase = arguments[0];
				srcMap = arguments[1];
				envId = dataBase.getEnvId();
				tableNames = this.self(arguments).DEFAULT_TABLENAME;
			}
			else if(arguments.length==3)
			{
				if(qx.lang.Type.isNumber(arguments[0]))
				{
					// getTranslateMap(envId, dataBase, srcMap);
					envId = arguments[0];
					dataBase = arguments[1];
					srcMap = arguments[2];
					tableNames = this.self(arguments).DEFAULT_TABLENAME;
				}
				else if(qx.lang.Type.isString(arguments[2]))
				{
					// getTranslateMap(dataBase, srcMap, tableNames);
					dataBase = arguments[0];
					srcMap = arguments[1];
					tableNames = arguments[2];
					envId = dataBase.getEnvId();
				}
				else
				{
					// getTranslateMap(dataBase, srcMap, callback);
					dataBase = arguments[0];
					srcMap = arguments[1];
					envId = dataBase.getEnvId();
					tableNames = this.self(arguments).DEFAULT_TABLENAME;
					callback = arguments[2];
				}
			}
			else if(arguments.length==4)
			{
				if(qx.lang.Type.isNumber(arguments[0]))
				{
					if(qx.lang.Type.isString(arguments[3]))
					{
						// getTranslateMap(envId, dataBase, srcMap, tableNames);
						envId = arguments[0];
						dataBase = arguments[1];
						srcMap = arguments[2];
						tableNames = arguments[3];
					}
					else
					{
						// getTranslateMap(envId, dataBase, srcMap, callback);
						envId = arguments[0];
						dataBase = arguments[1];
						srcMap = arguments[2];
						tableNames = this.self(arguments).DEFAULT_TABLENAME;
						callback = arguments[3];
					}
				}
				else
				{
					// getTranslateMap(dataBase, srcMap, tableNames, callback);
					dataBase = arguments[0];
					srcMap = arguments[1];
					tableNames = arguments[2];
					envId = dataBase.getEnvId();
					callback = arguments[3];
				}
			}
			else
			{
				// getTranslateMap(envId, dataBase, srcMap, tableNames, callback);
				envId = arguments[0];
				dataBase = arguments[1];
				srcMap = arguments[2];
				tableNames = arguments[3];
				callback = arguments[4];
			}
			if(tableNames==null) tableNames = this.self(arguments).DEFAULT_TABLENAME;
			else if(this.self(arguments).DEFAULT_TABLENAME!=tableNames&&tableNames.indexOf(this.self(arguments).DEFAULT_TABLENAME)==-1) tableNames += xlsys.base.XLSYS.KEY_CODE_SEPARATOR + this.self(arguments).DEFAULT_TABLENAME;
			var paramMap = new xlsys.java.util.HashMap();
			paramMap.put(xlsys.base.database.IDataBase.BUFFER_KEY_TABLE_NAME, tableNames);
			var bufferName = xlsys.base.XLSYS.BUFFER_TRANSLATION_PREFIX+this._language;
			if(callback) this.getBufferData(bufferName, paramMap, qx.Bootstrap.bind(this.__getTranslateMap1, this, srcMap, callback));
			else
			{
				var tableMap = this.getBufferData(bufferName, paramMap);
				return this.__getTranslateMap1(srcMap, null, tableMap);
			}
		},
		
		__getTranslateMap1 : function(srcMap, callback, tableMap)
		{
			var mapClass = srcMap.classname;
			var retMap = xlsys.base.util.ReflectUtil.newInstance(mapClass);
			if(!tableMap.isEmpty())
			{
				var keys = srcMap.keys();
				for(var i=0;i<keys.length;++i)
				{
					var transValue = tableMap.get(srcMap.get(keys[i]));
					if(transValue==null) retMap.put(keys[i], srcMap.get(keys[i]));
					else retMap.put(keys[i], transValue);
				}
			}
			else retMap.putAll(srcMap);
			if(callback) callback(retMap);
			else return retMap;
		},
		
		/**
		 * 支持以下方法:
		 * translateDataSet(dataBase, dataSet);
		 * translateDataSet(dataBase, dataSet, callback);
		 * translateDataSet(envId, dataBase, dataSet);
		 * translateDataSet(envId, dataBase, dataSet, callback);
		 * translateDataSet(dataBase, dataSet, tableNames);
		 * translateDataSet(dataBase, dataSet, tableNames, callback);
		 * translateDataSet(envId, dataBase, dataSet, tableNames);
		 * translateDataSet(envId, dataBase, dataSet, tableNames, callback);
		 * translateDataSet(dataBase, dataSet, tableNames, transCols);
		 * translateDataSet(dataBase, dataSet, tableNames, transCols, callback);
		 * translateDataSet(envId, dataBase, dataSet, tableNames, transCols);
		 * translateDataSet(envId, dataBase, dataSet, tableNames, transCols, callback);
		 * translateDataSet(envId, dataBase, dataSet, tableNames, transCols, colSpliter);
		 * translateDataSet(envId, dataBase, dataSet, tableNames, transCols, colSpliter, callback);
		 */
		translateDataSet : function()
		{
			var envId = null;
			var dataBase = null;
			var dataSet = null;
			var tableNames = null;
			var transCols = null;
			var colSpliter = null;
			var callback = null;
			if(arguments.length==2)
			{
				// translateDataSet(dataBase, dataSet);
				dataBase = arguments[0];
				dataSet = arguments[1];
			}
			else if(arguments.length==3)
			{
				if(qx.lang.Type.isNumber(arguments[0]))
				{
					// translateDataSet(envId, dataBase, dataSet);
					envId = arguments[0];
					dataBase = arguments[1];
					dataSet = arguments[2];
				}
				else if(qx.lang.Type.isString(arguments[2]))
				{
					// translateDataSet(dataBase, dataSet, callback);
					dataBase = arguments[0];
					dataSet = arguments[1];
					callback = arguments[2];
				}
				else
				{
					// translateDataSet(dataBase, dataSet, tableNames);
					dataBase = arguments[0];
					dataSet = arguments[1];
					tableNames = arguments[2];
				}
			}
			else if(arguments.length==4)
			{
				if(qx.lang.Type.isNumber(arguments[0]))
				{
					if(qx.lang.Type.isString(arguments[3]))
					{
						// translateDataSet(envId, dataBase, dataSet, tableNames);
						envId = arguments[0];
						dataBase = arguments[1];
						dataSet = arguments[2];
						tableNames = arguments[3];
					}
					else
					{
						// translateDataSet(envId, dataBase, dataSet, callback);
						envId = arguments[0];
						dataBase = arguments[1];
						dataSet = arguments[2];
						callback = arguments[3];
					}
				}
				else
				{
					if(qx.lang.Type.isString(arguments[3]))
					{
						// translateDataSet(dataBase, dataSet, tableNames, transCols);
						dataBase = arguments[0];
						dataSet = arguments[1];
						tableNames = arguments[2];
						transCols = arguments[3];
					}
					else
					{
						// translateDataSet(dataBase, dataSet, tableNames, callback);
						dataBase = arguments[0];
						dataSet = arguments[1];
						tableNames = arguments[2];
						callback = arguments[3];
					}
				}
			}
			else if(arguments.length==5)
			{
				if(qx.lang.Type.isNumber(arguments[0]))
				{
					if(qx.lang.Type.isString(arguments[4]))
					{
						// translateDataSet(envId, dataBase, dataSet, tableNames, transCols);
						envId = arguments[0];
						dataBase = arguments[1];
						dataSet = arguments[2];
						tableNames = arguments[3];
						transCols = arguments[4];
					}
					else
					{
						// translateDataSet(envId, dataBase, dataSet, tableNames, callback);
						envId = arguments[0];
						dataBase = arguments[1];
						dataSet = arguments[2];
						tableNames = arguments[3];
						callback = arguments[4];
					}
				}
				else
				{
					// translateDataSet(dataBase, dataSet, tableNames, transCols, callback);
					dataBase = arguments[0];
					dataSet = arguments[1];
					tableNames = arguments[2];
					transCols = arguments[3];
					callback = arguments[4];
				}
			}
			else if(arguments.length==6)
			{
				if(qx.lang.Type.isNumber(arguments[0]))
				{
					// translateDataSet(envId, dataBase, dataSet, tableNames, transCols, colSpliter);
					envId = arguments[0];
					dataBase = arguments[1];
					dataSet = arguments[2];
					tableNames = arguments[3];
					transCols = arguments[4];
					colSpliter = arguments[5];
				}
				else
				{
					// translateDataSet(envId, dataBase, dataSet, tableNames, transCols, callback);
					envId = arguments[0];
					dataBase = arguments[1];
					dataSet = arguments[2];
					tableNames = arguments[3];
					transCols = arguments[4];
					callback = arguments[5];
				}
			}
			else
			{
				// translateDataSet(envId, dataBase, dataSet, tableNames, transCols, colSpliter, callback);
				envId = arguments[0];
				dataBase = arguments[1];
				dataSet = arguments[2];
				tableNames = arguments[3];
				transCols = arguments[4];
				colSpliter = arguments[5];
				callback = arguments[6];
			}
			if(dataSet==null||dataSet.getRowCount()==0) return;
			if(envId==null) envId = dataBase.getEnvId();
			if(transCols==null) transCols = "name";
			if(colSpliter==null) colSpliter = xlsys.base.XLSYS.KEY_CODE_SEPARATOR;
			if(tableNames==null) tableNames = this.self(arguments).DEFAULT_TABLENAME;
			else if(this.self(arguments).DEFAULT_TABLENAME!=tableNames&&tableNames.indexOf(this.self(arguments).DEFAULT_TABLENAME)==-1) tableNames += xlsys.base.XLSYS.KEY_CODE_SEPARATOR + this.self(arguments).DEFAULT_TABLENAME;
			if(dataSet.getTableName)
			{
				var tableName = dataSet.getTableName();
				if(tableName!=null&&tableNames.indexOf(tableName)==-1) tableNames = tableName + xlsys.base.XLSYS.KEY_CODE_SEPARATOR + tableNames;
			}
			var paramMap = new xlsys.java.util.HashMap();
			paramMap.put(xlsys.base.database.IDataBase.BUFFER_KEY_TABLE_NAME, tableNames);
			var bufferName = xlsys.base.XLSYS.BUFFER_TRANSLATION_PREFIX+this._language;
			if(callback) this.getBufferData(bufferName, paramMap, qx.Bootstrap.bind(this.__translateDataSet1, this, dataSet, transCols, colSpliter, callback));
			else
			{
				var tableMap = this.getBufferData(bufferName, paramMap);
				this.__translateDataSet1(dataSet, transCols, colSpliter, null, tableMap);
			}
		},
		
		__translateDataSet1 : function(dataSet, transCols, colSpliter, callback, tableMap)
		{
			if(!tableMap.isEmpty())
			{
				var transColArr = transCols.split(colSpliter);
				for(var i=0;i<transColArr.length;++i)
				{
					var colName = transColArr[i];
					var dsc = dataSet.getColumn(colName);
					if(dsc!=null)
					{
						var colIndex = dataSet.getColumnIndex(colName);
						var rowCount = dataSet.getRowCount();
						for(var j=0;j<rowCount;++j)
						{
							var dsCell = dataSet.getRow(j).getCell(colIndex);
							var oldValue = dsCell.getContent();
							var newValue = null;
							if(oldValue!=null)
							{
								newValue = tableMap.get(oldValue);
								if(newValue!=null) dsCell.setContent(newValue);
							}
						}
					}
				}
			}
			if(callback) callback();
		},
		
		/**
		 * 支持以下方法:
		 * translateModel(dataBase, model);
		 * translateModel(dataBase, model, callback);
		 * translateModel(envId, dataBase, model);
		 * translateModel(envId, dataBase, model, callback);
		 * translateModel(dataBase, model, tableNames);
		 * translateModel(dataBase, model, tableNames, callback);
		 * translateModel(envId, dataBase, model, tableNames);
		 * translateModel(envId, dataBase, model, tableNames, callback);
		 * translateModel(dataBase, model, tableNames, transFields);
		 * translateModel(dataBase, model, tableNames, transFields, callback);
		 * translateModel(envId, dataBase, model, tableNames, transFields);
		 * translateModel(envId, dataBase, model, tableNames, transFields, callback);
		 * translateModel(envId, dataBase, model, tableNames, transFields, fieldSpliter);
		 * translateModel(envId, dataBase, model, tableNames, transFields, fieldSpliter, callback);
		 */
		translateModel : function()
		{
			var envId = null;
			var dataBase = null;
			var model = null;
			var tableNames = null;
			var transFields = null;
			var fieldSpliter = null;
			var callback = null;
			if(arguments.length==2)
			{
				// translateModel(dataBase, model);
				dataBase = arguments[0];
				model = arguments[1];
			}
			else if(arguments.length==3)
			{
				if(qx.lang.Type.isNumber(arguments[0]))
				{
					// translateModel(envId, dataBase, model);
					envId = arguments[0];
					dataBase = arguments[1];
					model = arguments[2];
				}
				else if(qx.lang.Type.isString(arguments[2]))
				{
					// translateModel(dataBase, model, tableNames);
					dataBase = arguments[0];
					model = arguments[1];
					tableNames = arguments[2];
				}
				else
				{
					// translateModel(dataBase, model, callback);
					dataBase = arguments[0];
					model = arguments[1];
					callback = arguments[2];
				}
			}
			else if(arguments.length==4)
			{
				if(qx.lang.Type.isNumber(arguments[0]))
				{
					if(qx.lang.Type.isString(arguments[3]))
					{
						// translateModel(envId, dataBase, model, tableNames);
						envId = arguments[0];
						dataBase = arguments[1];
						model = arguments[2];
						tableNames = arguments[3];
					}
					else
					{
						// translateModel(envId, dataBase, model, callback);
						envId = arguments[0];
						dataBase = arguments[1];
						model = arguments[2];
						callback = arguments[3];
					}
				}
				else
				{
					if(qx.lang.Type.isString(arguments[3]))
					{
						// translateModel(dataBase, model, tableNames, transFields);
						dataBase = arguments[0];
						model = arguments[1];
						tableNames = arguments[2];
						transFields = arguments[3];
					}
					else
					{
						// translateModel(dataBase, model, tableNames, callback);
						dataBase = arguments[0];
						model = arguments[1];
						tableNames = arguments[2];
						callback = arguments[3];
					}
				}
			}
			else if(arguments.length==5)
			{
				if(qx.lang.Type.isNumber(arguments[0]))
				{
					if(qx.lang.Type.isString(arguments[4]))
					{
						// translateModel(envId, dataBase, model, tableNames, transFields);
						envId = arguments[0];
						dataBase = arguments[1];
						model = arguments[2];
						tableNames = arguments[3];
						transFields = arguments[4];
					}
					else
					{
						// translateModel(envId, dataBase, model, tableNames, callback);
						envId = arguments[0];
						dataBase = arguments[1];
						model = arguments[2];
						tableNames = arguments[3];
						callback = arguments[4];
					}
				}
				else
				{
					// translateModel(dataBase, model, tableNames, transFields, callback);
					dataBase = arguments[0];
					model = arguments[1];
					tableNames = arguments[2];
					transFields = arguments[3];
					callback = arguments[4];
				}
			}
			else if(arguments.length==6)
			{
				if(qx.lang.Type.isString(arguments[5]))
				{
					// translateModel(envId, dataBase, model, tableNames, transFields, fieldSpliter);
					envId = arguments[0];
					dataBase = arguments[1];
					model = arguments[2];
					tableNames = arguments[3];
					transFields = arguments[4];
					fieldSpliter = arguments[5];
				}
				else
				{
					// translateModel(envId, dataBase, model, tableNames, transFields, callback);
					envId = arguments[0];
					dataBase = arguments[1];
					model = arguments[2];
					tableNames = arguments[3];
					transFields = arguments[4];
					callback = arguments[5];
				}
			}
			else
			{
				// translateModel(envId, dataBase, model, tableNames, transFields, fieldSpliter, callback);
				envId = arguments[0];
				dataBase = arguments[1];
				model = arguments[2];
				tableNames = arguments[3];
				transFields = arguments[4];
				fieldSpliter = arguments[5];
				callback = arguments[6];
			}
			if(envId==null) envId = dataBase.getEnvId();
			if(transFields==null) transFields = "name";
			if(fieldSpliter==null) fieldSpliter = xlsys.base.XLSYS.KEY_CODE_SEPARATOR;
			var srcModelList = new xlsys.java.util.ArrayList();
			srcModelList.add(model);
			if(callback) this.translateModelList(envId, dataBase, srcModelList, tableNames, transFields, fieldSpliter, qx.Bootstrap.bind(this.__translateModel1, this, callback));
			else
			{
				var translatedModelList = this.translateModelList(envId, dataBase, srcModelList, tableNames, transFields, fieldSpliter);
				if(translatedModelList.isEmpty()) return null;
				return translatedModelList.get(0);
			}
		},
		
		__translateModel1 : function(callback, translatedModelList)
		{
			if(translatedModelList.isEmpty()) callback(null);
			else callback(translatedModelList.get(0));
		},
		
		/**
		 * 支持以下方法:
		 * translateModelList(dataBase, modelList);
		 * translateModelList(dataBase, modelList, callback);
		 * translateModelList(envId, dataBase, modelList);
		 * translateModelList(envId, dataBase, modelList, callback);
		 * translateModelList(dataBase, modelList, tableNames);
		 * translateModelList(dataBase, modelList, tableNames, callback);
		 * translateModelList(envId, dataBase, modelList, tableNames);
		 * translateModelList(envId, dataBase, modelList, tableNames, callback);
		 * translateModelList(dataBase, modelList, tableNames, transFields);
		 * translateModelList(dataBase, modelList, tableNames, transFields, callback);
		 * translateModelList(envId, dataBase, modelList, tableNames, transFields);
		 * translateModelList(envId, dataBase, modelList, tableNames, transFields, callback);
		 * translateModelList(envId, dataBase, modelList, tableNames, transFields, fieldSpliter);
		 * translateModelList(envId, dataBase, modelList, tableNames, transFields, fieldSpliter, callback);
		 */
		translateModelList : function()
		{
			var envId = null;
			var dataBase = null;
			var modelList = null;
			var tableNames = null;
			var transFields = null;
			var fieldSpliter = null;
			var callback = null;
			if(arguments.length==2)
			{
				// translateModelList(dataBase, modelList);
				dataBase = arguments[0];
				modelList = arguments[1];
			}
			else if(arguments.length==3)
			{
				if(qx.lang.Type.isNumber(arguments[0]))
				{
					// translateModelList(envId, dataBase, modelList);
					envId = arguments[0];
					dataBase = arguments[1];
					modelList = arguments[2];
				}
				else if(qx.lang.Type.isString(arguments[2]))
				{
					// translateModelList(dataBase, modelList, tableNames);
					dataBase = arguments[0];
					modelList = arguments[1];
					tableNames = arguments[2];
				}
				else
				{
					// translateModelList(dataBase, modelList, callback);
					dataBase = arguments[0];
					modelList = arguments[1];
					callback = arguments[2];
				}
			}
			else if(arguments.length==4)
			{
				if(qx.lang.Type.isNumber(arguments[0]))
				{
					if(qx.lang.Type.isString(arguments[3]))
					{
						// translateModelList(envId, dataBase, modelList, tableNames);
						envId = arguments[0];
						dataBase = arguments[1];
						modelList = arguments[2];
						tableNames = arguments[3];
					}
					else
					{
						// translateModelList(envId, dataBase, modelList, callback);
						envId = arguments[0];
						dataBase = arguments[1];
						modelList = arguments[2];
						callback = arguments[3];
					}
				}
				else
				{
					if(qx.lang.Type.isString(arguments[3]))
					{
						// translateModelList(dataBase, modelList, tableNames, transFields);
						dataBase = arguments[0];
						modelList = arguments[1];
						tableNames = arguments[2];
						transFields = arguments[3];
					}
					else
					{
						// translateModelList(dataBase, modelList, tableNames, callback);
						dataBase = arguments[0];
						modelList = arguments[1];
						tableNames = arguments[2];
						callback = arguments[3];
					}
				}
			}
			else if(arguments.length==5)
			{
				if(qx.lang.Type.isNumber(arguments[0]))
				{
					if(qx.lang.Type.isString(arguments[4]))
					{
						// translateModelList(envId, dataBase, modelList, tableNames, transFields);
						envId = arguments[0];
						dataBase = arguments[1];
						modelList = arguments[2];
						tableNames = arguments[3];
						transFields = arguments[4];
					}
					else
					{
						// translateModelList(envId, dataBase, modelList, tableNames, callback);
						envId = arguments[0];
						dataBase = arguments[1];
						modelList = arguments[2];
						tableNames = arguments[3];
						callback = arguments[4];
					}
				}
				else
				{
					// translateModelList(dataBase, modelList, tableNames, transFields, callback);
					dataBase = arguments[0];
					modelList = arguments[1];
					tableNames = arguments[2];
					transFields = arguments[3];
					callback = arguments[4];
				}
			}
			else if(arguments.length==6)
			{
				if(qx.lang.Type.isString(arguments[5]))
				{
					// translateModelList(envId, dataBase, modelList, tableNames, transFields, fieldSpliter);
					envId = arguments[0];
					dataBase = arguments[1];
					modelList = arguments[2];
					tableNames = arguments[3];
					transFields = arguments[4];
					fieldSpliter = arguments[5];
				}
				else
				{
					// translateModelList(envId, dataBase, modelList, tableNames, transFields, callback);
					envId = arguments[0];
					dataBase = arguments[1];
					modelList = arguments[2];
					tableNames = arguments[3];
					transFields = arguments[4];
					callback = arguments[5];
				}
			}
			else
			{
				// translateModelList(envId, dataBase, modelList, tableNames, transFields, fieldSpliter, callback);
				envId = arguments[0];
				dataBase = arguments[1];
				modelList = arguments[2];
				tableNames = arguments[3];
				transFields = arguments[4];
				fieldSpliter = arguments[5];
				callback = arguments[6];
			}
			var cloneModelList = new xlsys.java.util.ArrayList();
			if(modelList==null||modelList.isEmpty()) return cloneModelList;
			if(envId==null) envId = dataBase.getEnvId();
			if(tableNames==null) tableNames = this.self(arguments).DEFAULT_TABLENAME;
			else if(this.self(arguments).DEFAULT_TABLENAME!=tableNames&&tableNames.indexOf(this.self(arguments).DEFAULT_TABLENAME)==-1) tableNames += xlsys.base.XLSYS.KEY_CODE_SEPARATOR + this.self(arguments).DEFAULT_TABLENAME;
			var firstModel = modelList.get(0);
			if(firstModel.getRefTableName)
			{
				var tableName = firstModel.getRefTableName();
				if(tableName!=null&&tableNames.indexOf(tableName)==-1) tableNames = tableName + xlsys.base.XLSYS.KEY_CODE_SEPARATOR + tableNames;
			}
			var paramMap = new xlsys.java.util.HashMap();
			paramMap.put(xlsys.base.database.IDataBase.BUFFER_KEY_TABLE_NAME, tableNames);
			var bufferName = xlsys.base.XLSYS.BUFFER_TRANSLATION_PREFIX+this._language;
			if(callback) this.getBufferData(bufferName, paramMap, qx.Bootstrap.bind(this.__translateModelList1, this, modelList, transFields, fieldSpliter, cloneModelList, callback));
			else
			{
				var tableMap = this.getBufferData(bufferName, paramMap);
				return this.__translateModelList1(modelList, transFields, fieldSpliter, cloneModelList, null, tableMap);
			}
		},
		
		__translateModelList1 : function(modelList, transFields, fieldSpliter, cloneModelList, callback, tableMap)
		{
			if(transFields==null) transFields = "name";
			if(fieldSpliter==null) fieldSpliter = xlsys.base.XLSYS.KEY_CODE_SEPARATOR;
			for(var i=0;i<modelList.size();++i)
			{
				var model = modelList.get(i);
				var cloneModel = xlsys.base.model.ModelUtil.cloneModel(model);
				cloneModelList.add(cloneModel);
			}
			if(!tableMap.isEmpty())
			{
				var transFieldArr = transFields.split(fieldSpliter);
				for(var i=0;i<transFieldArr.length;++i)
				{
					var fieldName = transFieldArr[i];
					for(var j=0;j<cloneModelList.size();++j)
					{
						var cloneModel = cloneModelList.get(j);
						var oldValue = cloneModel[fieldName];
						if(oldValue!=null)
						{
							var newValue = tableMap.get(oldValue);
							if(newValue!=null) cloneModel[fieldName] = newValue;
						}
					}
				}
			}
			if(callback) callback(cloneModelList);
			else return cloneModelList;
		},
		
		getStorageObject : function(bufferName, mark)
		{
			var arr = new Array();
			var session = xlsys.base.session.SessionManager.getInstance().getCurrentSession();
			var envId = session.getAttribute(xlsys.base.XLSYS.SESSION_ENV_ID);
			arr.push(this.getBufferMap(envId, bufferName));
			if(mark==xlsys.base.buffer.XlsysBuffer.MARK_ALL) arr.push(this._tableCount);
			return arr;
		},
		
		isBufferComplete : function(bufferName, callback)
		{
			if(this._tableCount==null)
			{
				var dataBase = xlsys.base.database.ClientDataBase.getInstance();
				var selectSql = "select count(1) from (select distinct tablename from xlsys_translator where language=?) lxd_sub";
				var pb = new xlsys.base.database.bean.ParamBean(selectSql);
				pb.addParamGroup();
				pb.setParam(1, this._language);
				if(callback)
				{
					dataBase.sqlSelectAsOneValue(pb, qx.Bootstrap.bind(this.__isBufferComplete1, this, dataBase, bufferName, callback));
					return;
				}
				else
				{
					this._tableCount = dataBase.sqlSelectAsOneValue(pb);
					dataBase.close();
				}
			}
			var session = xlsys.base.session.SessionManager.getInstance().getCurrentSession();
			var envId = session.getAttribute(xlsys.base.XLSYS.SESSION_ENV_ID);
			var isComplete = this.getBufferMap(envId, bufferName).size()==this._tableCount;
			if(callback) callback(isComplete);
			else return isComplete;
		},

		__isBufferComplete1 : function(dataBase, bufferName, callback, tableCount)
		{
			this._tableCount = tableCount;
			dataBase.close();
			var session = xlsys.base.session.SessionManager.getInstance().getCurrentSession();
			var envId = session.getAttribute(xlsys.base.XLSYS.SESSION_ENV_ID);
			var isComplete = this.getBufferMap(envId, bufferName).size()==this._tableCount;
			callback(isComplete);
		},
		
		reloadDataDirectly : function(bufferName, paramMap, forceLoad, callback)
		{
			var tableNames = null;
			if(paramMap!=null) tableNames = paramMap.get(xlsys.base.database.IDataBase.BUFFER_KEY_TABLE_NAME);
			var session = xlsys.base.session.SessionManager.getInstance().getCurrentSession();
			var envId = session.getAttribute(xlsys.base.XLSYS.SESSION_ENV_ID);
			var dictionaryMap = this.getBufferMap(envId, bufferName);
			if(tableNames!=null)
			{
				var tableArr = tableNames.split(xlsys.base.XLSYS.KEY_CODE_SEPARATOR);
				for(var i=0;i<tableArr.length;++i)
				{
					dictionaryMap.remove(tableArr[i]);
				}
			}
			else
			{
				dictionaryMap.clear();
				this._tableCount = null;
			}
			if(forceLoad) this.getBufferData(bufferName, paramMap, callback);
		},
		
		loadDataFromStorageObject : function(bufferName, storageObj, mark)
		{
			var session = xlsys.base.session.SessionManager.getInstance().getCurrentSession();
			var envId = session.getAttribute(xlsys.base.XLSYS.SESSION_ENV_ID);
			var dictionaryMap = this.getBufferMap(envId, bufferName);
			if(mark==xlsys.base.buffer.XlsysBuffer.MARK_ALL)
			{
				dictionaryMap.clear();
				this._tableCount = storageObj[1];
			}
			dictionaryMap.putAll(storageObj[0]);
			return true;
		},
		
		doGetBufferData : function(bufferName, paramMap, callback)
		{
			var session = xlsys.base.session.SessionManager.getInstance().getCurrentSession();
			var envId = session.getAttribute(xlsys.base.XLSYS.SESSION_ENV_ID);
			var tableNames = null;
			if(paramMap!=null) tableNames = paramMap.get(xlsys.base.database.IDataBase.BUFFER_KEY_TABLE_NAME);
			var dictionaryMap = this.getBufferMap(envId, bufferName);
			var ret = null;
			if(this._tableCount==null||this._tableCount!=dictionaryMap.size())
			{
				var map = new xlsys.java.util.HashMap();
				map.put("language", this._language);
				dictionaryMap.clear();
				if(callback)
				{
					xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_GET_TRANSLATE_DATA, map, qx.Bootstrap.bind(this.__doGetBufferData1, this, dictionaryMap, tableNames, callback));
					return;
				}
				else
				{
					var dataMap = xlsys.base.io.transfer.client.ClientTransfer.defaultPost(xlsys.base.XLSYS.COMMAND_GET_TRANSLATE_DATA, map);
					if(dataMap!=null) dictionaryMap.putAll(dataMap);
					this._tableCount = dictionaryMap.size();
				}
			}
			return this.__doGetBufferData2(tableNames, dictionaryMap, null);
		},
		
		__doGetBufferData1 : function(dictionaryMap, tableNames, callback, dataMap)
		{
			if(dataMap!=null) dictionaryMap.putAll(dataMap);
			this._tableCount = dictionaryMap.size();
			this.__doGetBufferData2(tableNames, dictionaryMap, callback);
		},
		
		__doGetBufferData2 : function(tableNames, dictionaryMap, callback)
		{
			var ret = null;
			if(tableNames!=null)
			{
				var tableMap = new xlsys.java.util.HashMap();
				var tableArr = tableNames.split(xlsys.base.XLSYS.KEY_CODE_SEPARATOR);
				for(var i=0;i<tableArr.length;++i)
				{
					var tablecolumnMap = dictionaryMap.get(tableArr[i]);
					if(tablecolumnMap!=null) tableMap.putAll(tablecolumnMap);
				}
				ret = tableMap;
			}
			else ret = dictionaryMap;
			if(callback) callback(ret);
			else return ret;
		},
		
		saveDataToLocalStorage : function(bufferName, storageObject, callback)
		{
			if(callback) callback(false);
			else return false;
		},
		
		reloadDataFromLocalStorage : function(bufferName, callback)
		{
			if(callback) callback(false);
			else return false;
		},
		
		getLocalStorageVersion : function(bufferName, callback)
		{
			if(callback) callback(-1);
			else return -1;
		}
	}
});