/**
 * HashMap.
 * 注意此HashMap并不是严格使用对象计算得出的Hash值来进行判断, 如果需要严格意义上的HashMap, 请使用StrictHashMap
 */
qx.Class.define("xlsys.java.util.HashMap",
{
	extend : xlsys.java.io.Serializable,
	implement : [xlsys.java.util.Map],
	
	construct: function()
	{
		this.__keymap = new Object();
		this.__valuemap = new Object();
		this.__size = 0;
	},
	
	members :
	{
		__keymap : null,
		__valuemap : null,
		__size : null,
		
		_getHashKey : function(obj)
		{
			return xlsys.base.util.ObjectUtil._getHashKey(obj);
		},
		
		getClassName : function()
		{
			return "java.util.HashMap";
		},
		
		getSerializableMembers : function()
		{
			var arr = new Array();
			arr.push("__keymap");
			arr.push("__valuemap");
			arr.push("__size");
			return arr;
		},
		
		clear : function()
		{
			for(var fieldName in this.__keymap)
			{
				delete this.__keymap[fieldName];
			}
			for(var fieldName in this.__valuemap)
			{
				delete this.__valuemap[fieldName];
			}
			this.__size = 0;
		},

		containsKey : function(key)
		{
			var keyHash = this._getHashKey(key);
			var contains = this.__keymap[keyHash]!=null;
			return contains;
		},

		containsValue : function(value)
		{
			var contains = false;
			for(var fieldName in this.__valuemap)
			{
				if(this._getHashKey(this.__valuemap[fieldName])==this._getHashKey(value))
				{
					contains = true;
					break;
				}
			}
			return contains;
		},

		get : function(key)
		{
			var keyHash = this._getHashKey(key);
			return this.__valuemap[keyHash];
		},

		isEmpty : function()
		{
			return this.__size==0;
		},

		put : function(key, value)
		{
			if(!this.containsKey(key)) ++this.__size;
			var keyHash = this._getHashKey(key);
			this.__keymap[keyHash] = key;
			this.__valuemap[keyHash] = value;
		},

		putAll : function(m)
		{
			var keyArr = m.keys();
			for(var i=0;i<keyArr.length;++i)
			{
				this.put(keyArr[i], m.get(keyArr[i]));
			}
		},

		remove : function(key)
		{
			var keyHash = this._getHashKey(key);
			if(delete this.__keymap[keyHash])
			{
				delete this.__valuemap[keyHash];
				--this.__size;
			}
		},

		size : function()
		{
			return this.__size;
		},

		keys : function()
		{
			var tempArr = new Array();
			for(var fieldName in this.__keymap)
			{
				if(fieldName=="__hashcode"||fieldName=="$$hash") continue;
				tempArr.push(this.__keymap[fieldName]);
			}
			return tempArr;
		},

		values : function()
		{
			var tempArr = new Array();
			for(var fieldName in this.__valuemap)
			{
				if(fieldName=="__hashcode"||fieldName=="$$hash") continue;
				tempArr.push(this.__valuemap[fieldName]);
			}
			return tempArr;
		},

		keySet : function()
		{
			var set = new xlsys.java.util.HashSet();
			for(var fieldName in this.__keymap)
			{
				if(fieldName=="__hashcode"||fieldName=="$$hash") continue;
				set.add(this.__keymap[fieldName]);
			}
			return set;
		},
		
		entrySet : function()
		{
			var set = new xlsys.java.util.HashSet();
			for(var fieldName in this.__keymap)
			{
				if(fieldName=="__hashcode"||fieldName=="$$hash") continue;
				var entry = new xlsys.java.util.SimpleEntry(this.__keymap[fieldName], this.__valuemap[fieldName]);
				set.add(entry);
			}
			return set;
		}
	}
});